//package NetML;
import java.io.*;
import java.util.*;


public class Firewall {

  private String type; 
  private String filexml ; // The path of the xml file
  private String filexsl ; // The path of the xml file
  private String xsltContent; // The content of a xslt file
  private String MID; // The machine with firewall id
  private String result; // the result of the transformation

  //Patterns to match in order to modify the xslt
  private static String selectMachineIdDelimit = "<!-- ###MachineID### -->";



  //private static String prefixXsl = "/home/ivan79/cvsgiga/project/xslt/";
  private static Hashtable pathXsl;

  /**
   * Replace the firewall-id at the beginning of the xsl file
   * The xslt is the already loaded file!
   */
  public static String replaceMID ( String xslt , String MID){

  return  xslt.replaceFirst(
    selectMachineIdDelimit +".*"+ selectMachineIdDelimit ,
    selectMachineIdDelimit + "<xsl:variable name=\"machineID\">"+ MID + "</xsl:variable>" + selectMachineIdDelimit);
    }

  /**
   * Load the xsl file, change it, call trans to make the transformation!
   *
   */

  public String start () {

    try {

      //Load the file (must be initialized first!)
      this.xsltContent = utils.loadFile(this.filexsl);
      this.xsltContent = Firewall.replaceMID(this.xsltContent, this.MID);

      //Change the xslt include directives to match the real location
      //Uses this.prefixXsl
      Vector include = new Vector();
      //Cisco stylesheets
      if (this.type.startsWith("cisco")) {

	include.add("fw-cisco-pix.xsl");

      }
      // Iptables
      if (this.type.startsWith("iptables")) {

	include.add("fw-iptables.xsl");
      }
      // Ipfilter
      if (this.type.startsWith("ipfilter")) {

	include.add("ipfilter.xsl");

      }
      
      // Ipfirewall
      if (this.type.startsWith("ipfirewall")) {

	include.add("ipfirewall.xsl");

      }

   


    
	String tmp = (String) include.elementAt(0);
	this.xsltContent = this.xsltContent.replaceFirst(
                   tmp, utils.xslPath() + tmp);

      


  

      Trans t = new Trans();
      this.result = t.start(this.filexml, this.xsltContent);
      return this.result;
    }
    catch (Exception e) {
      e.printStackTrace();
      return null;
}



  }

  //Construct the application
  public Firewall(String fxml , String mId , String t) {

    //Assign each letter type a filename
    //This is done once by the first class created
    if (Firewall.pathXsl == null) {
      Firewall.pathXsl = new Hashtable(20);
      // The path for the complete configuration
      Firewall.pathXsl.put("cisco", "fw-cisco-pix.xsl");
      Firewall.pathXsl.put("ipfilter", "fw-ipfilter.xsl");
      Firewall.pathXsl.put("iptables", "fw-iptables.xsl");
      Firewall.pathXsl.put("ipfirewall", "fw-ipfirewall.xsl");
}


    this.filexml= fxml;
    this.MID = mId;
    //Check that the type passed is valid
    //   t = t.toLowerCase(); NOT ALLOWED!!!
    // To do the comparison get the enumeration of the keys and make a
    //comparison with the type t passed:
    Enumeration keys = this.pathXsl.keys();
    boolean found = false;
    while (keys.hasMoreElements() && ! found) {
      String tmp = (String) keys.nextElement();
      found = tmp.equals(t);
    }
    if (found) {
      this.type = t;
      this.filexsl= utils.xslPath() + this.pathXsl.get(t);

    } else {
      System.err.println("Type invalid!");
      System.exit(100);
    }

  }

  //Main method
  public static void main(String[] args) {

    //System.out.println( utils.pwd());
    //System.out.println( utils.xsdPath());
    //System.out.println( utils.xslPath());
    //System.out.println( args[0]);
    //System.exit(12);


    //These are the variables that are to be passed via command line
    String fXml ;
    String outputDir;
    String type;

    if (args.length==1 && (args[0].equals("--help") || args[0].equals("-h"))) {
      Firewall.usage();
      System.exit(0);
    }

    //arguments check (START)
    HashMap params = utils.parseParams(args);
    if (params == null) { //Something has gone wrong

      System.err.println("Bad arguments!");
      Firewall.usage();
      System.exit(1001);
    }

    fXml = (String) params.remove("x");

    if (fXml == null ) {
      System.err.println("You must specify a xml file");
      System.exit(1002);
    }

    outputDir = (String) params.remove("d");
    if (outputDir == null ) {
      System.err.println("You must specify an output directory");
      System.exit(1003);
    }

    type = (String) params.remove("t");
    if (type == null ) {
      System.err.println("You must specify an output type");
      System.exit(1004);
    }

    if (! params.isEmpty()) {
      System.err.println("Too many arguments!");
      Firewall.usage();
      System.exit(1005);

    }


    //arguments check (END)

    //Gets the lists of machine id and hostnames
    // String [] routers = utils.GetRouterList(fXml);
    String [] firewalls = utils.GetFirewallNames(fXml);
    
    System.out.println(firewalls.length);


 

      //for each firewall apply the transformation
      int errors = 0;  // unused yet
      for (int i = 0; i < firewalls.length; i++) {

        System.out.println();
        System.out.println("--------------------------");
        System.out.println("FirewallID: " + firewalls[i]);

        //do the transformation
        Firewall f = new Firewall(fXml, firewalls[i], type);
        String output = f.start();

        //System.out.println(output);

        //write to file
        long n = -1;
        if (output.length() > 0) {
          n = utils.writeFile(outputDir + firewalls[i]+ "." +
                              type, output);
        }
        if (n != -1) {
          System.out.println("-------DONE------");
        }
        else {
          System.out.println("-------ERROR?------");
          errors++;
        }
      }
   


    









/*
                  cat > /etc/zebra/daemons <<EOF
# This file tells the zebra package
# which daemons to start.
zebra=yes
bgpd=yes
ospfd=no
ospf6d=no
ripd=no
ripngd=no
EOF



*/






      }




 
  /**
   * Firewall.usage() prints out all the possible parameters of the application
   */
  static void usage() {
    System.out.println();
    System.out.println("Available arguments:");
    System.out.println();
    System.out.println("-x filexml");
    System.out.println("-t type ( cisco | iptables | ipfilter | ipfirewall )");
    System.out.println("-d outputDir");
    System.out.println();
    System.out.println(
        "The outputDir is intended to be a subdir of the 'output' directory. "+
        "If it doesn't exist it will be created");
    System.out.println();


  }
}
