<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
               version="1.0"
							 xmlns:netml="http://www.xmlnetwork.org"
							 exclude-result-prefixes="netml">

	<xsl:output method="text"  indent="no"/>

	<!-- ###RouterID### --><xsl:variable name="routerID">r01</xsl:variable><!-- ###RouterID### -->
	<xsl:key name="kRouter" match="netml:NetML/netml:Routers/netml:RouterConf" use="@id"/>
	<xsl:key name="kIface" match="netml:NetML/netml:Routers/netml:RouterConf[@id=string($routerID)]/netml:Interface/netml:ipAddress" use="../@name"/>
	<xsl:key name="kConfederation" match="netml:NetML/netml:AS-List/netml:Confederations/netml:C" use="netml:ASMember/@id"/>

	<xsl:key name="kList" match="netml:NetML/netml:Routers/netml:RouterConf[@id=string($routerID)]/netml:List/netml:RouteList" use="netml:id"/>

	<xsl:template match="/">
		<xsl:choose>
			<xsl:when test="count(key('kRouter',string($routerID))//netml:BGPConf/netml:view)>1">
				<xsl:text>    routing-instances {</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>        </xsl:text><xsl:value-of select="key('kRouter',string($routerID))/netml:BGPConf/netml:view"/><xsl:text> {</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>            instance-type vrf;</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>            routing-options {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:variable name="localAS" select="key('kRouter',string($routerID))/netml:BGPConf/@as"/>
				<xsl:text>                autonomous-system </xsl:text><xsl:value-of select="$localAS"/>
				<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:RouterId)>0">
					<xsl:text>                router-id </xsl:text><xsl:value-of select="key('kRouter',string($routerID))/netml:BGPConf/netml:RouterId"/>
					<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:if test="count(key('kConfederation',string($localAS)))>0">
					<xsl:for-each select="key('kConfederation',string($localAS))">
						<xsl:text>                confederation </xsl:text><xsl:value-of select="@id"/>
						<xsl:text> members [ </xsl:text>
						<xsl:for-each select="netml:ASMember">
							<xsl:value-of select="@id"/><xsl:text> </xsl:text>
						</xsl:for-each>
						<xsl:text>];</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:for-each>
				</xsl:if>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:Default-Originate)>0 or count(key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes)>0">
					<xsl:text>                static {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:Default-Originate)>0">
						<xsl:text>                     route default {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                           next-hop self; </xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                           retain;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                     }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:if>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes)>0">
						<xsl:for-each select="key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes/netml:s">
							<xsl:text>                     route </xsl:text><xsl:value-of select="@address"/>
							<xsl:text> {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							<xsl:text>                           next-hop self;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							<xsl:text>                           retain;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							<xsl:text>                     }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						</xsl:for-each>
					</xsl:if>
					<xsl:text>                }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:AggregateRoutes)>0">
					<xsl:text>                aggregate {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					<xsl:for-each select="key('kRouter',string($routerID))/netml:BGPConf/netml:AggregateRoutes/netml:route">
						<xsl:text>                    route </xsl:text><xsl:value-of select="@address"/>
						<xsl:text> {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                          next-hop self;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:if test="count(netml:advertise)>0 or count(netml:attribute)>0">
							<xsl:text>                          policy [ </xsl:text>
							<xsl:if test="count(netml:advertise)>0">
								<xsl:value-of select="netml:advertise"/><xsl:text> </xsl:text>
							</xsl:if>
							<xsl:if test="count(netml:attribute)>0">
								<xsl:value-of select="netml:attribute"/>
							</xsl:if>
							<xsl:text> ];</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						</xsl:if>
						<xsl:choose>
							<xsl:when test="@as-set='true'">
								<xsl:text>                          full;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							</xsl:when>
							<xsl:otherwise>
								<xsl:text>                          brief;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							</xsl:otherwise>
						</xsl:choose>
						<xsl:text>                          </xsl:text><xsl:value-of select="@type"/>
						<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                    </xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:for-each>
					<xsl:text>                }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:text>            }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>            protocols {</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>                bgp {</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:apply-templates select="key('kRouter',string($routerID))/netml:BGPConf"/>
				<xsl:text>                }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>            }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>        }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>    }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
			<xsl:when test="count(key('kRouter',string($routerID))//netml:BGPConf/netml:view)=1">
				<xsl:text>routing-options {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:variable name="localAS" select="key('kRouter',string($routerID))/netml:BGPConf/@as"/>
				<xsl:text>     autonomous-system </xsl:text><xsl:value-of select="$localAS"/>
				<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:RouterId)>0">
					<xsl:text>     router-id </xsl:text><xsl:value-of select="key('kRouter',string($routerID))/netml:BGPConf/netml:RouterId"/>
					<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:if test="count(key('kConfederation',string($localAS)))>0">
					<xsl:for-each select="key('kConfederation',string($localAS))">
						<xsl:text>     confederation </xsl:text><xsl:value-of select="@id"/>
						<xsl:text> members [ </xsl:text>
						<xsl:for-each select="netml:ASMember">
							<xsl:value-of select="@id"/><xsl:text> </xsl:text>
						</xsl:for-each>
						<xsl:text>];</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:for-each>
				</xsl:if>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:Default-Originate)>0 or count(key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes)>0">
					<xsl:text>     static {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:Default-Originate)>0">
						<xsl:text>         route default {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                next-hop self; </xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                retain;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>         }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:if>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes)>0">
						<xsl:for-each select="key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes/netml:s">
							<xsl:text>         route </xsl:text><xsl:value-of select="@address"/>
							<xsl:text> {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							<xsl:text>                next-hop self;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							<xsl:text>                retain;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							<xsl:text>         }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						</xsl:for-each>
					</xsl:if>
					<xsl:text>     }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:AggregateRoutes)>0">
					<xsl:text>     aggregate {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					<xsl:for-each select="key('kRouter',string($routerID))/netml:BGPConf/netml:AggregateRoutes/netml:route">
						<xsl:text>         route </xsl:text><xsl:value-of select="@address"/>
						<xsl:text> {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                next-hop self;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:if test="count(netml:advertise)>0 or count(netml:attribute)>0">
							<xsl:text>             policy [ </xsl:text>
							<xsl:if test="count(netml:advertise)>0">
								<xsl:value-of select="netml:advertise"/><xsl:text> </xsl:text>
							</xsl:if>
							<xsl:if test="count(netml:attribute)>0">
								<xsl:value-of select="netml:attribute"/>
							</xsl:if>
							<xsl:text> ];</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						</xsl:if>
						<xsl:choose>
							<xsl:when test="@as-set='true'">
								<xsl:text>             full;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							</xsl:when>
							<xsl:otherwise>
								<xsl:text>             brief;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							</xsl:otherwise>
						</xsl:choose>
						<xsl:text>             </xsl:text><xsl:value-of select="@type"/>
						<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>         }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:for-each>
					<xsl:text>    }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:text>}</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>protocols {</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>    bgp {</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:apply-templates select="key('kRouter',string($routerID))/netml:BGPConf"/>
				<xsl:text>    }</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>}</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="netml:BGPConf">
		<xsl:if test="count(netml:Description)>0">
			<xsl:text>        description </xsl:text><xsl:value-of select="netml:Description"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Authentication)>0">
			<xsl:text>        authentication-key </xsl:text><xsl:value-of select="netml:Authentication/@password"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Local-Preference)>0">
			<xsl:text>        local-preference </xsl:text><xsl:value-of select="netml:Local-Preference"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Timers/netml:holdtime)>0">
			<xsl:text>        hold-time </xsl:text><xsl:value-of select="netml:Timers/netml:holdtime"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Dampening)>0">
			<xsl:text>        damping;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:LogUpDown)>0">
			<xsl:text>        log-updown;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:BestPathAlwaysCompareMed)>0">
			<xsl:text>        path-selection always-compare-med;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Policy)>0">
			<xsl:variable name="impPols">
				<xsl:for-each select="netml:Policy">
					<xsl:if test="@type='import'">
						<xsl:value-of select="."/><xsl:text> </xsl:text>
					</xsl:if>
				</xsl:for-each>
				<xsl:if test="count(netml:Dampening)>0">
					<xsl:text>damp-BGP</xsl:text>
					<xsl:if test="count(netml:view)>0">
						<xsl:text>-</xsl:text><xsl:value-of select="netml:view"/>
					</xsl:if>
				</xsl:if>
			</xsl:variable>
			<xsl:variable name="expPols">
				<xsl:for-each select="netml:Policy">
					<xsl:if test="@type='export'">
						<xsl:value-of select="."/><xsl:text> </xsl:text>
					</xsl:if>
				</xsl:for-each>
			</xsl:variable>
			<xsl:call-template name="Policy">
				<xsl:with-param name="parent">bgp</xsl:with-param>
				<xsl:with-param name="exportList" select="$expPols"/>
				<xsl:with-param name="importList" select="$impPols"/>
			</xsl:call-template>
		</xsl:if>
		<xsl:apply-templates select="netml:PeerGroup"/>
	</xsl:template>

	<xsl:template match="netml:PeerGroup">
		<xsl:text>        group </xsl:text><xsl:value-of select="@name"/>
		<xsl:text> {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		<xsl:if test="count(netml:Description)>0">
			<xsl:text>            description </xsl:text><xsl:value-of select="netml:Description"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Peer-AS)>0">
			<xsl:text>            peer-as </xsl:text><xsl:value-of select="netml:Peer-AS"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:variable name="peerAS" select="netml:Peer-AS/@name or ../netml:Peer-AS/@name"/>
		<xsl:variable name="localAS" select="../@as"/>
		<xsl:choose>
			<xsl:when test="$peerAS=$localAS">
				<xsl:text>            type internal;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text>            type external;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:if test="count(netml:Local-AS)>0">
			<xsl:text>            local-as </xsl:text><xsl:value-of select="netml:Local-AS/@name"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Update-Source)>0">
			<xsl:variable name="ipAddr" select="key('kIface',@iface)"/>
			<xsl:text>            local-address </xsl:text><xsl:value-of select="$ipAddr"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Multihop)>0">
			<xsl:text>            multihop </xsl:text><xsl:value-of select="netml:Multihop"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Prefix-Limit)>0">
			<xsl:apply-templates select="netml:Prefix-Limit"/>
		</xsl:if>
		<xsl:if test="count(netml:Authentication)>0">
			<xsl:text>            authentication-key </xsl:text><xsl:value-of select="netml:Authentication/@password"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Remove-Private-AS)>0">
			<xsl:text>            remove-private;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Timers/netml:holdtime)>0">
			<xsl:text>            hold-time </xsl:text><xsl:value-of select="netml:Timers/netml:holdtime"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Policy)>0 or count(netml:List)>0 or count(netml:Next-Hop-Self)>0">
			<xsl:variable name="impPols">
				<xsl:for-each select="netml:Policy">
					<xsl:if test="@type='import'">
						<xsl:value-of select="."/><xsl:text> </xsl:text>
					</xsl:if>
				</xsl:for-each>
				<xsl:for-each select="netml:List">
					<xsl:if test="@inOut='import'">
						<xsl:value-of select="."/><xsl:text> </xsl:text>
					</xsl:if>
				</xsl:for-each>
			</xsl:variable>
			<xsl:variable name="expPols">
				<xsl:for-each select="netml:Policy">
					<xsl:if test="@type='export'">
						<xsl:value-of select="."/><xsl:text> </xsl:text>
					</xsl:if>
				</xsl:for-each>
				<xsl:for-each select="netml:List">
					<xsl:if test="@inOut='export'">
						<xsl:value-of select="."/><xsl:text> </xsl:text>
					</xsl:if>
				</xsl:for-each>
				<xsl:if test="count(netml:Next-Hop-Self)>0">
					<xsl:text>Next-Hop-Self</xsl:text>
				</xsl:if>
			</xsl:variable>
			<xsl:call-template name="Policy">
				<xsl:with-param name="parent">group</xsl:with-param>
				<xsl:with-param name="importList" select="$impPols"/>
				<xsl:with-param name="exportList" select="$expPols"/>
			</xsl:call-template>
		</xsl:if>
		<xsl:apply-templates select="netml:Neighbor"/>
		<xsl:text>        }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template match="netml:Neighbor">
		<xsl:text>            neighbor </xsl:text><xsl:value-of select="@address"/>
		<xsl:text> {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		<xsl:if test="count(netml:Description)>0">
			<xsl:text>                description </xsl:text><xsl:value-of select="netml:Description"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Peer-AS)>0">
			<xsl:text>                peer-as </xsl:text><xsl:value-of select="netml:Peer-AS"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Local-AS)>0">
			<xsl:text>                local-as </xsl:text><xsl:value-of select="netml:Local-AS/@name"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Update-Source)>0">
			<xsl:variable name="ipAddr" select="key('kIface',@iface)"/>
			<xsl:text>                local-address </xsl:text><xsl:value-of select="$ipAddr"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Multihop)>0">
			<xsl:text>                multihop </xsl:text><xsl:value-of select="netml:Multihop"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Prefix-Limit)>0">
			<xsl:apply-templates select="netml:Prefix-Limit"/>
		</xsl:if>
		<xsl:if test="count(netml:Authentication)>0">
			<xsl:text>                authentication-key </xsl:text><xsl:value-of select="netml:Authentication/@password"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:Timers/netml:holdtime)>0">
			<xsl:text>                hold-time </xsl:text><xsl:value-of select="netml:Timers/netml:holdtime"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(../../netml:Route-Reflector)>0">
			<xsl:apply-templates select="../../netml:Route-Reflector">
				<xsl:with-param name="neighAddr" select="@address"/>
			</xsl:apply-templates>
		</xsl:if>
		<xsl:if test="count(netml:Policy)>0 or count(netml:List)>0 or count(netml:Next-Hop-Self)>0">
			<xsl:variable name="impPols">
				<xsl:for-each select="netml:Policy">
					<xsl:if test="@type='import'">
						<xsl:value-of select="."/><xsl:text> </xsl:text>
					</xsl:if>
				</xsl:for-each>
				<xsl:for-each select="netml:List">
					<xsl:if test="@inOut='import'">
						<xsl:value-of select="."/><xsl:text> </xsl:text>
					</xsl:if>
				</xsl:for-each>
			</xsl:variable>
			<xsl:variable name="expPols">
				<xsl:for-each select="netml:Policy">
					<xsl:if test="@type='export'">
						<xsl:value-of select="."/><xsl:text> </xsl:text>
					</xsl:if>
				</xsl:for-each>
				<xsl:for-each select="netml:List">
					<xsl:if test="@inOut='export'">
						<xsl:value-of select="."/><xsl:text> </xsl:text>
					</xsl:if>
				</xsl:for-each>
				<xsl:if test="count(netml:Next-Hop-Self)>0">
					<xsl:text>Next-Hop-Self</xsl:text>
				</xsl:if>
			</xsl:variable>
			<xsl:call-template name="Policy">
				<xsl:with-param name="parent">neigh</xsl:with-param>
				<xsl:with-param name="importList" select="$impPols"/>
				<xsl:with-param name="exportList" select="$expPols"/>
			</xsl:call-template>
		</xsl:if>
		<xsl:text>            }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template name="Policy">
		<xsl:param name="parent"/>
		<xsl:param name="importList"/>
		<xsl:param name="exportList"/>
		<xsl:choose>
			<xsl:when test="$parent='bgp'">
				<xsl:text>        import [ </xsl:text><xsl:value-of select="$importList"/>
				<xsl:text> ];</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>        export [ </xsl:text><xsl:value-of select="$exportList"/>
				<xsl:text> ];</xsl:text><xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
			<xsl:when test="$parent='group'">
				<xsl:text>            import [ </xsl:text><xsl:value-of select="$importList"/>
				<xsl:text> ];</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>            export [ </xsl:text><xsl:value-of select="$exportList"/>
				<xsl:text> ];</xsl:text><xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text>                import [ </xsl:text><xsl:value-of select="$importList"/>
				<xsl:text> ];</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>                export [ </xsl:text><xsl:value-of select="$exportList"/>
				<xsl:text> ];</xsl:text><xsl:text>&#13;&#10;</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="netml:Prefix-Limit">
		<xsl:variable name="parent" select="name(parent::*)"/>
		<xsl:choose>
			<xsl:when test="$parent='PeerGroup'">
				<xsl:text>            family inet {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>                any {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>                   prefix-limit {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>                      maximum </xsl:text><xsl:value-of select="@maximum"/>
				<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:if test="count(netml:teardown)>0">
					<xsl:text>                       teardown </xsl:text><xsl:value-of select="netml:threshold"/>
					<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:text>                   }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>                }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>            }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text>                family inet {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>                   any {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>                      prefix-limit {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>                         maximum </xsl:text><xsl:value-of select="@maximum"/>
				<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:if test="count(netml:teardown)>0">
					<xsl:text>                      teardown </xsl:text><xsl:value-of select="netml:threshold"/>
					<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:text>                      }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>                   }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>                }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="netml:Route-Reflector">
		<xsl:param name="neighAddr"/>
		<xsl:for-each select="netml:Cluster">
			<xsl:for-each select="netml:Client">
				<xsl:if test="@address=$neighAddr">
					<xsl:text>                cluster </xsl:text><xsl:value-of select="../@id"/>
					<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					<xsl:if test="../@fullyMeshed='true'">
						<xsl:text>                no-client-reflect;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:if>
				</xsl:if>
			</xsl:for-each>
		</xsl:for-each>
	</xsl:template>

</xsl:stylesheet>
