<?xml version='1.0' encoding="ISO-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                           version="1.0"
                           xmlns:netml="http://www.xmlnetwork.org">

   <xsl:output method="text" />
   <xsl:strip-space elements="*"/>

   <!-- As the definitions of the templates for Policy and for List already exist
we can include their files and use those definitions -->
   <!-- ###ProtocolIncludeStart###
   <xsl:include href="policy-zebra.xsl"/>
   <xsl:include href="list-zebra.xsl"/>
   ###ProtocolIncludeEnd### -->

   <!-- ###RouterID### --><xsl:variable name="routerID">r01</xsl:variable><!-- ###RouterID### -->
   <xsl:key name="kRouter" match="netml:NetML/netml:Routers/netml:RouterConf" use="@id"/>
   <xsl:key name="kPolicy" match="netml:NetML/netml:Routers/netml:RouterConf[@id=string($routerID)]/netml:Policy" use="@name"/>
   <xsl:key name="kConfederation" match="netml:NetML/netml:AS-List/netml:Confederations/netml:C" use="netml:ASMember/@id"/>

   <xsl:template match="/">
<!-- ###Policy-List-Start###
      <xsl:if test="count (key('kRouter',string($routerID))//netml:BGPConf ) >0">
      <xsl:text> password zebra &#10;</xsl:text>
      <xsl:if test="count(key('kRouter',string($routerID))/netml:Policy)>0">
         <xsl:text>!Start of Policy Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:text>!&#13;&#10;</xsl:text>
         <xsl:apply-templates select="key('kRouter',string($routerID))/netml:Policy/netml:Redistribution//netml:term | key('kRouter',string($routerID))/netml:Policy/netml:General//netml:term"/>
         <xsl:text>!End of Policy Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:text>!&#13;&#10;</xsl:text>
      </xsl:if>
      <xsl:if test="count(key('kRouter',string($routerID))/netml:List)>0 or
         count(key('kRouter',string($routerID))//netml:metric_offset)>0 or
         count(key('kRouter',string($routerID))//netml:key_chain)>0">
         <xsl:text>!Start of Access or Prefix List Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:text>!&#13;&#10;</xsl:text>
         <xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:PrefixList)>0">
            <xsl:apply-templates select="key('kRouter',string($routerID))/netml:List/PrefixList"/>
            <xsl:text>!&#13;&#10;</xsl:text>
         </xsl:if>
         
         <xsl:if test="count(key('kRouter',string($routerID))/netml:List) >0">
            <xsl:apply-templates select="key('kRouter',string($routerID))/netml:List"/>
            <xsl:text>!&#13;&#10;</xsl:text>
            <xsl:text>!Blocco aggiunto da Ivan &#13;&#10;</xsl:text>
         </xsl:if>

         <xsl:if test="count(key('kRouter',string($routerID))//netml:metric_offset)>0">
            <xsl:text>!Creating the access lists which concern the metric offset value from RipConf or from the neighbour configurations</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
            <xsl:apply-templates select="key('kRouter',string($routerID))//netml:metric_offset"/>
            <xsl:text>!End of access lists concerning RIPCONF</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
         </xsl:if>
         <xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:RouteList)>0">
            <xsl:text>!Start of Prefix List Configuration</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
            <xsl:apply-templates select="key('kRouter',string($routerID))/netml:List/netml:RouteList"/>
            <xsl:text>!End of Prefix List Configuration</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
         </xsl:if>
         <xsl:if test="count(key('kRouter',string($routerID))//netml:key_chain)>0">
            <xsl:text>!Creating any key chains needed for authenticating</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
            <xsl:apply-templates select="key('kRouter',string($routerID))//netml:key_chain"/>
            <xsl:text>!End of key chain creation</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
         </xsl:if>
         <xsl:text>!End of Access or Prefix List Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:text>!&#13;&#10;</xsl:text>
      </xsl:if>
      </xsl:if>
###Policy-List-End### -->

      <xsl:apply-templates select="key('kRouter',string($routerID))/netml:BGPConf"/>
   </xsl:template>

   <xsl:template match="netml:BGPConf">
      <xsl:text>router bgp </xsl:text><xsl:value-of select="@as"/>
      <xsl:if test="count(netml:view)>0">
         <xsl:text> view </xsl:text><xsl:value-of select="netml:view"/>
      </xsl:if>
      <xsl:text>&#13;&#10;</xsl:text>
      <xsl:if test="count(netml:Description)>0">
         <xsl:apply-templates select="netml:Description">
            <xsl:with-param name="name">bgp</xsl:with-param>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:RouterId)>0">
         <xsl:variable name="id">
            <xsl:choose>
               <xsl:when test="substring-before(netml:RouterId,'/')!=''">
                  <xsl:value-of select="substring-before(netml:RouterId,'/')"/>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="netml:RouterId"/>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:variable>
         <xsl:text>	bgp router-id </xsl:text><xsl:value-of select="$id"/>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:if>
      <xsl:if test="count(key('kConfederation',string(@as)))>0">
         <xsl:for-each select="key('kConfederation',string(@as))">
            <xsl:text>	bgp confederation identifier </xsl:text>
            <xsl:value-of select="@id"/><xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>	bgp confederation peers </xsl:text>
            <xsl:for-each select="netml:ASMember">
               <xsl:value-of select="@id"/><xsl:text> </xsl:text>
            </xsl:for-each>
            <xsl:text>&#13;&#10;</xsl:text>
         </xsl:for-each>
      </xsl:if>
      <xsl:if test="count(netml:Local-Preference)>0">
         <xsl:text>	bgp default local-preference </xsl:text><xsl:value-of select="netml:Local-Preference"/>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:if>
      <xsl:if test="count(netml:Timers)>0">
         <xsl:apply-templates select="netml:Timers">
            <xsl:with-param  name="parent">bgp</xsl:with-param>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:Dampening)>0">
         <xsl:apply-templates select="netml:Dampening"/>
      </xsl:if>
      <xsl:if test="count(netml:LogUpDown)>0">
         <xsl:text>	log file log.txt</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:if>
      <xsl:if test="count(netml:BestPathAlwaysCompareMed)>0">
         <xsl:text>	bgp always-compare-med</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:if>
      <xsl:if test="count(netml:StaticRoutes)>0">
         <xsl:apply-templates select="netml:StaticRoutes"/>
      </xsl:if>
      <xsl:if test="count(netml:AggregateRoutes)>0">
         <xsl:apply-templates select="netml:AggregateRoutes"/>
      </xsl:if>
      <xsl:if test="count(netml:Policy)>0">
         <xsl:apply-templates select="netml:Policy">
            <xsl:with-param name="parent">bgp</xsl:with-param>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:Default-Originate)>0">
         <xsl:for-each select="netml:PeerGroup">
            <xsl:for-each select="netml:Neighbor">
               <xsl:variable name="address">
                  <xsl:choose>
                     <xsl:when test="substring-before(@address,'/')!=''">
                        <xsl:value-of select="substring-before(@address,'/')"/>
                     </xsl:when>
                     <xsl:otherwise>
                        <xsl:value-of select="@address"/>
                     </xsl:otherwise>
                  </xsl:choose>
               </xsl:variable>
               <xsl:text>	neighbor </xsl:text><xsl:value-of select="$address"/>
               <xsl:text> default-originate</xsl:text>
               <xsl:text>&#13;&#10;</xsl:text>
            </xsl:for-each>
         </xsl:for-each>
      </xsl:if>
      <xsl:if test="count(netml:Authentication)>0">
         <xsl:variable name="pwd" select="netml:Authentication/@password"/>
         <xsl:text>	! bgp password </xsl:text><xsl:value-of select="$pwd"/>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:if>
      <!-- <xsl:apply-templates select="//netml:PeerGroup"/> -->
      <xsl:apply-templates select=".//netml:PeerGroup"/>
      <xsl:if test="count(netml:Route-Reflector)>0">
         <xsl:variable name="id">
            <xsl:choose>
               <xsl:when test="substring-before(netml:Route-Reflector/netml:Cluster/@id,'/')!=''">
                  <xsl:value-of select="substring-before(netml:Route-Reflector/netml:Cluster/@id,'/')"/>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="netml:Route-Reflector/netml:Cluster/@id"/>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:variable>
         <xsl:text>	bgp cluster-id </xsl:text><xsl:value-of select="$id"/>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:if test="netml:Route-Reflector/netml:Cluster/@fullyMeshed='true'">
            <xsl:text>	bgp client-to-client reflection</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
         </xsl:if>
         <xsl:apply-templates select="netml:Route-Reflector/netml:Cluster"/><!-- per ogni neighbor che  client -->
      </xsl:if>
   </xsl:template>

   <xsl:template match="netml:Timers">
      <xsl:param name="parent"/>
      <xsl:variable name="defaultKeep">60</xsl:variable>
      <xsl:variable name="defaultHold">180</xsl:variable>
      <xsl:choose>
         <xsl:when test="$parent='bgp'">
            <xsl:text>	timers bgp </xsl:text>
            <xsl:choose>
               <xsl:when test="count(netml:keepalive)>0">
                  <xsl:value-of select="netml:keepalive"/>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="$defaultKeep"/>
               </xsl:otherwise>
            </xsl:choose>
            <xsl:text> </xsl:text>
            <xsl:choose>
               <xsl:when test="count(netml:holdtime)>0">
                  <xsl:value-of select="netml:holdtime"/>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="$defaultHold"/>
               </xsl:otherwise>
            </xsl:choose>
            <xsl:text>&#13;&#10;</xsl:text>
         </xsl:when>
         <xsl:when test="$parent='group'">
            <xsl:text>	neighbor </xsl:text><xsl:value-of select="../@name"/>
            <xsl:text> timers </xsl:text>
            <xsl:choose>
               <xsl:when test="count(netml:keepalive)>0">
                  <xsl:value-of select="netml:keepalive"/>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="$defaultKeep"/>
               </xsl:otherwise>
            </xsl:choose>
            <xsl:text> </xsl:text>
            <xsl:choose>
               <xsl:when test="count(netml:holdtime)>0">
                  <xsl:value-of select="netml:holdtime"/>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="$defaultHold"/>
               </xsl:otherwise>
            </xsl:choose>
            <xsl:text>&#13;&#10;</xsl:text>
         </xsl:when>
         <xsl:otherwise>
            <xsl:variable name="address">
               <xsl:choose>
                  <xsl:when test="substring-before(../@address,'/')!=''">
                     <xsl:value-of select="substring-before(../@address,'/')"/>
                  </xsl:when>
                  <xsl:otherwise>
                     <xsl:value-of select="../@address"/>
                  </xsl:otherwise>
               </xsl:choose>
            </xsl:variable>
            <xsl:text>	neighbor </xsl:text><xsl:value-of select="$address"/>
            <xsl:text> timers </xsl:text>
            <xsl:choose>
               <xsl:when test="count(netml:keepalive)>0">
                  <xsl:value-of select="netml:keepalive"/>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="$defaultKeep"/>
               </xsl:otherwise>
            </xsl:choose>
            <xsl:text> </xsl:text>
            <xsl:choose>
               <xsl:when test="count(netml:holdtime)>0">
                  <xsl:value-of select="netml:holdtime"/>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="$defaultHold"/>
               </xsl:otherwise>
            </xsl:choose>
            <xsl:text>&#13;&#10;</xsl:text>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <xsl:template match="netml:Dampening">
      <xsl:variable name="defaultHalf">15</xsl:variable>
      <xsl:variable name="defaultReuse">750</xsl:variable>
      <xsl:variable name="defaultSuppress">2000</xsl:variable>
      <xsl:variable name="defaultMaxSuppress">60</xsl:variable>
      <xsl:text>	bgp dampening </xsl:text>
      <xsl:choose>
         <xsl:when test="count(netml:halflife)>0">
            <xsl:value-of select="netml:halflife"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="$defaultHalf"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:text> </xsl:text>
      <xsl:choose>
         <xsl:when test="count(netml:reuse)>0">
            <xsl:value-of select="netml:reuse"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="$defaultReuse"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:text> </xsl:text>
      <xsl:choose>
         <xsl:when test="count(netml:suppress)>0">
            <xsl:value-of select="netml:suppress"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="$defaultSuppress"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:text> </xsl:text>
      <xsl:choose>
         <xsl:when test="count(netml:maxSuppressTime)>0">
            <xsl:value-of select="netml:maxSuppressTime"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="$defaultMaxSuppress"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:text>&#13;&#10;</xsl:text>
   </xsl:template>

   <xsl:template match="netml:StaticRoutes">
      <xsl:for-each select="netml:s">
         <xsl:text>	network </xsl:text><xsl:value-of select="@address"/>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:for-each>
   </xsl:template>

   <xsl:template match="netml:AggregateRoutes">
      <xsl:for-each select="netml:route">
         <xsl:variable name="address">
            <xsl:choose>
               <xsl:when test="substring-before(@address,'/')!=''">
                  <xsl:value-of select="substring-before(@address,'/')"/>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="@address"/>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:variable>
         <xsl:text>	aggregate-address </xsl:text><xsl:value-of select="$address"/>
         <xsl:if test="substring-after(@address,'/')!=''">
            <xsl:text> </xsl:text>
            <xsl:call-template name="PrefixL2Netmask">
               <xsl:with-param name="tmp1" select="substring-after(@address,'/')"/>
            </xsl:call-template>
         </xsl:if>
         <xsl:if test="@as-set='true'">
            <xsl:text> as-set</xsl:text>
         </xsl:if>
         <xsl:if test="count(netml:advertise)>0">
            <xsl:text> advertise-map </xsl:text><xsl:value-of select="netml:advertise"/>
         </xsl:if>
         <xsl:if test="count(netml:attribute)>0">
            <xsl:text> attribute-map </xsl:text><xsl:value-of select="netml:attribute"/>
         </xsl:if>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:for-each>
   </xsl:template>

   <xsl:template match="netml:PeerGroup">
      <xsl:variable name="name" select="@name"/>
      <xsl:text>	neighbor </xsl:text><xsl:value-of select="$name"/>
      <xsl:text> peer-group</xsl:text>
      <xsl:text>&#13;&#10;</xsl:text>
      <xsl:if test="count(netml:Description)>0">
         <xsl:apply-templates select="netml:Description">
            <xsl:with-param name="name" select="$name"/>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:Authentication)>0">
         <xsl:text>	! Group </xsl:text><xsl:value-of select="$name"/>
         <xsl:text> password </xsl:text><xsl:value-of select="netml:Authentication/@password"/>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:if>
      <xsl:if test="count(netml:Local-AS)>0">
         <xsl:apply-templates select="netml:Local-AS">
            <xsl:with-param name="name" select="$name"/>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:Peer-AS)>0">
         <xsl:text>	neighbor </xsl:text><xsl:value-of select="$name"/>
         <xsl:text> remote-as </xsl:text><xsl:value-of select="netml:Peer-AS/@name"/>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:if>
      <xsl:if test="count(netml:Update-Source)>0">
         <xsl:apply-templates select="netml:Update-Source">
            <xsl:with-param name="name" select="$name"/>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(../netml:DefaultOriginate)=0 and count(netml:DefaultOriginate)>0">
         <xsl:for-each select="netml:Neighbor">
            <xsl:variable name="address">
               <xsl:choose>
                  <xsl:when test="substring-before(@address,'/')!=''">
                     <xsl:value-of select="substring-before(@address,'/')"/>
                  </xsl:when>
                  <xsl:otherwise>
                     <xsl:value-of select="@address"/>
                  </xsl:otherwise>
               </xsl:choose>
            </xsl:variable>
            <xsl:text>	neighbor </xsl:text><xsl:value-of select="$address"/>
            <xsl:text> default-originate</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
         </xsl:for-each>
      </xsl:if>
      <xsl:if test="count(netml:Multihop)>0">
         <xsl:apply-templates select="netml:Multihop">
            <xsl:with-param name="name" select="$name"/>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:Prefix-Limit)>0">
         <xsl:apply-templates select="netml:Prefix-Limit">
            <xsl:with-param name="parent">group</xsl:with-param>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:Next-Hop-Self)>0">
         <xsl:apply-templates select="netml:Next-Hop-Self">
            <xsl:with-param name="name" select="$name"/>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:Remove-Private-AS)>0">
         <xsl:text>	neighbor </xsl:text><xsl:value-of select="$name"/>
         <xsl:text> remove-private-as</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:if>
      <xsl:if test="count(netml:Timers)>0">
         <xsl:apply-templates select="netml:Timers">
            <xsl:with-param name="parent">group</xsl:with-param>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:Policy)>0">
         <xsl:apply-templates select="netml:Policy">
            <xsl:with-param name="parent">group</xsl:with-param>
            <xsl:with-param name="name" select="$name"/>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:apply-templates select=".//netml:Neighbor"/>
   </xsl:template>

   <xsl:template match="netml:Cluster">
      <xsl:for-each select="netml:Client">
         <xsl:variable name="address">
            <xsl:choose>
               <xsl:when test="substring-before(@address,'/')!=''">
                  <xsl:value-of select="substring-before(@address,'/')"/>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:value-of select="@address"/>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:variable>
         <xsl:text>	neighbor </xsl:text><xsl:value-of select="$address"/>
         <xsl:text> route-reflector-client</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:for-each>
   </xsl:template>

   <xsl:template match="netml:Neighbor">
      <xsl:variable name="address">
         <xsl:choose>
            <xsl:when test="substring-before(@address,'/')!=''">
               <xsl:value-of select="substring-before(@address,'/')"/>
            </xsl:when>
            <xsl:otherwise>
               <xsl:value-of select="@address"/>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:variable>
      <xsl:variable name="remoteAS" select="netml:Peer-AS/@name|../netml:Peer-AS/@name"/>
      <xsl:text>	neighbor </xsl:text><xsl:value-of select="$address"/>
      <xsl:text> remote-as </xsl:text><xsl:value-of select="$remoteAS"/>
      <xsl:text>&#13;&#10;</xsl:text>
      <xsl:if test="count(netml:Description)>0">
         <xsl:apply-templates select="netml:Description">
            <xsl:with-param name="name" select="$address"/>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:Authentication)>0">
         <xsl:text>	! Neighbor </xsl:text><xsl:value-of select="$address"/>
         <xsl:text> password </xsl:text><xsl:value-of select="netml:Authentication/@password"/>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:if>
      <xsl:text>	neighbor </xsl:text><xsl:value-of select="$address"/>
      <xsl:text> peer-group </xsl:text><xsl:value-of select="../@name"/>
      <xsl:text>&#13;&#10;</xsl:text>
      <xsl:if test="count(../../netml:DefaultOriginate)=0 and count(../netml:DefaultOriginate)=0 and count(netml:DefaultOriginate)>0">
         <xsl:for-each select="netml:Neighbor">
            <xsl:text>	neighbor </xsl:text><xsl:value-of select="$address"/>
            <xsl:text> default-originate</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
         </xsl:for-each>
      </xsl:if>
      <xsl:if test="count(netml:Multihop)>0">
         <xsl:apply-templates select="netml:Multihop">
            <xsl:with-param name="name" select="$address"/>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:Prefix-Limit)>0">
         <xsl:apply-templates select="netml:Prefix-Limit">
            <xsl:with-param name="name" select="$address"/>
            <xsl:with-param name="parent">neigh</xsl:with-param>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:Next-Hop-Self)>0">
         <xsl:apply-templates select="netml:Next-Hop-Self">
            <xsl:with-param name="name" select="$address"/>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:Timers)>0">
         <xsl:apply-templates select="netml:Timers">
            <xsl:with-param name="parent">neighbor</xsl:with-param>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:Local-AS)>0">
         <xsl:apply-templates select="netml:Local-AS">
            <xsl:with-param name="name" select="$address"/>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:Update-Source)>0">
         <xsl:apply-templates select="netml:Update-Source">
            <xsl:with-param name="name" select="$address"/>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:if test="count(netml:Policy)>0">
         <xsl:apply-templates select="netml:Policy">
            <xsl:with-param name="parent">neighbor</xsl:with-param>
            <xsl:with-param name="name" select="$address"/>
         </xsl:apply-templates>
      </xsl:if>
   </xsl:template>

   <xsl:template match="netml:Description">
      <xsl:param name="name"/>
      <xsl:choose>
         <xsl:when test="$name=bgp">
            <xsl:text>	! </xsl:text><xsl:value-of select="."/>
            <xsl:text>&#13;&#10;</xsl:text>
         </xsl:when>
         <xsl:otherwise>
         <xsl:text>	neighbor </xsl:text><xsl:value-of select="$name"/>
         <xsl:text> description </xsl:text><xsl:value-of select="."/>
         <xsl:text>&#13;&#10;</xsl:text>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <xsl:template match="netml:Multihop">
      <xsl:param name="name"/>
      <xsl:text>	neighbor </xsl:text><xsl:value-of select="$name"/>
      <xsl:text> ebgp-multihop </xsl:text><xsl:value-of select="."/>
      <xsl:text>&#13;&#10;</xsl:text>
   </xsl:template>

   <xsl:template match="netml:Prefix-Limit">
      <xsl:param name="name"/>
      <xsl:param name="parent"/>
      <xsl:variable name="maximum" select="@maximum"/>
      <xsl:variable name="threshold" select="netml:threshold"/>
      <xsl:choose>
         <xsl:when test="$parent='group'">
            <xsl:for-each select="netml:neighbor">
               <xsl:variable name="address">
                  <xsl:choose>
                     <xsl:when test="substring-before(@address,'/')!=''">
                        <xsl:value-of select="substring-before(@address,'/')"/>
                     </xsl:when>
                     <xsl:otherwise>
                        <xsl:value-of select="@address"/>
                     </xsl:otherwise>
                  </xsl:choose>
               </xsl:variable>
               <xsl:choose>
                  <xsl:when test="count(netml:Prefix-Limit)=0">
                     <xsl:text>	neighbor </xsl:text><xsl:value-of select="$address"/><xsl:text> maximum-prefix </xsl:text>
                     <xsl:value-of select="$maximum"/>
                     <xsl:if test="count(../netml:threshold)>0">
                        <xsl:text> </xsl:text><xsl:value-of select="$threshold"/>
                     </xsl:if>
                     <xsl:if test="count(../netml:teardown)>0">
                        <xsl:text> warning-only</xsl:text>
                     </xsl:if>
                     <xsl:text>&#13;&#10;</xsl:text>
                  </xsl:when>
                  <xsl:otherwise>
                     <xsl:text>	neighbor </xsl:text><xsl:value-of select="$address"/><xsl:text> maximum-prefix </xsl:text>
                     <xsl:value-of select="@maximum"/>
                     <xsl:if test="count(netml:threshold)>0">
                        <xsl:text> </xsl:text><xsl:value-of select="netml:threshold"/>
                     </xsl:if>
                     <xsl:if test="count(netml:teardown)>0">
                        <xsl:text> warning-only</xsl:text>
                     </xsl:if>
                     <xsl:text>&#13;&#10;</xsl:text>
                  </xsl:otherwise>
               </xsl:choose>
            </xsl:for-each>
         </xsl:when>
         <xsl:otherwise>
            <xsl:if test="count(../netml:Prefix-Limit)=0">
               <xsl:text>	neighbor </xsl:text><xsl:value-of select="$name"/><xsl:text> maximum-prefix </xsl:text>
               <xsl:value-of select="@maximum"/>
               <xsl:if test="count(netml:threshold)>0">
                  <xsl:text> </xsl:text><xsl:value-of select="netml:threshold"/>
               </xsl:if>
               <xsl:if test="count(netml:teardown)>0">
                  <xsl:text> warning-only</xsl:text>
               </xsl:if>
               <xsl:text>&#13;&#10;</xsl:text>
            </xsl:if>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <xsl:template match="netml:Next-Hop-Self">
      <xsl:param name="name"/>
      <xsl:text>	neighbor </xsl:text><xsl:value-of select="$name"/><xsl:text> next-hop-self</xsl:text>
      <xsl:text>&#13;&#10;</xsl:text>
   </xsl:template>

   <xsl:template match="netml:Local-AS">
      <xsl:param name="name"/>
      <xsl:text>	neighbor </xsl:text><xsl:value-of select="$name"/><xsl:text> local-as </xsl:text><xsl:value-of select="@name"/>
      <xsl:text>&#13;&#10;</xsl:text>
   </xsl:template>

   <xsl:template match="netml:Update-Source">
      <xsl:param name="name"/>
      <xsl:text>	neighbor </xsl:text><xsl:value-of select="$name"/><xsl:text> update-source </xsl:text><xsl:value-of select="@iface"/>
      <xsl:text>&#13;&#10;</xsl:text>
   </xsl:template>

   <xsl:template match="netml:Policy">
      <xsl:param name="parent"/>
      <xsl:param name="name"/>
      <xsl:choose>
         <xsl:when test="$parent='bgp'"><!-- servono le redistribute policy e le table-map -->
            <xsl:choose>
               <xsl:when test="count(key('kPolicy',.)/netml:Redistribution)>0 and key('kPolicy',.)/netml:Redistribution/netml:to_protocol='bgp'">
                  <xsl:text>	redistribute </xsl:text><xsl:value-of select="key('kPolicy',.)/netml:Redistribution/netml:from_protocol"/>
                  <xsl:if test="count(key('kPolicy',.)/netml:Redistribution/netml:term)>0">
                     <xsl:text> route-map </xsl:text><xsl:value-of select="."/>
                  </xsl:if>
                  <xsl:text>&#13;&#10;</xsl:text>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:message>Warning: Distribution and General policies do not exist in Zebra BGP configurations. They will be ignored.</xsl:message>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:when>
         <xsl:otherwise><!-- servono le prefix-list, filter-list, route-map, unsuppress-map -->
            <xsl:choose>
               <xsl:when test="count(key('kPolicy',.)/netml:General)>0">
                  <xsl:text>	neighbor </xsl:text><xsl:value-of select="$name"/><xsl:text> route-map </xsl:text><xsl:value-of select="."/>
                  <xsl:text>&#13;&#10;</xsl:text>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:message>Warning: Redistribution or Distribution Policies cannot be called at Group or Neighbor level. These policies will be ignored.</xsl:message>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <xsl:template name="PrefixL2Netmask">
      <xsl:param name="tmp1"/>
      <xsl:variable name="tmp2" select="$tmp1 - 8"/>
      <xsl:variable name="tmp3" select="$tmp2 - 8"/>
      <xsl:variable name="tmp4" select="$tmp3 - 8"/>

      <xsl:variable name="var1">
         <xsl:choose>
            <xsl:when test="$tmp1>=0">
               <xsl:call-template name="convertNumber">
                  <xsl:with-param name="tmp" select="$tmp1"/>
               </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
               <xsl:message>Error: This is a negative prefix length!</xsl:message>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:variable>

      <xsl:variable name="var2">
         <xsl:choose>
            <xsl:when test="$tmp2>=0">
               <xsl:call-template name="convertNumber">
                  <xsl:with-param name="tmp" select="$tmp2"/>
               </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
               <xsl:call-template name="convertNumber">
                  <xsl:with-param name="tmp">0</xsl:with-param>
               </xsl:call-template>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:variable>

      <xsl:variable name="var3">
         <xsl:choose>
            <xsl:when test="$tmp3>=0">
               <xsl:call-template name="convertNumber">
                  <xsl:with-param name="tmp" select="$tmp3"/>
               </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
               <xsl:call-template name="convertNumber">
                  <xsl:with-param name="tmp">0</xsl:with-param>
               </xsl:call-template>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:variable>

      <xsl:variable name="var4">
         <xsl:choose>
            <xsl:when test="$tmp4>=0">
               <xsl:call-template name="convertNumber">
                  <xsl:with-param name="tmp" select="$tmp4"/>
               </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
               <xsl:call-template name="convertNumber">
                  <xsl:with-param name="tmp">0</xsl:with-param>
               </xsl:call-template>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:variable>

      <xsl:value-of select="$var1"/><xsl:text>.</xsl:text>
      <xsl:value-of select="$var2"/><xsl:text>.</xsl:text>
      <xsl:value-of select="$var3"/><xsl:text>.</xsl:text>
      <xsl:value-of select="$var4"/>
   </xsl:template>

   <xsl:template name="convertNumber">
      <xsl:param name="tmp"/>
      <xsl:choose>
         <xsl:when test="$tmp='0'">
            <xsl:text>0</xsl:text>
         </xsl:when>
         <xsl:when test="$tmp='1'">
            <xsl:text>128</xsl:text>
         </xsl:when>
         <xsl:when test="$tmp='2'">
            <xsl:text>192</xsl:text>
         </xsl:when>
         <xsl:when test="$tmp='3'">
            <xsl:text>224</xsl:text>
         </xsl:when>
         <xsl:when test="$tmp='4'">
            <xsl:text>240</xsl:text>
         </xsl:when>
         <xsl:when test="$tmp='5'">
            <xsl:text>248</xsl:text>
         </xsl:when>
         <xsl:when test="$tmp='6'">
            <xsl:text>252</xsl:text>
         </xsl:when>
         <xsl:when test="$tmp='7'">
            <xsl:text>254</xsl:text>
         </xsl:when>
         <xsl:when test="$tmp>='8'">
            <xsl:text>255</xsl:text>
         </xsl:when>
      </xsl:choose>
   </xsl:template>

   <xsl:template match="netml:List">
      <xsl:param name="name"/>
      <xsl:choose>
      <xsl:when test="@type='Prefix'">
            <xsl:choose>
               <xsl:when test="count(key('kList',.)/netml:RouteList)>0">
                  <xsl:text>	neighbor </xsl:text><xsl:value-of select="$name"/>
                  <xsl:text> prefix-list </xsl:text><xsl:value-of select="."/>
                  <xsl:text> </xsl:text><xsl:value-of select="@inOut"/>
                  <xsl:text>&#13;&#10;</xsl:text>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:message>Warning: NetML PrefixLists, ASPathLists and CommunityLists are not suitable for Zebra's Neighbor PrefixLists</xsl:message>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:when>
         <xsl:otherwise>
            <xsl:choose>
               <xsl:when test="count(key('kList',.)/netml:ASPathList)>0">
                  <xsl:text>	neighbor </xsl:text><xsl:value-of select="$name"/>
                  <xsl:text> filter-list </xsl:text><xsl:value-of select="."/>
                  <xsl:text> </xsl:text><xsl:value-of select="@inOut"/>
                  <xsl:text>&#13;&#10;</xsl:text>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:message>Warning: NetML PrefixLists, RouteLists and CommunityLists are not suitable for Zebra's BGP Neighbor FilterLists</xsl:message>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

</xsl:stylesheet>

