<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"  version="1.0" xmlns:netml="http://www.xmlnetwork.org">
<!--xsl:comment>Translations for IpFilter v 0.1 author Daniele Cerra</xsl:comment-->
	<xsl:output method="text"/>
	<xsl:strip-space elements="*"/>
        <!-- ###MachineID### --><xsl:variable name="machineID">temp</xsl:variable><!-- ###MachineID### -->
	<xsl:variable name="ackp"/>
	<xsl:variable name="synp"/>
        
	<xsl:template match="/">
		<xsl:choose>
			<xsl:when test="$machineID = 'temp'">
		<xsl:text>&#xA;  # Automatically Generated IPFilter Firewall Configurations &#xA;&#xA;&#xA;</xsl:text><xsl:apply-templates select="//netml:FwallConf"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:for-each select="//netml:FwallConf">
					<xsl:if test="netml:link =$machineID">
						<xsl:apply-templates select="."/>
					</xsl:if>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="netml:FwallConf">
		<!-- Process each FirewallConstruct -->
		<xsl:text>  ###############&#xA;</xsl:text>
		<xsl:text>  # </xsl:text>
		<xsl:value-of select="netml:acl/netml:name"/>
		<xsl:if test="//@id">
			<xsl:text> rules </xsl:text>
		</xsl:if>
		<xsl:for-each select="//@id">
			<xsl:value-of select="."/>
			<xsl:if test="not(position() = last())">
				<xsl:text>,</xsl:text>
			</xsl:if>
		</xsl:for-each>
		<xsl:text>&#xA;</xsl:text>
		<xsl:text>  # </xsl:text>
		<xsl:value-of select="netml:acl/netml:effect"/>
		<xsl:text>&#xA;&#xA;</xsl:text>


			<xsl:for-each select="netml:acl/netml:construct">
			<xsl:text># construct </xsl:text><xsl:value-of select="@name"/><xsl:text>: </xsl:text><xsl:value-of select="@description"/><xsl:text>&#xD;&#xA;</xsl:text>
				<xsl:for-each select="netml:rule">

			<xsl:choose>
			
				<xsl:when test="not(./netml:interface/netml:direction)">
				
					<xsl:apply-templates select=".">
						<xsl:with-param name="netml:direction">in</xsl:with-param>
					
					</xsl:apply-templates>
				
					<xsl:apply-templates select=".">
						<xsl:with-param name="netml:direction">out</xsl:with-param>
					
					</xsl:apply-templates>
				</xsl:when>
				<xsl:otherwise>
					<!-- Process this rule using the standard definition -->
					<xsl:apply-templates select="."/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
			<xsl:text>&#xA;</xsl:text>
		</xsl:for-each>
		<xsl:text>&#xA;</xsl:text>
		<xsl:text>&#xA;</xsl:text>
	</xsl:template>

	<xsl:template match="netml:rule">
		<xsl:param name="direction"/>
	
		<xsl:apply-templates select="netml:action"/>
	
		<xsl:choose>
			<!-- Override it, use $direction value instead -->
			<xsl:when test="$direction">
				<xsl:value-of select="$direction"/>
				<xsl:text> </xsl:text>
			</xsl:when>
			<!-- Either explicitly going in or out -->
			<xsl:otherwise>
				<xsl:value-of select="./netml:interface/netml:direction"/>
				<xsl:text> </xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	
		<xsl:if test="./netml:log">
			<xsl:text>log </xsl:text>
			<xsl:if test="netml:log/@level">
				<xsl:text>level auth.</xsl:text>
				<xsl:choose>
					<xsl:when test="netml:log/@level = '0'">
						<xsl:text>debug</xsl:text>
					</xsl:when>
					<xsl:when test="netml:log/@level = '1'">
						<xsl:text>info</xsl:text>
					</xsl:when>
					<xsl:when test="netml:log/@level = '2'">
						<xsl:text>notice</xsl:text>
					</xsl:when>
					<xsl:when test="netml:log/@level = '3'">
						<xsl:text>warn</xsl:text>
					</xsl:when>
					<xsl:when test="netml:log/@level = '4'">
						<xsl:text>err</xsl:text>
					</xsl:when>
					<xsl:when test="netml:log/@level = '5'">
						<xsl:text>crit</xsl:text>
					</xsl:when>
					<xsl:when test="netml:log/@level = '6'">
						<xsl:text>alert</xsl:text>
					</xsl:when>
					<xsl:when test="netml:log/@level = '7'">
						<xsl:text>emerg</xsl:text>
					</xsl:when>
				</xsl:choose>
				<xsl:text> </xsl:text>
			</xsl:if>
		</xsl:if>
	
		<xsl:text>quick </xsl:text>
	
		<xsl:if test="./netml:interface/netml:via != ''">
			<xsl:text>on </xsl:text>
			<xsl:value-of select="./netml:interface/netml:via"/>
			<xsl:text> </xsl:text>
		</xsl:if>
	
		<xsl:if test="self::node()[netml:protocol] and ./netml:protocol != 'all'">
			<xsl:value-of select="netml:protocol"/>
			<xsl:text> </xsl:text>
		</xsl:if>

		<xsl:choose>
			<xsl:when test="not(self::node()[netml:source]) and not(self::node()[netml:destination])">
				<xsl:text>all </xsl:text>
			</xsl:when>
			<xsl:when test="not(self::node()[netml:source]) and netml:destination = 'any'">
				<xsl:text>all </xsl:text>
			</xsl:when>
			<xsl:when test="netml:source = 'any' and not(self::node()[netml:destination])">
				<xsl:text>all </xsl:text>
			</xsl:when>
			<xsl:otherwise>
		
				<xsl:text>from </xsl:text>
				<xsl:choose>
					<xsl:when test="netml:source">
						<xsl:apply-templates select="netml:source"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>any </xsl:text>
						<xsl:apply-templates select="netml:options/netml:sourceport"/>
					</xsl:otherwise>
				</xsl:choose>
				<xsl:text>to </xsl:text>
				<xsl:choose>
					<xsl:when test="netml:destination">
						<xsl:apply-templates select="netml:destination"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>any </xsl:text>
						<xsl:apply-templates select="netml:options/netml:destport"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:if test="self::node()[netml:options]">
			<xsl:apply-templates select="netml:options"/>
		</xsl:if>
		<xsl:text>&#xA;</xsl:text>
	
	</xsl:template>

	<xsl:template match="netml:action">
		<xsl:if test=".='permit'">
			<xsl:text>pass</xsl:text>
		</xsl:if>
		<xsl:if test=".='deny'">
			<xsl:text>block</xsl:text>
		</xsl:if>
		<xsl:if test=".='reject'">
			<xsl:text>block return-rst</xsl:text>
		</xsl:if>
	
		<xsl:text> </xsl:text>
	</xsl:template>

	<xsl:template match="netml:protocol">
		<xsl:text>proto </xsl:text>
		<xsl:value-of select="."/>
		<xsl:text> </xsl:text>

	</xsl:template>
	<!-- The source address -->
	<xsl:template match="netml:source">
	
		<xsl:choose>
			<xsl:when test="@mask">
				<xsl:value-of select="."/>
				<xsl:text> mask </xsl:text>
				<xsl:value-of select="@mask"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="."/>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:text> </xsl:text>
	
		<xsl:if test="../netml:options/netml:sourceport">

			<xsl:for-each select="../netml:options/netml:sourceport">
				<xsl:if test="position() = 1">
					<xsl:text>port </xsl:text>
					<xsl:if test="@negate = 'true'">
						<xsl:text>!</xsl:text>
					</xsl:if>
					<xsl:text>= </xsl:text>
				</xsl:if>
				<xsl:value-of select="."/>
				<xsl:if test="position() != last()">
					<xsl:text>,</xsl:text>
				</xsl:if>
			</xsl:for-each>
			<xsl:text> </xsl:text>
		</xsl:if>
	
		<xsl:if test="../netml:options/netml:sourceports">
			
			<xsl:value-of select="../netml:options/netml:sourceports/netml:from"/>
			<xsl:text disable-output-escaping="yes">&gt;&lt;</xsl:text>
			<xsl:value-of select="../netml:options/netml:sourceports/netml:to"/>
			<xsl:text> </xsl:text>
		</xsl:if>
	</xsl:template>
	<xsl:template match="netml:sourceport">
		<xsl:for-each select=".">
			<xsl:if test="position() = 1">
				<xsl:text>port </xsl:text>
				<xsl:if test="@negate = 'true'">
					<xsl:text>!</xsl:text>
				</xsl:if>
				<xsl:text>= </xsl:text>
			</xsl:if>
			<xsl:value-of select="."/>
			<xsl:if test="position() != last()">
				<xsl:text>,</xsl:text>
			</xsl:if>
		</xsl:for-each>
		<xsl:text> </xsl:text>
		
		<xsl:if test="../netml:options/netml:sourceports">
		
			<xsl:value-of select="../netml:options/netml:sourceports/netml:from"/>
			<xsl:text disable-output-escaping="yes">&gt;&lt;</xsl:text>
			<xsl:value-of select="../netml:options/netml:sourceports/netml:to"/>
			<xsl:text> </xsl:text>
		</xsl:if>
	</xsl:template>

	<xsl:template match="netml:destination">
	
		<xsl:choose>
			<xsl:when test="@mask">
				<xsl:value-of select="."/>
				<xsl:text> mask </xsl:text>
				<xsl:value-of select="@mask"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="."/>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:text> </xsl:text>
		<!-- For sets of ports -->
		<xsl:if test="../netml:options/netml:destport">
		
			<xsl:text>port= </xsl:text>
			<xsl:value-of select="../netml:options/netml:destport"/>

			<xsl:text> </xsl:text>
		</xsl:if>
	
		<xsl:if test="../netml:options/netml:destports">
		
			<xsl:value-of select="../netml:options/netml:destports/netml:from"/>
			<xsl:text disable-output-escaping="yes">&gt;&lt;</xsl:text>
			<xsl:value-of select="../netml:options/netml:destports/netml:to"/>
			<xsl:text> </xsl:text>
		</xsl:if>
	</xsl:template>
	<xsl:template match="netml:destport">
		<xsl:for-each select=".">
			<xsl:if test="position() = 1">
				<xsl:text>port </xsl:text>
				<xsl:if test="@negate = 'true'">
					<xsl:text>!</xsl:text>
				</xsl:if>
				<xsl:text>= </xsl:text>
			</xsl:if>
			<xsl:value-of select="."/>
			<xsl:if test="position() != last()">
				<xsl:text>,</xsl:text>
			</xsl:if>
		</xsl:for-each>
		<xsl:text> </xsl:text>
		
		<xsl:if test="../netml:options/netml:sourceports">
		
			<xsl:value-of select="../netml:options/netml:sourceports/netml:from"/>
			<xsl:text disable-output-escaping="yes">&gt;&lt;</xsl:text>
			<xsl:value-of select="../netml:options/netml:sourceports/netml:to"/>
			<xsl:text> </xsl:text>
		</xsl:if>
	</xsl:template>

	<xsl:template match="netml:interface">
		<xsl:if test="self::node()[netml:via]">
			<xsl:text>via </xsl:text>
			<xsl:value-of select="netml:via"/>
			<xsl:text> </xsl:text>
		</xsl:if>
	

	</xsl:template>

	<xsl:template match="netml:options">
		<!-- Process all the TCP flag elements -->
		<xsl:if test="(count(netml:TcpFlag) &gt; 0) or (netml:state)">
			<xsl:text>flags </xsl:text>
			<xsl:variable name="synp">
				<xsl:for-each select="./netml:TcpFlag">
					<xsl:if test="(self::netml:TcpFlag = 'SYN')">true</xsl:if>
				</xsl:for-each>
			</xsl:variable>
			<xsl:variable name="ackp">
				<xsl:for-each select="./netml:TcpFlag">
					<xsl:if test=" (self::netml:TcpFlag = 'ACK')">true</xsl:if>
				</xsl:for-each>
			</xsl:variable>
			<!-- First output the list of flags that we want to be present -->
			<xsl:for-each select="./netml:TcpFlag">
				<xsl:choose>
					<xsl:when test=".='SYN'">
						<xsl:text>S</xsl:text>
					</xsl:when>
					<xsl:when test=".='FIN'">
						<xsl:text>F</xsl:text>
					</xsl:when>
					<xsl:when test=".='RST'">
						<xsl:if test="not(ancestor::options/state = 'new')">
							<xsl:text>R</xsl:text>
						</xsl:if>
					</xsl:when>
					<xsl:when test=".='PUSH'">
						<xsl:text>P</xsl:text>
					</xsl:when>
					<xsl:when test=".='ACK'">
						<xsl:if test="not(ancestor::options/state = 'new')">
							<xsl:text>A</xsl:text>
						</xsl:if>
					</xsl:when>
					<xsl:when test=".='URG'">
						<xsl:text>U</xsl:text>
					</xsl:when>
				</xsl:choose>
			</xsl:for-each>

			<xsl:if test="netml:state='established'">
				<xsl:if test="not($ackp='true')">
					<xsl:text>A</xsl:text>
				</xsl:if>
			</xsl:if>
			<xsl:if test="netml:state='new'">
				<xsl:if test="not($synp='true')">
					<xsl:text>S</xsl:text>
				</xsl:if>
			</xsl:if>
	
		<xsl:text>/</xsl:text>
	
		<xsl:for-each select="./netml:TcpFlag">
			<xsl:choose>
				<xsl:when test=".='SYN'">
					<xsl:text>S</xsl:text>
				</xsl:when>
				<xsl:when test=".='FIN'">
					<xsl:text>F</xsl:text>
				</xsl:when>
				<xsl:when test=".='RST'">
					<xsl:text>R</xsl:text>
				</xsl:when>
				<xsl:when test=".='PUSH'">
					<xsl:text>P</xsl:text>
				</xsl:when>
				<xsl:when test=".='ACK'">
					<xsl:text>A</xsl:text>
				</xsl:when>
				<xsl:when test=".='URG'">
					<xsl:text>U</xsl:text>
				</xsl:when>
			</xsl:choose>
		</xsl:for-each>
		
	
		<xsl:if test="netml:state='established'">
			<xsl:if test="not($synp='true')">
				<xsl:text>S</xsl:text>
			</xsl:if>
			<xsl:if test="not($ackp= 'true')">
				<xsl:text>A</xsl:text>
			</xsl:if>
		</xsl:if>
		<xsl:if test="netml:state='new'">
			<xsl:if test="not($synp = 'true')">
				<xsl:text>S</xsl:text>
			</xsl:if>
			<xsl:if test="not($ackp ='true')">
				<xsl:text>A</xsl:text>
			</xsl:if>
				<xsl:text> </xsl:text>
		</xsl:if>
			</xsl:if>
		<xsl:if test="netml:fragmented">
			<xsl:text>with frags </xsl:text>
		</xsl:if>

		<!-- Process all the ICMP elements -->
		<xsl:if test="count(netml:IcmpType) &gt; 0">
			<xsl:text>icmp-type </xsl:text>
			<xsl:for-each select="./netml:IcmpType">
				<!-- WHAT ABOUT MULTIPLE ICMP REQUESTS? -->
				<xsl:choose>
					<xsl:when test=".='echo-reply'">
						<xsl:text>0</xsl:text>
					</xsl:when>
					<xsl:when test=".='dst-unreach'">
						<xsl:text>3</xsl:text>
					</xsl:when>
					<xsl:when test=".='source-quench'">
						<xsl:text>4</xsl:text>
					</xsl:when>
					<xsl:when test=".='redirect'">
						<xsl:text>5</xsl:text>
					</xsl:when>
					<xsl:when test=".='echo-request'">
						<xsl:text>8</xsl:text>
					</xsl:when>
					<xsl:when test=".='router-ad'">
						<xsl:text>9</xsl:text>
					</xsl:when>
					<xsl:when test=".='router-sol'">
						<xsl:text>10</xsl:text>
					</xsl:when>
					<xsl:when test=".='ttl-exceed'">
						<xsl:text>11</xsl:text>
					</xsl:when>
					<xsl:when test=".='header-bad'">
						<xsl:text>12</xsl:text>
					</xsl:when>
					<xsl:when test=".='ts-request'">
						<xsl:text>13</xsl:text>
					</xsl:when>
					<xsl:when test=".='ts-reply'">
						<xsl:text>14</xsl:text>
					</xsl:when>
					<xsl:when test=".='info-request'">
						<xsl:text>15</xsl:text>
					</xsl:when>
					<xsl:when test=".='info-reply'">
						<xsl:text>16</xsl:text>
					</xsl:when>
					<xsl:when test=".='add-mask-request'">
						<xsl:text>17</xsl:text>
					</xsl:when>
					<xsl:when test=".='add-mask-reply'">
						<xsl:text>18</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="."/>
					</xsl:otherwise>
				</xsl:choose>
				<xsl:if test="not(position()=last())"><xsl:text>,</xsl:text></xsl:if>
			</xsl:for-each>
		</xsl:if>
		<xsl:text> </xsl:text>
		<xsl:if test="netml:TOS">
			<xsl:text>tos </xsl:text>
			<xsl:value-of select="netml:TOS"/>
			<xsl:text> </xsl:text>
		</xsl:if>
		<xsl:if test="netml:TTL">
		<xsl:text>ttl </xsl:text>
		<xsl:value-of select="netml:TTL"/>
		<xsl:text> </xsl:text>
		</xsl:if>
	
	</xsl:template>
</xsl:stylesheet><!-- Stylus Studio meta-information - (c) 2004-2005. Progress Software Corporation. All rights reserved.
<metaInformation>
<scenarios ><scenario default="no" name="Testing Area" userelativepaths="yes" externalpreview="no" url="..\..\my xsd\prova completa.xml" htmlbaseurl="" outputurl="..\..\my xsd\output_ipfilter.txt" processortype="internal" useresolver="no" profilemode="0" profiledepth="" profilelength="" urlprofilexml="" commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" validateoutput="no" validator="internal" customvalidator=""/><scenario default="yes" name="Firewall rules" userelativepaths="yes" externalpreview="no" url="Fwrules(diversi firewall).xml" htmlbaseurl="" outputurl="file:///f:/Demo/Automatically_generated_fwall_rules/Ipfilter.txt" processortype="internal" useresolver="no" profilemode="0" profiledepth="" profilelength="" urlprofilexml="" commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" validateoutput="no" validator="internal" customvalidator=""/></scenarios><MapperMetaTag><MapperInfo srcSchemaPathIsRelative="yes" srcSchemaInterpretAsXML="no" destSchemaPath="" destSchemaRoot="" destSchemaPathIsRelative="yes" destSchemaInterpretAsXML="no"/><MapperBlockPosition></MapperBlockPosition><TemplateContext></TemplateContext><MapperFilter side="source"></MapperFilter></MapperMetaTag>
</metaInformation>
-->