<?xml version='1.0'?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"  version="1.0" xmlns:netml="http://www.xmlnetwork.org">
<!--xsl:comment>Translations for IpFW v 0.1 author Daniele Cerra</xsl:comment-->
  <xsl:output method="text"/>
<xsl:strip-space elements="*"/>
<!-- ###MachineID### --><xsl:variable name="machineID">temp</xsl:variable><!-- ###MachineID### -->

<xsl:template match="/">
<xsl:choose>
<xsl:when test="$machineID = 'temp'">
    <xsl:text>&#xa;  # Automatically Generated IPFW Firewall Configurations &#xa;&#xa;</xsl:text>
      <xsl:text>  ###############&#xa;</xsl:text>

<xsl:apply-templates select="//netml:FwallConf"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:for-each select="//netml:FwallConf">
					<xsl:if test="netml:link =$machineID">
						<xsl:apply-templates select="."/>
					</xsl:if>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="netml:FwallConf">
        
      <xsl:text>  # </xsl:text><xsl:value-of select="netml:acl/netml:name"/>
	  		<xsl:if test="//@id"><xsl:text> rules </xsl:text></xsl:if>
		<xsl:for-each select="//@id">
		<xsl:value-of select="."/><xsl:if test="not(position() = last())"><xsl:text>,</xsl:text></xsl:if>
		</xsl:for-each>
	  <xsl:text>&#xa;</xsl:text>
      <xsl:text>  # </xsl:text><xsl:value-of select="netml:acl/netml:effect"/><xsl:text>&#xa;&#xa;</xsl:text>

	<xsl:for-each select="netml:acl/netml:construct">
			<xsl:text>  # construct </xsl:text><xsl:value-of select="@name"/><xsl:text>: </xsl:text><xsl:value-of select="@description"/><xsl:text>&#xD;&#xA;</xsl:text>
				<xsl:for-each select="netml:rule">


<xsl:choose>
		<xsl:when test="not(netml:protocol = 'all')">
        <xsl:apply-templates select="."/>
		</xsl:when>
		<xsl:otherwise><xsl:apply-templates select=".">
		<xsl:with-param name="proto">tcp/udp</xsl:with-param>
		</xsl:apply-templates>
		<xsl:apply-templates select=".">
		<xsl:with-param name="proto">icmp</xsl:with-param>
		</xsl:apply-templates>
		</xsl:otherwise>
		</xsl:choose>

	      </xsl:for-each>
		  <xsl:text>&#xa;</xsl:text>
		  </xsl:for-each>
  </xsl:template>

 
<xsl:template match="netml:rule">


<xsl:param name="proto"/>
        <xsl:text>ipfw add </xsl:text>
       
        <xsl:apply-templates select="netml:action"/>   
		<xsl:if test="netml:log"><xsl:text>log </xsl:text>
		<xsl:if test="netml:log/@level"><xsl:text>level </xsl:text><xsl:value-of select="log/@level"/><xsl:text> </xsl:text></xsl:if>
		</xsl:if>
		<xsl:choose>
		<xsl:when test="$proto = 'tcp/udp'">tcp/udp</xsl:when>
		<xsl:when test="$proto = 'icmp'">icmp</xsl:when>
		<xsl:otherwise><xsl:value-of select="netml:protocol"/></xsl:otherwise>
		</xsl:choose>
		<xsl:text> </xsl:text>

        <xsl:if test="self::node()[netml:source]">
          <xsl:text>from </xsl:text>
          <xsl:apply-templates select="netml:source"/>
        </xsl:if>

        <xsl:if test="self::node()[netml:destination]">
          <xsl:text>to </xsl:text>
          <xsl:apply-templates select="netml:destination"/>
        </xsl:if>

        <xsl:if test="self::node()[netml:options]">
          <xsl:apply-templates select="netml:options"/>
        </xsl:if>

        <xsl:if test="self::node()[netml:interface]">
          <xsl:apply-templates select="netml:interface"/>
        </xsl:if>
 
        <xsl:text>&#xa;</xsl:text>
      
 
</xsl:template>

 

 
  <xsl:template match="netml:action">

    <xsl:if test=".='permit'"> <xsl:text>pass</xsl:text> </xsl:if>
    <xsl:if test=".='deny'"> <xsl:text>deny</xsl:text> </xsl:if>
    <xsl:if test=".='reject'"> <xsl:text>unreach host</xsl:text> </xsl:if>
   
    <xsl:text> </xsl:text>

  </xsl:template>
 



 
  <xsl:template match="netml:source">
  

    <xsl:choose>
      <xsl:when test="@mask">
        <xsl:value-of select="."/>:<xsl:value-of select="@mask"/>
      </xsl:when>
      <xsl:when test=". ='any'"> <xsl:text>any</xsl:text> </xsl:when>
	  <xsl:otherwise><xsl:value-of select="."/></xsl:otherwise>
    </xsl:choose>

    <xsl:text> </xsl:text>
     
   
    <xsl:if test="../netml:options/netml:sourceport">
		<xsl:if test="../netml:options/netml:sourceport/@negate = 'true'"><xsl:text>not </xsl:text></xsl:if>
      <xsl:for-each select="../netml:options/netml:sourceport">
         <xsl:value-of select="."/> 
         <xsl:if test="not(position()=last())"> <xsl:text>,</xsl:text> </xsl:if>
      </xsl:for-each>
      <xsl:text> </xsl:text>
    </xsl:if>

  
    <xsl:if test="../netml:options/netml:sourceports">
	<xsl:if test="../netml:options/netml:sourceports/@negate = 'true'"><xsl:text>not </xsl:text></xsl:if>
   
      
         <xsl:value-of select="../netml:options/netml:sourceports/netml:from"/> <xsl:text>-</xsl:text> <xsl:value-of select="../netml:options/netml:sourceports/netml:to"/> 
      <xsl:text> </xsl:text>
    </xsl:if>

  </xsl:template>


 
  <xsl:template match="netml:destination">
  

  <xsl:choose>
      <xsl:when test="@mask">
        <xsl:value-of select="."/>:<xsl:value-of select="@mask"/>
      </xsl:when>
      <xsl:when test=". ='any'"> <xsl:text>any</xsl:text> </xsl:when>
	  <xsl:otherwise><xsl:value-of select="."/></xsl:otherwise>
    </xsl:choose>

    <xsl:text> </xsl:text>
     
   
    <xsl:if test="../netml:options/netml:destport">
	<xsl:if test="../netml:options/netml:destport/@negate = 'true'"><xsl:text>not </xsl:text></xsl:if>
      <xsl:for-each select="../netml:options/netml:destport">
         <xsl:value-of select="."/> 
         <xsl:if test="not(position()=last())"> <xsl:text>,</xsl:text> </xsl:if>
      </xsl:for-each>
      <xsl:text> </xsl:text>
    </xsl:if>

  
    <xsl:if test="../netml:options/netml:destports">
      <xsl:if test="../netml:options/netml:destports/@negate = 'true'"><xsl:text>not </xsl:text></xsl:if>
         <xsl:value-of select="../netml:options/netml:destports/netml:from"/> <xsl:text>-</xsl:text> <xsl:value-of select="../netml:options/netml:destports/netml:to"/> 
      <xsl:text> </xsl:text>
    </xsl:if>

  </xsl:template>


 
  <xsl:template match="netml:interface">

    <xsl:if test="netml:direction = 'in'"><xsl:text>in </xsl:text></xsl:if>
    <xsl:if test="netml:direction = 'out'"><xsl:text>out </xsl:text></xsl:if>

    <xsl:if test="netml:via"><xsl:text>via </xsl:text>
      <xsl:value-of select="netml:via"/><xsl:text> </xsl:text>
    </xsl:if>


  </xsl:template>


  
  <xsl:template match="netml:options">

   
    <xsl:if test="netml:state = 'established'"> <xsl:text>established </xsl:text> </xsl:if>
	<xsl:if test="netml:state = 'new'"><xsl:text>setup </xsl:text></xsl:if>
	<xsl:if test="netml:fragmented"><xsl:text>frag </xsl:text></xsl:if>
	<xsl:if test="netml:TOS"><xsl:text>iptos </xsl:text><xsl:value-of select="netml:TOS"/><xsl:text> </xsl:text></xsl:if>
		<xsl:if test="netml:TTL"><xsl:text>ipttl </xsl:text><xsl:value-of select="netml:TTL"/><xsl:text> </xsl:text></xsl:if>
   

    <xsl:if test="count(netml:TcpFlag) > 0"> 
      <xsl:text>tcpflags </xsl:text>
      <xsl:for-each select="./netml:TcpFlag">
     
        <xsl:choose>
          <xsl:when test=".='SYN'"> <xsl:text>syn</xsl:text> </xsl:when>
          <xsl:when test=".='FIN'"> <xsl:text>fin</xsl:text> </xsl:when>
          <xsl:when test=".='RST'"> <xsl:text>rst</xsl:text> </xsl:when>
          <xsl:when test=".='PUSH'"> <xsl:text>push</xsl:text> </xsl:when>
          <xsl:when test=".='ACK'"> <xsl:text>ack</xsl:text> </xsl:when>
          <xsl:when test=".='URG'"> <xsl:text>urg</xsl:text> </xsl:when>
        </xsl:choose>
        <xsl:if test="not(position()=last())"> <xsl:text>,</xsl:text> </xsl:if>
      </xsl:for-each>
      <xsl:text> </xsl:text>
    </xsl:if>

  
    <xsl:if test="count(netml:IcmpType) > 0"> 
      <xsl:text>icmptypes </xsl:text>
      <xsl:for-each select="./netml:IcmpType">
	  <!--xsl:value-of select="."/-->
        <xsl:choose>
          <xsl:when test=".='echo-reply'"> <xsl:text>0</xsl:text> </xsl:when>
          <xsl:when test=".='dst-unreach'"> <xsl:text>3</xsl:text> </xsl:when>
          <xsl:when test=".='source-quench'"> <xsl:text>4</xsl:text> </xsl:when>
          <xsl:when test=".='redirect'"> <xsl:text>5</xsl:text> </xsl:when>
          <xsl:when test=".='echo-request'"> <xsl:text>8</xsl:text> </xsl:when>
          <xsl:when test=".='router-ad'"> <xsl:text>9</xsl:text> </xsl:when>
          <xsl:when test=".='router-sol'"> <xsl:text>10</xsl:text> </xsl:when>
          <xsl:when test=".='ttl-exceed'"> <xsl:text>11</xsl:text> </xsl:when>
          <xsl:when test=".='header-bad'"> <xsl:text>12</xsl:text> </xsl:when>
          <xsl:when test=".='ts-request'"> <xsl:text>13</xsl:text> </xsl:when>
          <xsl:when test=".='ts-reply'"> <xsl:text>14</xsl:text> </xsl:when>
          <xsl:when test=".='info-request'"> <xsl:text>15</xsl:text> </xsl:when>
          <xsl:when test=".='info-reply'"> <xsl:text>16</xsl:text> </xsl:when>
          <xsl:when test=".='add-mask-request'"> <xsl:text>17</xsl:text> </xsl:when>
          <xsl:when test=".='add-mask-reply'"> <xsl:text>18</xsl:text> </xsl:when>
       
		<xsl:otherwise><xsl:value-of select="."/></xsl:otherwise>
        </xsl:choose>
        <xsl:if test="not(position()=last())"> <xsl:text>,</xsl:text> </xsl:if>
      </xsl:for-each>
      <xsl:text> </xsl:text>
    </xsl:if>
    

  </xsl:template>
  

</xsl:stylesheet><!-- Stylus Studio meta-information - (c) 2004-2005. Progress Software Corporation. All rights reserved.
<metaInformation>
<scenarios ><scenario default="no" name="integr" userelativepaths="yes" externalpreview="no" url="base.xml" htmlbaseurl="" outputurl="" processortype="internal" useresolver="no" profilemode="0" profiledepth="" profilelength="" urlprofilexml="" commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" validateoutput="no" validator="internal" customvalidator=""/><scenario default="yes" name="Firewall rules" userelativepaths="yes" externalpreview="no" url="Fwrules(diversi firewall).xml" htmlbaseurl="" outputurl="file:///f:/Demo/Automatically_generated_fwall_rules/Ipfirewall.txt" processortype="internal" useresolver="no" profilemode="0" profiledepth="" profilelength="" urlprofilexml="" commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" validateoutput="no" validator="internal" customvalidator=""/></scenarios><MapperMetaTag><MapperInfo srcSchemaPathIsRelative="yes" srcSchemaInterpretAsXML="no" destSchemaPath="" destSchemaRoot="" destSchemaPathIsRelative="yes" destSchemaInterpretAsXML="no"/><MapperBlockPosition></MapperBlockPosition><TemplateContext></TemplateContext><MapperFilter side="source"></MapperFilter></MapperMetaTag>
</metaInformation>
-->