<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0"
								xmlns:netml="http://www.xmlnetwork.org">

	<xsl:output method="text" />
	<xsl:strip-space elements="*"/>

	<!-- ###RouterID### --><xsl:variable name="routerID">r01</xsl:variable><!-- 
###RouterID### -->
	<xsl:key name="kRouter" match="netml:NetML/netml:Routers/netml:RouterConf" 
use="@id"/>

	<xsl:template match="/">
					<xsl:text>&#13;&#10;</xsl:text>
		<xsl:if 
test="count(key('kRouter',string($routerID))/netml:List/netml:PrefixList)>0">
			<xsl:text>//Start of Access List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:List/netml:PrefixList"/>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:if 
test="count(key('kRouter',string($routerID))/netml:RipConf/netml:static_route)>0">
				<xsl:text>//Creating the access lists which concern static routes from 
RipConf configurations</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>ip access list standard RIP_STATIC </xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:call-template name="static_route_list"/>
			</xsl:if>
			<xsl:if 
test="count(key('kRouter',string($routerID))/netml:RipConf//netml:metric_offset)>0">
				<xsl:text>//Creating the access lists which concern the metric offset value 
from RipConf or from the Rip neighbour configurations</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:apply-templates 
select="key('kRouter',string($routerID))//netml:metric_offset"/>
				<xsl:text>//End of access lists concerning RIPCONF</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:if>
			<xsl:text>//End of Access List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if 
test="count(key('kRouter',string($routerID))/netml:List/netml:RouteList)>0">
			<xsl:text>//Start of Prefix List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:List/netml:RouteList"/>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>//End of Prefix List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:variable name="countCommList" 
select="count(key('kRouter',string($routerID))/netml:List/netml:ComunityList)"/>
		<xsl:if test="$countCommList>0">
			<xsl:text>//Start of Community List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:variable name="CommListNames">
				<xsl:for-each 
select="key('kRouter',string($routerID))/netml:List/netml:ComunityList">
					<xsl:value-of select="../netml:id"/><xsl:text> ; </xsl:text>
				</xsl:for-each>
			</xsl:variable>
			<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:List/netml:CommunityList">
				<xsl:with-param name="nameList" select="$CommListNames"/>
			</xsl:apply-templates>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>//End of Community List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if 
test="count(key('kRouter',string($routerID))/netml:List/netml:ASPathList)>0">
			<xsl:text>//Start of AS Path List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:variable name="ASPathListNames">
				<xsl:for-each 
select="key('kRouter',string($routerID))/netml:List/netml:ASPathList">
					<xsl:value-of select="../netml:id"/><xsl:text> ; </xsl:text>
				</xsl:for-each>
			</xsl:variable>
			<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:List/netml:ASPathList">
				<xsl:with-param name="nameList" select="$ASPathListNames"/>
			</xsl:apply-templates>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>//End of AS Path List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if 
test="count(key('kRouter',string($routerID))/netml:RipConf//netml:keychain)>0">
			<xsl:text>//Creating any key chains needed for authenticating</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:RipConf//netml:key_chain"/>
			<xsl:variable name="count" 
select="count(key('kRouter',string($routerID))/netml:RipConf/netml:authentication/netml:key)"/>
			<xsl:if test="$count>0">
				<xsl:call-template name="key_strings">
					<xsl:with-param name="key_count" select="$count"/>
				</xsl:call-template>
			</xsl:if>
			<xsl:text>//End of key chain creation</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
	</xsl:template>

	<xsl:template match="netml:PrefixList">
		<xsl:for-each select="netml:prefix">
			<xsl:text>ip access-list standard </xsl:text><xsl:value-of 
select="../../netml:id"/>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text> permit </xsl:text><xsl:value-of 
select="substring-before(.,'/')"/><xsl:text> </xsl:text>
			<xsl:call-template name="PrefixL2WildCard">
				<xsl:with-param name="tmp1" select="substring-after(.,'/')"/>
			</xsl:call-template><xsl:text>&#13;&#10;</xsl:text>
		</xsl:for-each>
		<xsl:text>&#13;&#10;</xsl:text>
		<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>
		
	<xsl:template match="netml:RouteList">
		<xsl:for-each select="netml:routeFilter">
			<xsl:text>ip prefix-list </xsl:text><xsl:value-of 
select="../../netml:id"/><xsl:text> </xsl:text><xsl:value-of 
select="netml:target"/>
			<xsl:text> </xsl:text><xsl:value-of select="netml:prefix"/>
			<xsl:choose>
				<xsl:when test="name(netml:matchType/child::*)='OrShorter'">
					<xsl:text> le </xsl:text>
					<xsl:call-template name="FindPrefixLength">
						<xsl:with-param name="prefix" select="netml:prefix"/>
					</xsl:call-template>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:when>
				<xsl:when test="name(netml:matchType/child::*)='Exact'">
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:when>
				<xsl:when test="name(netml:matchType/child::*)='OrLonger'">
					<xsl:text> ge </xsl:text>
					<xsl:call-template name="FindPrefixLength">
						<xsl:with-param name="prefix" select="netml:prefix"/>
					</xsl:call-template>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:when>
				<xsl:when test="name(netml:matchType/child::*)='PrefixLengthRange'">
				<xsl:text> ge </xsl:text><xsl:value-of
select="netml:matchType/netml:PrefixLengthRange/@lower_bound"/>
				<xsl:text> le </xsl:text><xsl:value-of
select="netml:matchType/netml:PrefixLengthRange/@upper_bound"/>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
			<xsl:when test="name(netml:matchType/child::*)='Upto'">
				<xsl:variable name="prefixLength"><xsl:call-template 
name="FindPrefixLength">
						<xsl:with-param name="prefix" select="netml:prefix"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:choose>
					<xsl:when 
test="$prefixLength>netml:matchType/netml:Upto/netml:prefixLength">
						<xsl:text> ge </xsl:text><xsl:value-of 
select="netml:matchType/netml:Upto/netml:prefixLength"/>
						<xsl:text> le </xsl:text><xsl:value-of select="$prefixLength"/>
						<xsl:text>&#13;&#10;</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text> ge </xsl:text><xsl:value-of select="$prefixLength"/>
						<xsl:text> le </xsl:text><xsl:value-of 
select="netml:matchType/netml:Upto/netml:prefixLength"/>
						<xsl:text>&#13;&#10;</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<xsl:message>Warning (MatchType <xsl:value-of 
select="name(netml:matchType/child::*)"/>): This match type is not implemented 
in Zebra. The prefix will be seen as an exact match</xsl:message>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:for-each>
	<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>
	
	<xsl:template name="FindPrefixLength">
		<xsl:param name="prefix"/>
		<xsl:variable name="prefixLength" select="substring-after($prefix,'/')"/>
		<xsl:value-of select="$prefixLength"/>
	</xsl:template>

	<xsl:template name="static_route_list">
		<xsl:for-each 
select="key('kRouter',$routerID)/netml:RipConf/netml:static_route/netml:prefix">
			<xsl:text>  permit </xsl:text><xsl:value-of 
select="substring-before(.,'/')"/><xsl:text> </xsl:text>
			<xsl:call-template name="PrefixL2WildCard">
				<xsl:with-param name="tmp1" select="substring-after(.,'/')"/>
			</xsl:call-template>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:for-each>
		<xsl:text>&#13;&#10;</xsl:text>
		<xsl:for-each 
select="key('kRouter',$routerID)/netml:RipConf/netml:static_route/netml:prefix">
			<xsl:text>ip route </xsl:text><xsl:value-of 
select="substring-before(.,'/')"/><xsl:text> </xsl:text>
			<xsl:call-template name="PrefixL2Netmask">
				<xsl:with-param name="tmp1" select="substring-after(.,'/')"/>
			</xsl:call-template>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:for-each>
		<xsl:text>&#13;&#10;</xsl:text>
		<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template name="PrefixL2Netmask">
		<xsl:param name="tmp1"/>
		<xsl:variable name="tmp2" select="$tmp1 - 8"/>
		<xsl:variable name="tmp3" select="$tmp2 - 8"/>
		<xsl:variable name="tmp4" select="$tmp3 - 8"/>

		<xsl:variable name="var1">
			<xsl:choose>
				<xsl:when test="$tmp1>=0">
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp" select="$tmp1"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:message>Error: This is a negative prefix length!</xsl:message>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:variable name="var2">
			<xsl:choose>
				<xsl:when test="$tmp2>=0">
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp" select="$tmp2"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp">0</xsl:with-param>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:variable name="var3">
			<xsl:choose>
				<xsl:when test="$tmp3>=0">
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp" select="$tmp3"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp">0</xsl:with-param>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		
		<xsl:variable name="var4">
			<xsl:choose>
				<xsl:when test="$tmp4>=0">
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp" select="$tmp4"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp">0</xsl:with-param>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:value-of select="$var1"/><xsl:text>.</xsl:text>
		<xsl:value-of select="$var2"/><xsl:text>.</xsl:text>
		<xsl:value-of select="$var3"/><xsl:text>.</xsl:text>
		<xsl:value-of select="$var4"/>
	</xsl:template>

	<xsl:template name="PrefixL2WildCard">
		<xsl:param name="tmp1"/>
		<xsl:variable name="tmp2" select="$tmp1 - 8"/>
		<xsl:variable name="tmp3" select="$tmp2 - 8"/>
		<xsl:variable name="tmp4" select="$tmp3 - 8"/>

		<xsl:variable name="var1">
			<xsl:choose>
				<xsl:when test="$tmp1>=0">
				<xsl:variable name="varA">
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp" select="$tmp1"/>
					</xsl:call-template>
					</xsl:variable>
					<xsl:value-of select="255- $varA"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:message>Error: This is a negative prefix length!</xsl:message>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:variable name="var2">
			<xsl:choose>
				<xsl:when test="$tmp2>=0">
				<xsl:variable name="varA">
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp" select="$tmp2"/>
					</xsl:call-template>
					</xsl:variable>
					<xsl:value-of select="255- $varA"/>
				</xsl:when>
				<xsl:otherwise>
				<xsl:variable name="varA">
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp">0</xsl:with-param>
					</xsl:call-template>
					</xsl:variable>
					<xsl:value-of select="255- $varA"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:variable name="var3">
			<xsl:choose>
				<xsl:when test="$tmp3>=0">
				<xsl:variable name="varA">
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp" select="$tmp3"/>
					</xsl:call-template>
					</xsl:variable>
					<xsl:value-of select="255- $varA"/>
				</xsl:when>
				<xsl:otherwise>
				<xsl:variable name="varA">
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp">0</xsl:with-param>
					</xsl:call-template>
					</xsl:variable>
					<xsl:value-of select="255- $varA"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		
		<xsl:variable name="var4">
			<xsl:choose>
				<xsl:when test="$tmp4>=0">
				<xsl:variable name="varA">
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp" select="$tmp4"/>
					</xsl:call-template>
					</xsl:variable>
					<xsl:value-of select="255- $varA"/>
				</xsl:when>
				<xsl:otherwise>
				<xsl:variable name="varA">
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp">0</xsl:with-param>
					</xsl:call-template>
					</xsl:variable>
					<xsl:value-of select="255- $varA"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:value-of select="$var1"/><xsl:text>.</xsl:text>
		<xsl:value-of select="$var2"/><xsl:text>.</xsl:text>
		<xsl:value-of select="$var3"/><xsl:text>.</xsl:text>
		<xsl:value-of select="$var4"/>
	</xsl:template>

	<xsl:template name="convertNumber">
		<xsl:param name="tmp"/>
		<xsl:choose>
			<xsl:when test="$tmp='0'">
				<xsl:text>0</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp='1'">
				<xsl:text>128</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp='2'">
				<xsl:text>192</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp='3'">
				<xsl:text>224</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp='4'">
				<xsl:text>240</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp='5'">
				<xsl:text>248</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp='6'">
				<xsl:text>252</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp='7'">
				<xsl:text>254</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp>='8'">
				<xsl:text>255</xsl:text>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="netml:metric_offset">
		<xsl:variable name="ancest" select="name(parent::*)"/>
		<xsl:choose>
			<xsl:when test="$ancest='RipConf'">
				<!-- creation of the RIP access-list -->
				<xsl:text>ip access-list standard RIP</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:for-each select="../netml:neighbour/netml:interface">
					<xsl:variable name="iface" select="."/>
					<xsl:for-each select="/netml:addresses/netml:interface/netml:address">
						<xsl:if test="../@name=$iface">
							<xsl:text>  permit  </xsl:text><xsl:value-of 
select="substring-before(.,'/')"/><xsl:text> </xsl:text>
							<xsl:call-template name="PrefixL2WildCard">
								<xsl:with-param name="tmp1" select="substring-after(.,'/')"/>
							</xsl:call-template>
							<xsl:text>&#13;&#10;</xsl:text>
						</xsl:if>
					</xsl:for-each>
				</xsl:for-each>
				<xsl:for-each select="../netml:neighbour/netml:address">
					<xsl:text>  permit </xsl:text>
					<xsl:variable name="prefix" select="substring-before(.,'/')"/>
					<xsl:choose>
						<xsl:when test="$prefix!=''">
							<xsl:value-of select="$prefix"/><xsl:text> </xsl:text>
							<xsl:call-template name="PrefixL2WildCard">
								<xsl:with-param name="tmp1" select="substring-after(.,'/')"/>
							</xsl:call-template>
							<xsl:text>&#13;&#10;</xsl:text>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text>host </xsl:text><xsl:value-of select="."/>
							<xsl:text>&#13;&#10;</xsl:text>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
				<xsl:text>  deny any</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
			<xsl:when test="$ancest='neighbour'">
				<xsl:if test="count(../netml:interface)>0">
					<!-- creation of the interface access-list -->
					<xsl:variable name="iface" select="../netml:interface"/>
					<xsl:text>ip access-list standard N_IFACE_</xsl:text><xsl:value-of 
select="$iface"/>
					<xsl:text>&#13;&#10;</xsl:text>
					<xsl:for-each select="/netml:addresses/netml:interface/netml:address">
						<xsl:if test="../@name=$iface">
							<xsl:variable name="prefix" select="substring-before(.,'/')"/>
							<xsl:choose>
								<xsl:when test="$prefix!=''">
									<xsl:text>  permit </xsl:text><xsl:value-of 
select="$prefix"/><xsl:text> </xsl:text>
									<xsl:call-template name="PrefixL2WildCard">
											<xsl:with-param name="tmp1" select="substring-after(.,'/')"/>
									</xsl:call-template>
									<xsl:text>&#13;&#10;</xsl:text>
								</xsl:when>
								<xsl:otherwise>
									<xsl:text>host </xsl:text><xsl:value-of select="."/>
									<xsl:text>&#13;&#10;</xsl:text>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:if>
					</xsl:for-each>
					<xsl:text>  deny any</xsl:text>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:if test="count(../netml:address)>0">
					<!-- creation of the network access-list -->
					<xsl:text>ip access-list standard N_ADDR_</xsl:text><xsl:value-of 
select="../netml:address"/>
					<xsl:text>&#13;&#10;</xsl:text>
					<xsl:variable name="prefix" 
select="substring-before(../netml:address,'/')"/>
					<xsl:choose>
						<xsl:when test="$prefix!=''">
							<xsl:text>  permit </xsl:text><xsl:value-of select="$prefix"/><xsl:text> 
</xsl:text>
							<xsl:call-template name="PrefixL2WildCard">
								<xsl:with-param name="tmp1" 
select="substring-after(../netml:address,'/')"/>
							</xsl:call-template>
							<xsl:text>&#13;&#10;</xsl:text>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text>host </xsl:text><xsl:value-of select="."/>
							<xsl:text>&#13;&#10;</xsl:text>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:text>  deny any</xsl:text>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
			</xsl:when>
		</xsl:choose>
		<xsl:text>&#13;&#10;</xsl:text>
		<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template match="netml:key_chain">
		<xsl:for-each select=".">
			<xsl:text>key chain </xsl:text><xsl:value-of select="netml:name"/>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:for-each select="netml:key_list/netml:key">
				<xsl:text>  key </xsl:text><xsl:value-of select="@id"/>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>    key-string </xsl:text><xsl:value-of 
select="netml:key_string"/>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:if test="count(netml:accept_lifetime)>0">
					<xsl:text>    accept-lifetime </xsl:text>
					<xsl:value-of 
select="substring-after(netml:accept_lifetime/netml:start,'T')"/><xsl:text> 
</xsl:text>
					<xsl:value-of 
select="substring-after(substring-before(netml:accept_lifetime/netml:start,'T'),'-')"/><xsl:text>-</xsl:text>
					<xsl:value-of 
select="substring-before(substring-before(netml:accept_lifetime/netml:start,'T'),'-')"/><xsl:text> </xsl:text>
					<xsl:value-of 
select="substring-after(netml:accept_lifetime/netml:stop,'T')"/><xsl:text> 
</xsl:text>
					<xsl:value-of 
select="substring-after(substring-before(netml:accept_lifetime/netml:stop,'T'),'-')"/><xsl:text>-</xsl:text>
					<xsl:value-of 
select="substring-before(substring-before(netml:accept_lifetime/netml:stop,'T'),'-')"/>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:if test="count(netml:send_lifetime)>0">
					<xsl:text>    send-lifetime </xsl:text>
					<xsl:value-of 
select="substring-after(netml:send_lifetime/netml:start,'T')"/><xsl:text> 
</xsl:text>
					<xsl:value-of 
select="substring-after(substring-before(netml:send_lifetime/netml:start,'T'),'-')"/><xsl:text>-</xsl:text>
					<xsl:value-of 
select="substring-before(substring-before(netml:send_lifetime/netml:start,'T'),'-')"/><xsl:text> </xsl:text>
					<xsl:value-of 
select="substring-after(netml:send_lifetime/netml:stop,'T')"/><xsl:text> 
</xsl:text>
					<xsl:value-of 
select="substring-after(substring-before(netml:send_lifetime/netml:stop,'T'),'-')"/><xsl:text>-</xsl:text>
					<xsl:value-of 
select="substring-before(substring-before(netml:send_lifetime/netml:stop,'T'),'-')"/>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
			</xsl:for-each>
		</xsl:for-each>
		<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>
	
	<xsl:template name="key_strings">
		<xsl:param name="key_count"/>
		<xsl:text>key chain RipAuthenticationStrings</xsl:text>
		<xsl:text>&#13;&#10;</xsl:text>
		<xsl:choose>
			<xsl:when test="$key_count='1'">
				<xsl:text>  key 1</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>    key-string </xsl:text><xsl:value-of 
select="/netml:RouterConf/netml:RipConf/netml:authentication/netml:key"/>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="key_list">
					<xsl:for-each select="/netml:RouterConf/netml:RipConf/netml:authentication/netml:key">
						<xsl:value-of select="."/><xsl:text> ; </xsl:text>
					</xsl:for-each>
				</xsl:variable>
				<xsl:call-template name="constructKeys">
					<xsl:with-param name="key_orig" select="$key_count"/>
					<xsl:with-param name="key_now" select="$key_count"/>
					<xsl:with-param name="key_list" select="$key_list"/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="constructKeys">
		<xsl:param name="key_orig"/>
		<xsl:param name="key_now"/>
		<xsl:param name="key_list"/>
		
		<xsl:variable name="key" select="substring-before($key_list,' ; ')"/>
		<xsl:variable name="rest" select="substring-after($key_list,' ; ')"/>
		<xsl:if test="$key_orig - $key_now >=0 and $key!=''">
			<xsl:text>  key </xsl:text><xsl:value-of select="$key_orig - $key_now + 1"/>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>    key string </xsl:text><xsl:value-of select="$key"/>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="$rest!=''">
			<xsl:call-template name="constructKeys">
					<xsl:with-param name="key_orig" select="$key_orig"/>
					<xsl:with-param name="key_now" select="$key_now -1"/>
					<xsl:with-param name="key_list" select="$rest"/>
				</xsl:call-template>
		</xsl:if>
	</xsl:template>

	<xsl:template name="indexSearch">
		<xsl:param name="nameList"/>
		<xsl:param name="index"/>
		<xsl:param name="listName"/>

		<xsl:variable name="first" select="substring-before($nameList,' ; ')"/>
		<xsl:variable name="rest" select="substring-after($nameList,' ; ')"/>

		<xsl:choose>
			<xsl:when test="$first!=''">
				<xsl:choose>
					<xsl:when test="$first=$listName">
						<xsl:value-of select="$index"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:if test="$rest!=''">
							<xsl:call-template name="indexSearch">
								<xsl:with-param name="nameList" select="$rest"/>
								<xsl:with-param name="index" select="$index+1"/>
								<xsl:with-param name="listName" select="$listName"/>
							</xsl:call-template>
						</xsl:if>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<xsl:message>ERROR: The List $listName does not seem to exist</xsl:message>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="netml:CommunityList">
		<xsl:param name="nameList"/>

		<xsl:variable name="index">
			<xsl:call-template name="indexSearch">
				<xsl:with-param name="nameList" select="$nameList"/>
				<xsl:with-param name="index" select="0"/>
				<xsl:with-param name="listName" select="../netml:id"/>
			</xsl:call-template>
		</xsl:variable>

		<xsl:text>ip bgp-community new-format</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		<xsl:text>ip extcommunity-list </xsl:text><xsl:value-of select="$index+100"/><xsl:text> </xsl:text>
		<xsl:value-of select="../netml:id"/><xsl:text> permit</xsl:text>
		<xsl:for-each select="netml:Community">
			<xsl:text> </xsl:text><xsl:value-of select="@asNumber"/>
			<xsl:text>:</xsl:text><xsl:value-of select="@CommunityValue"/>
		</xsl:for-each>
		<xsl:text>&#13;&#10;</xsl:text>
		<xsl:text>ip extcommunity-list </xsl:text><xsl:value-of select="$index"/><xsl:text> </xsl:text>
		<xsl:value-of select="../../netml:id"/><xsl:text> deny all </xsl:text><xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template match="netml:ASPathList">
		<xsl:param name="nameList"/>

		<xsl:variable name="index">
			<xsl:call-template name="indexSearch">
				<xsl:with-param name="nameList" select="$nameList"/>
				<xsl:with-param name="index" select="1"/>
				<xsl:with-param name="listName" select="../netml:id"/>
			</xsl:call-template>
		</xsl:variable>

		<xsl:for-each select="netml:RegExpr">
			<xsl:text>ip as-path access-list </xsl:text><xsl:value-of select="$index"/>
			<xsl:text> permit </xsl:text><xsl:value-of select="."/>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:for-each>
		<xsl:text>ip as-path access-list </xsl:text><xsl:value-of select="$index"/>
		<xsl:text> deny all</xsl:text>
		<xsl:text>&#13;&#10;</xsl:text>
		<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>

</xsl:stylesheet>
