<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0"
								xmlns:netml="http://www.xmlnetwork.org">

	<xsl:output method="text" />
	<xsl:strip-space elements="*"/>

	<!-- ###RouterID### --><xsl:variable name="routerID">r01</xsl:variable><!-- ###RouterID### -->
	<xsl:key name="kRouter" match="netml:NetML/netml:Routers/netml:RouterConf" use="@id"/>

	<xsl:template match="/">
					<xsl:text>&#13;&#10;</xsl:text>
		<xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:PrefixList)>0">
			<xsl:text>!Start of Access List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:apply-templates select="key('kRouter',string($routerID))/netml:List/netml:PrefixList"/>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:if test="count(key('kRouter',string($routerID))/netml:RipConf//netml:metric_offset)>0">
				<xsl:text>!Creating the access lists which concern the metric offset value from RipConf or from the Rip neighbour configurations</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
				<xsl:apply-templates select="key('kRouter',string($routerID))//netml:metric_offset"/>
				<xsl:text>!End of access lists concerning RIPCONF</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
			</xsl:if>
			<xsl:text>!End of Access List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:RouteList)>0">
			<xsl:text>!Start of Prefix List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:apply-templates select="key('kRouter',string($routerID))/netml:List/netml:RouteList"/>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:text>!End of Prefix List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:ComunityList)>0">
			<xsl:text>!Start of Community List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:apply-templates select="key('kRouter',string($routerID))/netml:List/netml:CommunityList"/>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:text>!End of Community List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:ASPathList)>0">
			<xsl:text>!Start of AS Path List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:apply-templates select="key('kRouter',string($routerID))/netml:List/netml:ASPathList"/>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:text>!End of AS Path List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(key('kRouter',string($routerID))/netml:RipConf//netml:keychain)>0">
			<xsl:text>!Creating any key chains needed for authenticating</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:apply-templates select="key('kRouter',string($routerID))/netml:RipConf//netml:key_chain"/>
			<xsl:text>!End of key chain creation</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
		</xsl:if>
	</xsl:template>

	<xsl:template match="netml:PrefixList">
		<xsl:for-each select="netml:prefix">
			<xsl:text>access-list </xsl:text><xsl:value-of select="../../netml:id"/><xsl:text> permit </xsl:text>
			<xsl:value-of select="."/><xsl:text>&#13;&#10;</xsl:text>
		</xsl:for-each>
		<xsl:text>!&#13;&#10;</xsl:text>
		<xsl:text>!&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template match="netml:RouteList">
		<xsl:for-each select="netml:routeFilter">
			<xsl:text>ip prefix-list </xsl:text><xsl:value-of select="../../netml:id"/><xsl:text> </xsl:text>
         <xsl:value-of select="netml:target"/>
			<xsl:text> </xsl:text><xsl:value-of select="netml:prefix"/>
			<xsl:choose>
				<xsl:when test="name(netml:matchType/child::*)='OrShorter'">
					<xsl:text> le </xsl:text>
					<xsl:call-template name="FindPrefixLength">
						<xsl:with-param name="prefix" select="netml:prefix"/>
					</xsl:call-template>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:when>
				<xsl:when test="name(netml:matchType/child::*)='Exact'">
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:when>
				<xsl:when test="name(netml:matchType/child::*)='OrLonger'">
					<xsl:text> ge </xsl:text>
					<xsl:call-template name="FindPrefixLength">
						<xsl:with-param name="prefix" select="netml:prefix"/>
					</xsl:call-template>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:when>
				<xsl:when test="name(netml:matchType/child::*)='PrefixLengthRange'">
					<xsl:text> le </xsl:text><xsl:value-of select="netml:matchType/netml:PrefixLengthRange/@lower_bound"/>
					<xsl:text> ge </xsl:text><xsl:value-of select="netml:matchType/netml:PrefixLengthRange/@upper_bound"/>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:when>
				<xsl:when test="name(netml:matchType/child::*)='Upto'">
					<xsl:variable name="prefixLength"><xsl:call-template name="FindPrefixLength">
							<xsl:with-param name="prefix" select="netml:prefix"/>
						</xsl:call-template>
					</xsl:variable>
					<xsl:choose>
						<xsl:when test="$prefixLength>netml:matchType/netml:Upto/netml:prefixLength">
							<xsl:text> ge </xsl:text><xsl:value-of select="netml:matchType/netml:Upto/netml:prefixLength"/>
							<xsl:text> le </xsl:text><xsl:value-of select="$prefixLength"/>
							<xsl:text>&#13;&#10;</xsl:text>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text> ge </xsl:text><xsl:value-of select="$prefixLength"/>
							<xsl:text> le </xsl:text><xsl:value-of select="netml:matchType/netml:Upto/netml:prefixLength"/>
							<xsl:text>&#13;&#10;</xsl:text>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:message>Warning (MatchType <xsl:value-of select="name(netml:matchType/child::*)"/>): </xsl:message>
               <xsl:message>This match type is not implemented in Zebra. The prefix will be seen as an exact match</xsl:message>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
		<xsl:text>!&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template name="FindPrefixLength">
		<xsl:param name="prefix"/>
			<xsl:variable name="prefixLength" select="substring-after($prefix,'/')"/>
			<xsl:value-of select="$prefixLength"/>
	</xsl:template>

	<xsl:template match="netml:metric_offset">
	<xsl:variable name="ancest" select="name(parent::*)"/>
		<xsl:choose>
			<xsl:when test="$ancest='RipConf'">
				<!-- creation of the RIP access-list -->
				<xsl:for-each select="../netml:neighbour/netml:interface">
					<xsl:variable name="iface" select="."/>
					<!--xsl:for-each select="/netml:addresses/netml:interface/netml:address">
						<xsl:if test="../@name=$iface"-->
					<xsl:for-each select="key('kIfaceAddr',string($iface))">
						<xsl:text>access-list RIP permit  </xsl:text><xsl:value-of select="."/>
						<xsl:text>&#13;&#10;</xsl:text>
						<!--/xsl:if-->
					</xsl:for-each>
				</xsl:for-each>
				<xsl:for-each select="../netml:neighbour/netml:address">
					<xsl:text>access-list RIP permit </xsl:text><xsl:value-of select="."/>
				<xsl:text>&#13;&#10;</xsl:text>
				</xsl:for-each>
				<xsl:text>access-list RIP deny any</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
			<xsl:when test="$ancest='neighbour'">
				<xsl:if test="count(../netml:interface)>0">
					<!-- creation of the interface access-list -->
					<xsl:variable name="iface" select="../netml:interface"/>
					<!--xsl:for-each select="/netml:addresses/netml:interface/netml:address">
						<xsl:if test="../@name=$iface"-->
					<xsl:for-each select="key('kIfaceAddr',string($iface))">
				 		<xsl:text>access-list N_IFACE_</xsl:text><xsl:value-of select="$iface"/><xsl:text> permit </xsl:text><xsl:value-of select="."/>
						<xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>access-list N_IFACE_</xsl:text><xsl:value-of select="$iface"/><xsl:text> deny any</xsl:text>
						<xsl:text>&#13;&#10;</xsl:text>
						<!--/xsl:if-->
					</xsl:for-each>
				</xsl:if>
				<xsl:if test="count(../netml:address)>0">
					<!-- creation of the network access-list -->
					<xsl:text>access-list N_ADDR_</xsl:text><xsl:value-of select="../netml:address"/><xsl:text> permit </xsl:text>
               <xsl:value-of select="../netml:address"/>
					<xsl:text>&#13;&#10;</xsl:text>
					<xsl:text>access-list N_ADDR_</xsl:text><xsl:value-of select="../netml:address"/><xsl:text> deny any</xsl:text>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
			</xsl:when>
		</xsl:choose>
		<xsl:text>!&#13;&#10;</xsl:text>
		<xsl:text>!&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template match="netml:key_chain">
		<xsl:for-each select=".">
			<xsl:text>key chain </xsl:text><xsl:value-of select="netml:name"/>
			<xsl:text>&#13;&#10;</xsl:text>
				<xsl:for-each select="netml:key_list/netml:key">
				<xsl:text>  key </xsl:text><xsl:value-of select="@id"/>
				<xsl:text>&#13;&#10;</xsl:text>
					<xsl:text>    key-string </xsl:text><xsl:value-of select="netml:key_string"/>
					<xsl:text>&#13;&#10;</xsl:text>
					<xsl:if test="count(netml:accept_lifetime)>0">
					<xsl:text>    accept-lifetime </xsl:text><xsl:value-of select="netml:accept_lifetime/netml:start"/><xsl:text> </xsl:text>
               <xsl:value-of select="netml:accept_lifetime/netml:stop"/>
					<xsl:text>&#13;&#10;</xsl:text>
					</xsl:if>
					<xsl:if test="count(netml:send_lifetime)>0">
					<xsl:text>    send-lifetime </xsl:text><xsl:value-of select="netml:send_lifetime/netml:start"/><xsl:text> </xsl:text>
               <xsl:value-of select="netml:send_lifetime/netml:stop"/>
					<xsl:text>&#13;&#10;</xsl:text>
					</xsl:if>
				</xsl:for-each>
			</xsl:for-each>
			<xsl:text>!&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template match="netml:CommunityList">
		<xsl:for-each select="netml:Community">
			<xsl:text>ip community-list </xsl:text><xsl:value-of select="../netml:id"/>
			<xsl:text> permit </xsl:text><xsl:value-of select="@asNumber"/>
			<xsl:text>:</xsl:text><xsl:value-of select="@CommunityValue"/>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:for-each>
		<xsl:text>ip community-list </xsl:text><xsl:value-of select="../netml:id"/>
		<xsl:text> deny all </xsl:text><xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template match="netml:ASPathList">
		<xsl:text>ip as-path access-list </xsl:text><xsl:value-of select="../netml:id"/>
		<xsl:text> permit </xsl:text>
		<xsl:for-each select="netml:RegExpr">
			<xsl:value-of select="."/><xsl:text> </xsl:text>
		</xsl:for-each>
		<xsl:text>&#13;&#10;</xsl:text>
		<xsl:text>ip as-path access-list </xsl:text><xsl:value-of select="../netml:id"/>
		<xsl:text> deny all</xsl:text>
		<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>

</xsl:stylesheet>
