<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0"
								xmlns:netml="http://www.xmlnetwork.org">

	<xsl:output method="text" />
	<xsl:strip-space elements="*"/>

	<!-- ###RouterID### --><xsl:variable name="routerID">r01</xsl:variable><!-- ###RouterID### -->
	<xsl:key name="kRouter" match="netml:NetML/netml:Routers/netml:RouterConf" use="@id"/>
	<xsl:key name="kCommList" match="netml:NetML/netml:Routers/netml:RouterConf[@id=string($routerID)]/netml:List/netml:CommunityList" use="../netml:id"/>
	
<!-- TOP LEVEL -->
	<xsl:template match="/">
		<xsl:apply-templates select="key('kRouter',string($routerID))/netml:Policy"/>
		<xsl:if test="count(key('kRouter',string($routerID))/netml:RipConf/netml:static_route)>0">
			<xsl:call-template name="static_route"/>
		</xsl:if>
	</xsl:template>

<!-- FIRST SUB-LEVEL -->
	<xsl:template match="netml:Policy/netml:Redistribution">
		<xsl:apply-templates select=".//netml:term"/>
	</xsl:template>

	<xsl:template match="netml:Policy/netml:General">
		<xsl:apply-templates select=".//netml:term"/>
	</xsl:template>
	
	<xsl:template match="netml:Policy/netml:Distribution">
	</xsl:template>

<!-- SECOND SUB-LEVEL -->
	<xsl:template match="netml:Policy/netml:Redistribution//netml:term | netml:Policy/netml:General//netml:term">
		<xsl:variable name="pname" select="../../@name"/>
		<xsl:text>&#13;&#10;</xsl:text>
		<xsl:text>route-map </xsl:text>
		<xsl:value-of select="$pname"/>
		<xsl:text> </xsl:text>
		<xsl:value-of select="@target"/>
		<xsl:text> </xsl:text>
		<xsl:value-of select="@priority"/>
		<xsl:text>&#13;&#10;</xsl:text>
		<xsl:if test="count(netml:matchConditions)>0">
			<xsl:apply-templates select="netml:matchConditions"/>
		</xsl:if>
		<xsl:if test="count(netml:setRules)>0">
			<xsl:apply-templates select="netml:setRules"/>
		</xsl:if>
		<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>


	<xsl:template match="netml:matchConditions">
		<xsl:variable name="family" select="netml:family"/>
		<xsl:variable name="protocol" select="netml:protocol"/>
		<xsl:variable name="origin" select="netml:origin"/>
		<xsl:variable name="level" select="netml:level"/>
		<xsl:variable name="tag" select="netml:tag"/>
		<xsl:variable name="preference" select="netml:preference"/>
		<xsl:variable name="colour" select="netml:colour"/>
		<xsl:variable name="local_preference" select="netml:local_preference"/>
		<!-- <xsl:variable name="metric" select="netml:metric"/> -->
		<!-- <xsl:variable name="external_route" select="netml:external_route"/> -->
		<!-- <xsl:variable name="packet_length_min" select="netml:packet_length/@min"/> -->
		<!-- <xsl:variable name="packet_length_max" select="netml:packet_length/@max"/> -->
		<!--<xsl:variable name="route_type" select="netml:route_type"/> -->
		<xsl:variable name="area" select="netml:area"/>
		<xsl:variable name="AS_Path_List" select="netml:AS_Path_List"/>
		<xsl:variable name="Community_List" select="netml:Community_List"/>
		<!-- <xsl:variable name="Interface" select="netml:Interface"/> -->
		<!-- <xsl:variable name="List" select="netml:List"/> -->

		<xsl:if test="count(netml:Interface) > 0">
			<xsl:text> match interface </xsl:text>
			<xsl:for-each select="netml:Interface">
				<xsl:value-of select="."/>
				<xsl:text> </xsl:text>
			</xsl:for-each>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>

		<xsl:if test=" count(netml:List) >0 ">
			<xsl:for-each select="netml:List">
				<xsl:variable name="match_type" select="@match"/>
				<xsl:choose>
					<xsl:when test=" $match_type = 'next_hop' ">
						<xsl:text> match ip next-hop </xsl:text>
					</xsl:when>
					<xsl:when test=" $match_type = 'neighbour' ">
						<xsl:text> match ip route-source </xsl:text>
					</xsl:when>
					<xsl:when test=" $match_type = 'IP_address' ">
						<xsl:text> match ip address </xsl:text>
					</xsl:when>
					<xsl:otherwise>ERROR</xsl:otherwise>
				</xsl:choose>
				<xsl:value-of select="." />
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:for-each>
		</xsl:if>

		<xsl:if test="count(netml:metric) > 0">
			<xsl:text> match metric </xsl:text>
			<xsl:value-of select="netml:metric"/>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>

		<xsl:if test="count(netml:packet_length) > 0">
			<xsl:text> match length </xsl:text>
			<xsl:value-of select="netml:packet_length/@min"/>
			<xsl:text> </xsl:text>
			<xsl:value-of select="netml:packet_length/@max"/>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>

		<xsl:if test="count(netml:route_type) > 0">
			<xsl:text> match route-type </xsl:text>
			<xsl:choose>
				<xsl:when test="netml:route_type = 'local' or netml:route_type = 'internal' ">
					<xsl:value-of select="netml:route_type"/>
				</xsl:when>
				<xsl:when test="netml:route_type = 'external' ">
					<xsl:text>external </xsl:text>
				</xsl:when>
				<xsl:when test="netml:route_type = 'external_type1' ">
					<xsl:text>external type-1</xsl:text>
				</xsl:when>
				<xsl:when test="netml:route_type = 'external_type2' ">
					<xsl:text>external type-2</xsl:text>
				</xsl:when>
				<xsl:when test="netml:route_type = 'level1' ">
					<xsl:text>external level-1</xsl:text>
				</xsl:when>
				<xsl:when test="netml:route_type = 'level2' ">
					<xsl:text>external level-2</xsl:text>
				</xsl:when>
			</xsl:choose>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<!-- Now calls the templates -->
		<!-- Indipendent -->

		<!-- Bgp -->
		<xsl:apply-templates select="netml:AS_Path_List"/>
		<xsl:apply-templates select="netml:Community_List"/>

	</xsl:template>


		<!-- Starting BGP templates-->

	<xsl:template match="netml:AS_Path_List">
		<xsl:text> match as-path </xsl:text>
		<xsl:value-of select="."/>
		<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template match="netml:Community_List">
			<xsl:variable name="CommListNames">
				<xsl:for-each select="key('kRouter',string($routerID))/netml:List/netml:ComunityList">
					<xsl:value-of select="../netml:id"/><xsl:text> ; </xsl:text>
				</xsl:for-each>
			</xsl:variable>
		<xsl:variable name="index">
			<xsl:call-template name="indexSearch">
				<xsl:with-param name="nameList" select="$CommListNames"/>
				<xsl:with-param name="index" select="0"/>
				<xsl:with-param name="listName" select="."/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:text> match community </xsl:text>
		<xsl:value-of select="$index"/>
		<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>



<!-- ################## SET RULES ################## -->
	<xsl:template match="netml:setRules">
		<xsl:text>&#13;&#10;</xsl:text>
	
		<xsl:if test="netml:automatic_tag = 'true' or netml:automatic_tag ='1' ">
			<xsl:text> set automatic-tag </xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>

<!-- Interface section: we can have "default interface" and "interface" -->
<!-- WATCH: this section cannot be put in a match-template! -->
		<xsl:if test="count(netml:Interface) > 0 ">
			<!-- interface  - not default -->
			<xsl:if test="count(netml:Interface[@default != 'true'] ) > 0 ">
				<xsl:text> set interface </xsl:text>
				<xsl:for-each select="netml:Interface[@default != 'true'] ">
					<xsl:value-of select="."/>
					<xsl:text> </xsl:text>
				</xsl:for-each>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:if>
			<!-- default interface -->
			<xsl:if test="count(netml:Interface[@default = 'true']) > 0 ">
				<xsl:text> set default interface </xsl:text>
				<xsl:for-each select="netml:Interface[@default = 'true'] ">
					<xsl:value-of select="."/>
					<xsl:text> </xsl:text>
				</xsl:for-each>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:if>
		</xsl:if>

		<!-- ip next hop - (da 0 a infiniti)-->
		<xsl:if test=" count(netml:List[@operation = 'IP_next_hop']) >0 ">
			<xsl:text> set ip next-hop </xsl:text>
			<xsl:for-each select="netml:List[@operation = 'IP_next_hop']">
				<xsl:value-of select="."/>
				<xsl:text> </xsl:text>
			</xsl:for-each>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>

		<!-- next hop (se ne dovrebbe avere solo uno)-->
		<xsl:if test=" count(netml:List[@operation = 'next_hop']) >0 ">
			<xsl:text> set next-hop </xsl:text>
			<xsl:for-each select="netml:List[@operation = 'next_hop']">
				<xsl:value-of select="."/>
				<xsl:text> </xsl:text>
			</xsl:for-each>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>


		<xsl:if test="count(netml:IP_default_next_hop) > 0 ">
			<xsl:text> set ip default next-hop </xsl:text>
			<xsl:for-each select="netml:IP_default_next_hop">
				<xsl:value-of select="."/>
				<xsl:text> </xsl:text>
			</xsl:for-each>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>


	<!-- set level -->
		<xsl:if test="count(netml:level) > 0">
			<xsl:text> set level</xsl:text>
			<xsl:choose>
				<xsl:when test="netml:level = 'stub-area' or netml:level = 'backbone' ">
					<xsl:value-of select="netml:level"/>
				</xsl:when>
				<xsl:when test="netml:level = 'level1' ">
					<xsl:text> level-1 </xsl:text>
				</xsl:when>
				<xsl:when test="netml:level = 'level2' ">
					<xsl:text> level-2 </xsl:text>
				</xsl:when>
				<xsl:when test="netml:level = 'level1-2' ">
					<xsl:text> level-1-2 </xsl:text>
				</xsl:when>
			</xsl:choose>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>

		<xsl:if test="count(netml:AS_Path_tag)>0">
			<xsl:if test="netml:AS_Path_tag='true'">
				<xsl:text> set as-path tag</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:if>
		</xsl:if>

		<xsl:if test="count(netml:Community_List)>0">
		
			<xsl:variable name="CommListNames">
				<xsl:for-each select="key('kRouter',string($routerID))/netml:List/netml:ComunityList">
					<xsl:value-of select="../netml:id"/><xsl:text> ; </xsl:text>
				</xsl:for-each>
			</xsl:variable>
			<xsl:variable name="index">
				<xsl:call-template name="indexSearch">
					<xsl:with-param name="nameList" select="$CommListNames"/>
					<xsl:with-param name="index" select="0"/>
					<xsl:with-param name="listName" select="netml:Community_List"/>
				</xsl:call-template>
			</xsl:variable>

			<xsl:choose>
				<xsl:when test="netml:Community_List/@operation='delete'">
					<xsl:text> set comm-list </xsl:text><xsl:value-of select="$index"/>
					<xsl:text> delete</xsl:text>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:when>
				<xsl:when test="netml:Community_List/@operation='none'">
					<xsl:text> set community none</xsl:text>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:when>
				<xsl:otherwise>
					<xsl:for-each select="key('kCommList', string(netml:Community_List))/netml:Community">
						<xsl:text> set community </xsl:text><xsl:value-of select="@asNumber"/>
						<xsl:text>:</xsl:text><xsl:value-of select="@CommunityValue"/>
						<xsl:text> additive</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:for-each>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>

	<!-- set metric-type -->
		<xsl:if test="count(netml:metric_type) > 0">
			<xsl:text> set metric-type</xsl:text>
			<xsl:choose>
				<xsl:when test="netml:metric_type = 'internal' or netml:metric_type = 'external' ">
					<xsl:value-of select="netml:metric_type"/>
				</xsl:when>
				<xsl:when test="netml:metric_type = 'type1' ">
					<xsl:text> type-1 </xsl:text>
				</xsl:when>
				<xsl:when test="netml:metric_type = 'type2' ">
					<xsl:text> type-2 </xsl:text>
				</xsl:when>
			</xsl:choose>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>



		<!-- Now calls the templates -->
		<!-- Indipendent -->
		<xsl:apply-templates select=".//netml:characteristics"/>

		<!-- Bgp -->

	</xsl:template>

	<!-- INDIPENDENT  templates-->
	
	<!--
	<xsl:template match="netml:characteristics/@netml:operation">
		<xsl:message>Policy/Characteristics: attribute operation, if present, will be ignored</xsl:message>
	</xsl:template>
-->
	<xsl:template match="netml:characteristics">
		<xsl:if test="./@netml:operation">
			<xsl:message>Policy/Characteristics: attribute operation, if present, will be ignored</xsl:message>
		</xsl:if>

		<xsl:choose>
				<!-- First specific cases that need a translation -->
			<xsl:when test="./netml:type = 'local preference' ">
				<xsl:text> set local-preference </xsl:text>
				<xsl:value-of select="./netml:value"/>
			</xsl:when>
				<!-- Then the not implemented -->
			<xsl:when test="./netml:type = 'colour'">
				<xsl:message>colour not yet implemented! It seems not to exist in cisco!</xsl:message>
			</xsl:when>
			<xsl:when test="./netml:type = 'preference'">
				<xsl:message>preference not yet implemented! It seems not to exist in cisco!</xsl:message>
			</xsl:when>
			<!-- Then the generic case: just write name and value -->
			<xsl:otherwise>
				<xsl:text> set </xsl:text>
				<xsl:value-of select="./netml:type"/>
				<xsl:text> </xsl:text>
				<xsl:value-of select="./netml:value"/>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template name="static_route">
		<xsl:text>route-map RIP_STATIC permit</xsl:text>
		<xsl:text>&#13;&#10;</xsl:text>
		<xsl:text> match ip address RIP_STATIC</xsl:text>
		<xsl:text>&#13;&#10;</xsl:text>
		<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>
</xsl:stylesheet>
