<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
               version="1.0"
							 xmlns:netml="http://www.xmlnetwork.org"
							 exclude-result-prefixes="netml">

	<xsl:output method="text"  indent="no"/>

	<!-- ###RouterID### --><xsl:variable name="routerID">r01</xsl:variable><!-- ###RouterID### -->
	<xsl:key name="kRouter" match="netml:NetML/netml:Routers/netml:RouterConf" use="@id"/>
	<xsl:key name="kCommList" match="netml:NetML/netml:Routers/netml:RouterConf[@id=string($routerID)]/netml:List/netml:CommunityList" use="../netml:id"/>
	<xsl:key name="kASPList" match="netml:NetML/netml:Routers/netml:RouterConf[@id=string($routerID)]/netml:List/netml:ASPathList" use="../netml:id"/>
	<xsl:key name="list" match="netml:NetML/netml:Routers/netml:RouterConf[@id=string($routerID)]/netml:List/netml:RouteList" use="../netml:id"/>

<xsl:template match="/">

	<xsl:text>policy-options {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		<!-- Policy and Prefix List  Configuration starts here -->
	<xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:PrefixList)>0">
		<xsl:apply-templates select="key('kRouter',string($routerID))/netml:List/netml:PrefixList"/>
	</xsl:if>
	<xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:ASPathList)>0">
		<xsl:apply-templates select="key('kRouter',string($routerID))/netmlList/netml:ASPathList"/>
	</xsl:if>
	<xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:CommunityList)>0">
		<xsl:apply-templates select="key('kRouter',string($routerID))/netml:List/netml:CommunityList"/>
	</xsl:if>
	<xsl:if test="count(key('kRouter',string($routerID))/netml:Policy)>0">
		<xsl:apply-templates select="key('kRouter',string($routerID))/netml:Policy"/>
	</xsl:if>
	<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:Next-Hop-Self)>0">
		<xsl:call-template name="netml:Next-Hop-Self"/>
	</xsl:if>
	<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:List)>0">
		<xsl:apply-templates select="key('kRouter',string($routerID))/netml:BGPConf//netml:List"/>
	</xsl:if>
	<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:Dampening)>0">
		<xsl:apply-templates select="key('kRouter',string($routerID))/netml:BGPConf/netml:Dampening"/>
	</xsl:if>
		<!-- Policy and Prefix List  Configuration ends here -->
<xsl:text>}</xsl:text><xsl:text>&#13;&#10;</xsl:text>

	</xsl:template>

	<!-- Configuring a prefix list  -->
	<xsl:template match="netml:PrefixList">
		<xsl:text>   prefix-list  </xsl:text><xsl:value-of select="../netml:id"/> <xsl:text>  { <!-- the name of the prefix list -->
	</xsl:text>
	<xsl:for-each select="netml:prefix"><!-- the information for a prefix is contained in a prefix-list-item -->
                <xsl:value-of select="."/> ; <!--prefix --></xsl:for-each>
 <xsl:text>   }</xsl:text>

	</xsl:template>

	<!-- Configuring AS Path Lists -->
	<xsl:template match="netml:ASPathList">
		<xsl:text>   as-path </xsl:text><xsl:value-of select="../netml:id"/>
		<xsl:text> [ </xsl:text>
		<xsl:for-each select="netml:RegExpr">
			<xsl:value-of select="."/><xsl:text> </xsl:text>
		</xsl:for-each>
		<xsl:text> ];</xsl:text>
	</xsl:template>
	
	<!-- Configuring Community Lists -->
	<xsl:template match="netml:CommunityList">
		<xsl:text>   community </xsl:text><xsl:value-of select="../netml:id"/><xsl:text> {
</xsl:text>
		<xsl:text>    members [ </xsl:text>
		<xsl:for-each select="netml:Community">
			<xsl:value-of select="@asNumber"/><xsl:text>:</xsl:text>
			<xsl:value-of select="@CommunityValue"/><xsl:text> </xsl:text>
		</xsl:for-each>
		<xsl:text>];</xsl:text>
<xsl:text>
}
</xsl:text>
	</xsl:template>

		<!-- Configuring a Policy -->
	<xsl:template match="netml:Policy">

 policy-statement <xsl:value-of select="@name"/>  { <!-- the policy name -->
					<xsl:apply-templates select="netml:Redistribution|netml:Distribution|netml:General"/><!-- each policy can have terms of three types -->
 }
	</xsl:template>

	<!-- Configuring a Redistribution Policy -->
	<xsl:template match="netml:Redistribution">
<xsl:text>   from { <!-- the from protocol match condition  -->
	protocol </xsl:text> <xsl:value-of select="netml:from_protocol"/><xsl:text>;
   }</xsl:text>

<xsl:text>   to  {<!-- the to protocol match condition -->
	protocol </xsl:text><xsl:value-of select="netml:to_protocol"/><xsl:text>;
   }</xsl:text>
					<xsl:for-each select="netml:term">
<xsl:text>   term </xsl:text><xsl:value-of select="@priority"/><xsl:text> {
</xsl:text>
				<xsl:if test="netml:matchConditions!=''">
<xsl:text> 	from { <!-- the from match conditions  -->
</xsl:text><xsl:apply-templates select="netml:matchConditions"/>
<xsl:text>	}
</xsl:text>
				</xsl:if>
<xsl:text>	then { <!-- the actions which a policy must do if all the match conditions are met -->
</xsl:text><xsl:apply-templates select="netml:setRules"/>
								<xsl:variable name="target" select="@target"/> <!-- the accept or reject action -->
								<xsl:choose>
										<xsl:when test="$target='permit'"><!-- permit=accept -->
		accept;
										</xsl:when>
										<xsl:otherwise><!-- deny -->
		reject;
										</xsl:otherwise>
									</xsl:choose>
	}
   }
				</xsl:for-each>

	</xsl:template>

	<!-- Configuring a Distribution Policy -->
	<xsl:template match="netml:Distribution">

<!-- Distribution Policies only have  interface and list values as match conditions and only accept or reject actions. -->

   term <xsl:value-of select="@priority"/> {

	from { <!-- the from match conditions  -->
									<xsl:variable name="interface" select="netml:interface"/>
									<xsl:variable name="list" select="netml:list"/>
									<xsl:call-template name="list"><!-- there are two types of lists: Prefix or Route Lists. With this template we correctly define the list.  -->
										<xsl:with-param name="listName"><xsl:value-of select="$list"/></xsl:with-param>
									</xsl:call-template>
									<xsl:if test="$interface!=''"><!-- the interface condition is optional  -->
		interface <xsl:value-of select="$interface"/>;
									</xsl:if>
	}

	then { <!-- the actions which a policy must do if all the match conditions are met -->
									<xsl:variable name="target" select="@target"/>
									<xsl:choose>
										<xsl:when test="$target='permit'">
		accept;
										</xsl:when>
										<xsl:otherwise><!-- $target='deny' -->
		reject;
										</xsl:otherwise>
									</xsl:choose>
	}
   }

	</xsl:template>

	<!-- Configuring a General Policy -->
	<xsl:template match="netml:General">

			<xsl:for-each select="netml:term">
   term <xsl:value-of select="@priority"/>  {

				<xsl:if test="netml:matchConditions!=''">
	from { <!-- the  match conditions -->
								<xsl:apply-templates select="netml:matchConditions"/>
	}
				</xsl:if>

	then { <!-- the actions which a policy must do if all the match conditions are met -->

		<xsl:apply-templates select="netml:setRules"/><!-- there can be other actions to do (defined in the setRules) -->
									<xsl:variable name="target" select="@target"/>
									<xsl:choose>
										<xsl:when test="$target='permit'">
		accept;
										</xsl:when>
										<xsl:otherwise><!-- $target='deny' -->
		reject;
										</xsl:otherwise>
									</xsl:choose>

	}
   }
				</xsl:for-each>

	</xsl:template>

	<!-- The match conditions are evaluated here -->
	<xsl:template match="netml:matchConditions">

<!-- As the match conditions are optional (not all may be  needed) we define a variable for each condition -->
	<xsl:variable name="family" select="netml:family"/>
	<xsl:variable name="protocol" select="netml:protocol"/>
	<xsl:variable name="origin" select="netml:origin"/>
	<xsl:variable name="level" select="netml:level"/>
	<xsl:variable name="tag" select="netml:tag"/>
	<xsl:variable name="preference" select="netml:preference"/>
	<xsl:variable name="colour" select="netml:colour"/>
	<xsl:variable name="local_preference" select="netml:local_preference"/>
	<xsl:variable name="metric" select="netml:metric"/>
	<xsl:variable name="external_route" select="netml:external_route"/>
	<xsl:variable name="packet_length_min" select="netml:packet_length/@min"/>
	<xsl:variable name="packet_length_max" select="netml:packet_length/@max"/>
	<xsl:variable name="route_type" select="netml:route_type"/>
	<xsl:variable name="area" select="netml:area"/>
	<xsl:variable name="AS_Path_List" select="netml:AS_Path_List"/>
	<xsl:variable name="Community_List" select="netml:Community_List"/>
	<xsl:variable name="Interface" select="netml:Interface"/>
	<xsl:variable name="List" select="netml:List"/>

	<!-- Now we start to test the conditions to see if they were requested or not and if they exist, we prepare the tags and their relative  values -->
	<xsl:if test="$family!=''"><!-- the address family (eg inet, inet6) -->

		family <xsl:value-of select="$family"/>;

	</xsl:if><!-- family -->

	<xsl:if test="$protocol!=''"><!-- the protocol (eg rip, bgp) -->

		protocol <xsl:value-of select="$protocol"/>;

	</xsl:if><!-- protocol -->

	<xsl:if test="$origin!=''"><!-- has this route come from inside the AS or from an external AS or do we not know?  -->

		origin <xsl:value-of select="$origin"/>;

	</xsl:if><!-- origin -->

	<xsl:if test="$level!=''"><!-- IS-IS level  -->
		level <xsl:value-of select="$level"/>;
	</xsl:if><!-- level -->

	<xsl:if test="$tag!=''">
		tag <xsl:value-of select="$tag"/>;
	</xsl:if><!-- tag -->

	<xsl:if test="$preference!=''">
		preference <xsl:value-of select="$preference"/>;
	</xsl:if><!-- preference -->

	<xsl:if test="$colour!=''">
		color <xsl:value-of select="$colour"/>;
	</xsl:if><!-- colour -->

	<xsl:if test="$local_preference!=''">
		local-preference <xsl:value-of select="$local_preference"/>;
	</xsl:if><!-- local preference -->

	<xsl:if test="$metric!=''">
		metric <xsl:value-of select="$metric"/>;
	</xsl:if><!-- metric -->

	<xsl:if test="$external_route!=''"><!-- OSPF metric types -->
		external type <xsl:value-of select="$external_route"/>;
	</xsl:if><!-- external route -->

	<xsl:if test="$packet_length_min!='' and $packet_length_max!=''"><!-- CISCO match condition -->
		<xsl:message> Warning: The packet length condition is not a valid condition for Juniper.</xsl:message>
	</xsl:if><!-- packet length min -->

	<xsl:if test="$route_type!=''"><!-- CISCO match condition -->
		<xsl:message> Warning: Route Type is not a valid condition for Juniper.</xsl:message>
	</xsl:if><!-- route type -->

	<xsl:if test="$area!=''"><!-- OSPF area identifier -->
		area <xsl:value-of select="$area"/>;
	</xsl:if><!-- area -->

	<xsl:if test="$AS_Path_List!=''"><!-- BGP: testing on AS-Paths -->
		as-path <xsl:value-of select="$AS_Path_List"/>;
	</xsl:if><!-- AS Path List -->

	<xsl:if test="$Community_List!=''"><!-- BGP: testing on Communities -->
		community [ <xsl:value-of select="$Community_List"/> ] ;
	</xsl:if><!-- Community List-->

	<xsl:if test="$Interface!=''">
		interface <xsl:value-of select="$Interface"/>;
	</xsl:if><!-- Interface -->

	<xsl:if test="$List!=''"><!-- There are two types of Lists which can be matched at different levels (next hop, neighbour) -->
		<xsl:variable name="match" select="@match"/>
		<xsl:choose>
			<xsl:when test="$match='next_hop'"><!-- next hop -->

		next-hop <xsl:value-of select="$List"/>;
			</xsl:when>
			<xsl:when test="$match='neighbour'"><!-- neighbour -->

		neighbor <xsl:value-of select="$List"/>;
			</xsl:when>
			<xsl:otherwise><!-- IP_Address -->
				<xsl:call-template name="list">
					<xsl:with-param name="listName"><xsl:value-of select="$List"/></xsl:with-param>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>

	</xsl:if><!-- List -->

	</xsl:template>

	<!-- Defining the Lists using different methods for the two types.  -->
	<xsl:template name="list">
		<xsl:param name="listName"/>

		<!-- We need to know if the list is a prefix list or a route list. -->
		<xsl:variable name="list_type"><xsl:for-each select="//netml:List">
			<xsl:if test="netml:id=$listName"><xsl:value-of select="netml:PrefixList"/></xsl:if>
			</xsl:for-each>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="$list_type!=''"><!-- It is a prefix list -->
		prefix-list <xsl:value-of select="$listName"/>;
			</xsl:when>
			<xsl:otherwise><!-- It is a route list -->
				<xsl:for-each select="key('list',$listName)/netml:routeFilter"><!-- Each route list can have more than one route defined through Route Filters. We need to define a route filter for each route. -->
					<xsl:variable name="matchType"><xsl:value-of select="name(key('list',$listName)/netml:routeFilter/netml:matchType/child::*)"/></xsl:variable><!-- we need to know which match type we are dealing with. (eg exact, longer, ...) -->

		route-filter <xsl:value-of select="key('list',$listName)/netml:routeFilter/netml:prefix"/><!-- identifier -->
										<xsl:if test="$matchType='Exact'"><!-- the address must be matched exactly (the even prefix length) -->
		 exact <!-- identifier -->
										</xsl:if>
										<xsl:if test="$matchType='Longer'"><!-- the address must have a longer prefix length -->
		longer <!-- identifier -->
										</xsl:if>
										<xsl:if test="$matchType='OrLonger'"><!-- the prefix length must be longer or equal to the defined address -->
		orlonger <!-- identifier -->
										</xsl:if>
										<xsl:if test="$matchType='Upto'"><!-- the prefix length is contained in the interval defed_prefix_length - upto_prefix_length -->
		upto <xsl:value-of select="key('list',$listName)/netml:routeFilter/netml:matchType/netml:Upto/netml:prefixLength"/><!-- identifier -->
										</xsl:if>
										<xsl:if test="$matchType='Through'"><!-- the addresses that are matched are contained in the interval defed_prefix  - through_prefix  -->
		through <xsl:value-of select="key('list',$listName)/netml:routeFilter/netml:matchType/netml:Through"/> <!-- identifier -->
										</xsl:if>
										<xsl:if test="$matchType='PrefixLengthRange'"><!-- the addresses matched have a prefix length contained in the interval min/max -->
		prefix-length-range <xsl:value-of select="key('list',$listName)/netml:routeFilter/netml:matchType/netml:PrefixLengthRange/@lower_bound"/> - <xsl:value-of select="key('list',$listName)/netml:routeFilter/netml:matchType/netml:PrefixLengthRange/@upper_bound"/>
										</xsl:if>
										<xsl:variable name="target" select="key('list',$listName)/netml:routeFilter/netml:target"/>
										<xsl:choose>
											<xsl:when test="$target='permit'">
		accept
											</xsl:when>
											<xsl:otherwise><!-- $target='deny' -->
		reject
											</xsl:otherwise>
										</xsl:choose>
										;
					</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- Configuring the  Set Rules -->
	<xsl:template match="netml:setRules">

	<!-- As the set rules are optional, we define variables to see if they exist -->
	<xsl:variable name="source_class" select="netml:source_class"/>
	<xsl:variable name="forwarding_class" select="netml:forwarding_class"/>
	<xsl:variable name="destination_class" select="netml:destination_class"/>
	<xsl:variable name="AS_Path_Expand_Last_AS_Count" select="netml:AS_Path_Expand_Last_AS_Count"/>
	<xsl:variable name="automatic_tag" select="netml:automatic_tag"/>
	<xsl:variable name="AS_Path_tag" select="netml:AS_Path_tag"/>
	<xsl:variable name="IP_default_next_hop" select="netml:IP_default_next_hop"/>
	<xsl:variable name="metric_type" select="netml:metric_type"/>
	<xsl:variable name="damping" select="netml:damping"/>
	<xsl:variable name="weight" select="netml:weight"/>
	<xsl:variable name="class_of_service" select="netml:class_of_service"/>
	<xsl:variable name="load_balancing_per_packet" select="netml:load_balancing_per_packet"/>
	<xsl:variable name="metric_igp" select="netml:metric_igp"/>
	<xsl:variable name="external_type" select="netml:external_type"/>
	<xsl:variable name="level" select="netml:level"/>
	<xsl:variable name="origin" select="netml:origin"/>
	<xsl:variable name="characteristics_type" select="netml:characteristics/netml:type"/>
	<xsl:variable name="AS_Path_list" select="netml:AS_Path_list"/>
	<xsl:variable name="Community_List" select="netml:Community_List"/>
	<xsl:variable name="Interface" select="netml:Interface"/>
	<xsl:variable name="List" select="netml:List"/>

	<!-- we begin testing the possibile rules -->
	<xsl:if test="$source_class!=''"><!-- adds the route to a source class  -->
		source-class <xsl:value-of select="$source_class"/>;
	</xsl:if><!-- source class -->
	
	<xsl:if test="$forwarding_class!=''"><!-- adds the route to a forwarding class -->
		forwarding-class <xsl:value-of select="$forwarding_class"/>;
	</xsl:if><!-- forwarding class -->

	<xsl:if test="$destination_class!=''"><!-- adds the route to a destination class -->
		destination-class <xsl:value-of select="$destination_class"/>;
	</xsl:if><!-- destination class -->

	<xsl:if test="$AS_Path_Expand_Last_AS_Count!=''"><!-- BGP: We can decide to repeat the last AS as many times as the variable says  -->
		as-path-expand last-as count  <xsl:value-of select="$AS_Path_Expand_Last_AS_Count"/>;
	</xsl:if><!-- AS_Path_Expand_Last_AS_Count -->

	<xsl:if test="$automatic_tag!=''"><!-- CISCO command -->
	<xsl:message>Warning: The 'automatic tag' condition is not implemented in the Juniper software.</xsl:message>
	</xsl:if><!-- automatic tag -->

	<xsl:if test="$AS_Path_tag!=''"><!-- CISCO command -->
		<xsl:message>Warning: The 'AS Path tag' condition is not implemented in the Juniper software.</xsl:message>
	</xsl:if><!-- AS Path tag -->

	<xsl:if test="$IP_default_next_hop!=''"><!-- CISCO command -->
		<xsl:message>Warning: The 'IP default next hop' condition is not implemented in the Juniper software.</xsl:message>
	</xsl:if><!-- IP default next hop -->
	
	<xsl:if test="$metric_type!=''"><!-- CISCO command -->
		<xsl:message>Warning: The 'metric type' condition is not implemented in the Juniper software.</xsl:message>
	</xsl:if><!-- metric type -->
	
	<xsl:if test="$damping!=''"><!-- BGP: to apply route-damping parameters to the routes -->
		damping <xsl:value-of select="$damping"/>;
	</xsl:if><!-- damping -->

	<xsl:if test="$weight!=''"><!-- CISCO command -->
		<xsl:message>Warning: The 'weight' condition is not implemented in the Juniper software.</xsl:message>
	</xsl:if><!-- weight  -->

	<xsl:if test="$class_of_service!=''"><!-- to apply class of service parameters to routes installed in the routing table -->
		class <xsl:value-of select="$class_of_service"/>;
	</xsl:if><!-- class_of_service -->

	<xsl:if test="$load_balancing_per_packet='true'"><!-- to install all the routes to the forwarding table allowing it to decide where to send the packets -->
		load balance per-packet;
	</xsl:if><!-- load_balancing_per_packet -->
	
	<xsl:if test="$metric_igp!=''"><!-- EBGP: to change meds with the specified offset. -->
		<xsl:variable name="type" select="netml:metric_igp/@type"/>
		<xsl:choose><!-- type -->
			<xsl:when test="$type='igp'">
		metric igp <xsl:value-of select="$metric_igp"/>;
			</xsl:when><!--/igp -->
			<xsl:otherwise><!-- $type='minimym_igp' -->
		metric minimum-igp <xsl:value-of select="$metric_igp"/>;
			</xsl:otherwise><!-- /min igp -->
		</xsl:choose><!--/type-->
	</xsl:if><!-- metric igp -->
	
	<xsl:if test="$external_type!=''"><!-- OSPF metric type -->
		external type <xsl:value-of select="$external_type"/>;
	</xsl:if><!-- external type -->

	<xsl:if test="$level!=''"><!-- CISCO command -->
		<xsl:message>Warning: The 'level' condition is not implemented in the Juniper software.</xsl:message>
	</xsl:if><!-- level -->
	
	<xsl:if test="$origin!=''"><!-- set the origin of the route (igp, egp...)  -->
		origin <xsl:value-of select="$origin"/>;
	</xsl:if><!-- origin -->

	<xsl:if test="$characteristics_type!=''"><!-- there are five characteristics which have a common structure  -->
		<xsl:choose><!-- characteristics_type -->
			<xsl:when test="$characteristics_type='local preference'">
				<xsl:variable name="operation" select="netml:characteristics/@operation"/>
				<xsl:variable name="value" select="netml:characteristics/netml:value"/>
				<xsl:choose><!-- local pref -->
					<xsl:when test="$operation='none'">
		local-preference <xsl:value-of select="$value"/>;
					</xsl:when><!-- /none -->
					<xsl:when test="$operation='add'">
		local-preference> add <xsl:value-of select="$value"/>;
					</xsl:when><!-- /add -->
					<xsl:otherwise><!-- $operation='subtract' -->
		local-preference subtract <xsl:value-of select="$value"/>;
					</xsl:otherwise><!-- /subtract -->
				</xsl:choose><!-- /local pref -->
			</xsl:when>
			<xsl:when test="$characteristics_type='preference'">
				<xsl:variable name="operation" select="netml:characteristics/@operation"/>
				<xsl:variable name="value" select="netml:characteristics/netml:value"/>
				<xsl:choose><!-- preference -->
					<xsl:when test="$operation='none'">
		preference <xsl:value-of select="$value"/>;
					</xsl:when><!-- /none -->
					<xsl:when test="$operation='add'">
		preference add <xsl:value-of select="$value"/>;
					</xsl:when><!-- /add -->
					<xsl:otherwise><!-- $operation='subtract' -->
		preference subtract <xsl:value-of select="$value"/>;
					</xsl:otherwise><!-- /subtract -->
				</xsl:choose><!-- /preference -->
			</xsl:when>
			<xsl:when test="$characteristics_type='tag'">
				<xsl:variable name="operation" select="netml:characteristics/@operation"/>
				<xsl:variable name="value" select="netml:characteristics/netml:value"/>
				<xsl:choose><!-- tag -->
					<xsl:when test="$operation='none'">
		tag <xsl:value-of select="$value"/>;
					</xsl:when><!-- /none -->
					<xsl:when test="$operation='add'">
		tag add <xsl:value-of select="$value"/>;
					</xsl:when><!-- /add -->
					<xsl:otherwise><!-- $operation='subtract' -->
		tag subtract <xsl:value-of select="$value"/>;
					</xsl:otherwise><!-- /subtract -->
				</xsl:choose><!-- /tag -->
			</xsl:when>
			<xsl:when test="$characteristics_type='metric'">
				<xsl:variable name="operation" select="netml:characteristics/@operation"/>
				<xsl:variable name="value" select="netml:characteristics/netml:value"/>
				<xsl:choose><!-- metric -->
					<xsl:when test="$operation='none'">
		metric <xsl:value-of select="$value"/>;
					</xsl:when><!-- /none -->
					<xsl:when test="$operation='add'">
		metric add <xsl:value-of select="$value"/>;
					</xsl:when><!-- /add -->
					<xsl:otherwise><!-- $operation='subtract' -->
		metric subtract <xsl:value-of select="$value"/>;
					</xsl:otherwise><!-- /subtract -->
				</xsl:choose><!-- /metric -->
			</xsl:when>
			<xsl:otherwise><!--$characteristics_type='colour' -->
				<xsl:variable name="operation" select="netml:characteristics/@operation"/>
				<xsl:variable name="value" select="netml:characteristics/netml:value"/>
				<xsl:choose>
					<xsl:when test="$operation='none'">
		color <xsl:value-of select="$value"/>;
					</xsl:when><!-- /none -->
					<xsl:when test="$operation='add'">
		color add <xsl:value-of select="$value"/>;
					</xsl:when><!-- /add -->
					<xsl:otherwise><!-- $operation='subtract' -->
		color subtract <xsl:value-of select="$value"/>;
					</xsl:otherwise><!-- /subtract -->
				</xsl:choose><!-- /colour -->
			</xsl:otherwise>
		</xsl:choose>
	</xsl:if><!-- characteristics -->

	<xsl:if test="$AS_Path_list!=''"><!-- AS Path Prepend -->
		<xsl:text> as-path-prepend "</xsl:text>
		<xsl:for-each select="key('kASPList',string(netml:AS_Path_List))/netml:RegExpr">
			<xsl:value-of select="."/>
		</xsl:for-each>
		<xsl:text>"</xsl:text><xsl:text>&#13;&#10;</xsl:text>
	</xsl:if><!-- AS Path List -->

	<xsl:if test="$Community_List!=''"><!-- add the communities to the community list in the route -->
		<xsl:variable name="operation" select="netml:Community_List/@operation"/>
		<xsl:for-each select="netml:Community_List">

		community
								<xsl:choose>
									<xsl:when test="$operation='add'">
		add
									</xsl:when>
									<xsl:when test="$operation='none'">
		set
									</xsl:when>
									<xsl:otherwise><!-- $operation='delete' -->
		delete
									</xsl:otherwise>
								</xsl:choose>
		[ <xsl:value-of select="@asNumber"/><xsl:text>:</xsl:text><xsl:value-of select="@CommunityValue"/> ]; <!-- identifier -->
		</xsl:for-each>
	</xsl:if><!-- Community List -->

	<xsl:if test="$Interface!=''"><!-- CISCO command -->
		<xsl:message>Warning: The 'Interface' condition is not implemented in the Juniper software.</xsl:message>
	</xsl:if><!-- Interface -->

	<xsl:if test="$List!=''"><!-- set the next hop or the 'install next hop' value for the route -->
		<xsl:variable name="operation" select="netml:List/@operation"/>

		<xsl:choose>
			<xsl:when test="$operation='next_hop'">
		next-hop <xsl:value-of select="$List"/>;
			</xsl:when>
			<xsl:otherwise><!-- $operation='install_next_hop' -->
		install-nexthop lsp <xsl:value-of select="$List"/>;
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:if><!-- List -->
	</xsl:template>
	
	<xsl:template name="netml:Next-Hop-Self">
		<xsl:text> policy-statement Next-Hop-Self {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		<xsl:text>    then {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		<xsl:text>       next-hop self;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		<xsl:text>    }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		<xsl:text> }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template match="netml:List">
		<xsl:choose>
			<xsl:when test="@type='Filter'">
				<xsl:text> policy-statement </xsl:text><xsl:value-of select="."/>
				<xsl:text> {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>        term 10 {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:choose>
					<xsl:when test="@inOut='import'">
						<xsl:text>            from {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                 as-path </xsl:text><xsl:value-of select="."/>
						<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>            }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>            to {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                 as-path </xsl:text><xsl:value-of select="."/>
						<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>            }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
				<xsl:text>        }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text> }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text> policy-statement </xsl:text><xsl:value-of select="."/>
				<xsl:text> {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>        term 10 {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:choose>
					<xsl:when test="@inOut='import'">
						<xsl:text>            from {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:for-each select="key('list', .)/netml:routeFilter">
							<xsl:text>                 route-filter </xsl:text><xsl:value-of select="netml:prefix"/>
							<xsl:text> </xsl:text><xsl:value-of select="netml:matchType"/>
							<xsl:text> </xsl:text><xsl:value-of select="netml:target"/>
							<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						</xsl:for-each>
						<xsl:text>            }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>           to {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:for-each select="key('list', .)/netml:routeFilter">
							<xsl:text>                 route-filter </xsl:text><xsl:value-of select="netml:prefix"/>
							<xsl:text> </xsl:text><xsl:value-of select="netml:matchType"/>
							<xsl:text> </xsl:text><xsl:value-of select="netml:target"/>
							<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						</xsl:for-each>
						<xsl:text>            }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
				<xsl:text>        }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text> }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="netml:Dampening">
		<xsl:text> damping BGP</xsl:text>
		<xsl:if test="count(../netml:view)>0">
			<xsl:text>-</xsl:text><xsl:value-of select="../netml:view"/>
		</xsl:if>
		<xsl:text> {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		<xsl:if test="count(netml:halflife)>0">
			<xsl:text>       half-life </xsl:text><xsl:value-of select="netml:halflife"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:suppress)>0">
			<xsl:text>       suppress </xsl:text><xsl:value-of select="netml:suppress"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:reuse)>0">
			<xsl:text>       reuse </xsl:text><xsl:value-of select="netml:reuse"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(netml:maxSuppressTime)>0">
			<xsl:text>       max-suppress </xsl:text><xsl:value-of select="netml:maxSuppressTime"/>
			<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:text> }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		<xsl:text> policy-statement damp-BGP</xsl:text>
		<xsl:if test="count(../netml:view)>0">
			<xsl:text>-</xsl:text><xsl:value-of select="../netml:view"/>
		</xsl:if>
		<xsl:text> {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		<xsl:text>       then damping BGP</xsl:text>
		<xsl:if test="count(../netml:view)>0">
			<xsl:text>-</xsl:text><xsl:value-of select="../netml:view"/>
		</xsl:if>
		<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		<xsl:text> }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>

</xsl:stylesheet>



