<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0"
								xmlns:netml="http://www.xmlnetwork.org">

   <xsl:output method="text" />
   <xsl:strip-space elements="*"/>

   <!-- ###RouterID### --><xsl:variable name="routerID">r01</xsl:variable><!-- ###RouterID### -->
   <xsl:key name="kRouter" match="netml:NetML/netml:Routers/netml:RouterConf" use="@id"/>
   <xsl:key name="kCommList" match="netml:NetML/netml:Routers/netml:RouterConf[@id=string($routerID)]/netml:List/netml:CommunityList" use="../netml:id"/>
   <xsl:key name="kASPList" match="netml:NetML/netml:Routers/netml:RouterConf[@id=string($routerID)]/netml:List/netml:ASPathList" use="../netml:id"/>
   <xsl:key name="kList" match="//netml:RouterConf[@id=string($routerID)]/netml:List" use="netml:id"/>

   <xsl:template match="/">
         <xsl:text>!&#13;&#10;</xsl:text>
         <xsl:text>!Start of Policy Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:apply-templates select="key('kRouter', string($routerID))/netml:Policy"/>
         <xsl:text>!End of Policy Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:text>!&#13;&#10;</xsl:text>
   </xsl:template>


      <xsl:template match="netml:Policy/netml:Redistribution">
         <xsl:apply-templates select="/netml:term"/>
      </xsl:template>

      <xsl:template match="netml:Policy/netml:General">
         <xsl:apply-templates select="/netml:term"/>
      </xsl:template>

      <xsl:template match="netml:Policy/netml:Distribution">
      </xsl:template>

      <xsl:template match="netml:Policy/netml:Redistribution/netml:term | netml:Policy/netml:General//netml:term">
         <xsl:variable name="pname" select="../../@name"/>
         <xsl:if test="count(netml:matchConditions/netml:Interface)>0 or
            count(netml:matchConditions/netml:List)>0 or
            count(netml:matchConditions/netml:metric)>0 or
            count(netml:matchConditions/netml:AS_Path_List)>0 or
            count(netml:matchConditions/netml:Community_List)>0 or
            count(netml:setRules/netml:weight)>0 or
            count(netml:setRules/netml:charcateristics[(netml:type='metric'and @operation='none') or (netml:type='local-preference'and @operation='none')])>0 or
            count(netml:setRules/netml:AS_Path_List)>0 or
            count(netml:setRules/netml:Community_List[@operation='none'])>0 or
            count(netml:setRules/netml:List[@operation!='install_next_hop'])>0">
            <xsl:text>route-map </xsl:text>
            <xsl:value-of select="$pname"/>
            <xsl:text> </xsl:text>
            <xsl:value-of select="@target"/>
            <xsl:text> </xsl:text>
            <xsl:value-of select="@priority"/>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:if test="count (netml:matchConditions) >0">
               <xsl:apply-templates select="netml:matchConditions"/>
            </xsl:if>
            <xsl:if test="count(netml:setRules)>0">
               <xsl:apply-templates select="netml:setRules"/>
            </xsl:if>
            <xsl:text>!&#13;&#10;</xsl:text>
         </xsl:if>
      </xsl:template>


   <xsl:template match="netml:matchConditions">

      <xsl:variable name="metric" select="netml:metric"/>
      <!--xsl:variable name="AS_Path_List" select="netml:AS_Path_List"/-->
      <!--xsl:variable name="Community_List" select="netml:Community_List"/-->
      <!-- <xsl:variable name="Interface" select="netml:Interface"/> -->
      <!-- <xsl:variable name="List" select="netml:List"/> -->

      <xsl:if test="count(netml:Interface) > 0">
         <xsl:for-each select="netml:Interface">
            <xsl:text> match interface </xsl:text>
            <xsl:value-of select="."/>
            <xsl:text> </xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
         </xsl:for-each>
      </xsl:if>

      <xsl:if test=" count(netml:List) >0 ">
         <xsl:for-each select="netml:List">
            <xsl:variable name="match_type" select="@match"/>
            <xsl:choose>
               <xsl:when test=" $match_type = 'next_hop' ">
                  <xsl:choose>
                     <xsl:when test="name(key('list', .)/child::*)='PrefixList'">
                        <xsl:for-each select="key('list',.)/netml:PrefixList/netml:prefix">
                           <xsl:text> match ip next-hop </xsl:text><xsl:value-of select="."/>
                           <xsl:text>&#13;&#10;</xsl:text>
                        </xsl:for-each>
                     </xsl:when>
                     <xsl:otherwise>
                        <xsl:for-each select="key('list',.)/netml:RouteList/netml:routeFilter/netml:prefix">
                        <xsl:text> match ip next-hop </xsl:text><xsl:value-of select="."/>
                        <xsl:text>&#13;&#10;</xsl:text>
                        </xsl:for-each>
                     </xsl:otherwise>
                  </xsl:choose>
               </xsl:when>
               <xsl:when test=" $match_type = 'neighbour' ">
                  <xsl:message>Warning (neighbour match condition): It is not possibile to define a match condition with this match type in Zebra.</xsl:message>
               </xsl:when>
               <xsl:when test=" $match_type = 'IP_address' ">
                  <xsl:text> match ip address </xsl:text><xsl:value-of select="." />
                  <xsl:text>&#13;&#10;</xsl:text>
               </xsl:when>
            </xsl:choose>
         </xsl:for-each>
      </xsl:if>

      <xsl:if test="$metric!=''">
         <xsl:text> match metric </xsl:text><xsl:value-of select="$metric"/>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:if>

      <xsl:if test="count(netml:AS_Path_List) > 0">
         <xsl:for-each select="netml:AS_Path_List">
            <xsl:text> match as-path </xsl:text><xsl:value-of select="."/>
            <xsl:text>&#13;&#10;</xsl:text>
         </xsl:for-each>
      </xsl:if>

      <xsl:if test="count(netml:Community_List) > 0">
         <xsl:for-each select="netml:Community_List">
            <xsl:text> match community </xsl:text><xsl:value-of select="."/>
            <xsl:text>&#13;&#10;</xsl:text>
         </xsl:for-each>
      </xsl:if>
   </xsl:template>

   <!-- Configuring the  Set Rules -->
   <xsl:template match="netml:setRules">

   <!-- As the set rules are optional, we define variables to see if they exist -->
   <xsl:variable name="weight" select="netml:weight"/>
   <xsl:variable name="characteristics_type" select="netml:characteristics/netml:type"/>
   <!--xsl:variable name="AS_Path_list" select="netml:AS_Path_list"/-->
   <!--xsl:variable name="Community_List" select="netml:Community_List"/-->
   <xsl:variable name="Interface" select="netml:Interface"/>
   <!--xsl:variable name="List" select="netml:List"/-->

   <!-- we test the possibile rules -->
   <xsl:if test="$weight!=''"><!-- weight -->
      <xsl:text> set weight </xsl:text><xsl:value-of select="$weight"/><xsl:text>&#13;&#10;</xsl:text>
   </xsl:if><!-- weight  -->

   <xsl:if test="count(netml:charcateristics)>'0'">
   <!-- there are five characteristics which have a common structure  -->
      <xsl:choose><!-- characteristics_type -->
         <xsl:when test="$characteristics_type='local preference'">
            <xsl:variable name="operation" select="netml:characteristics/@operation"/>
            <xsl:variable name="value" select="netml:characteristics/netml:value"/>
            <xsl:choose><!-- local pref -->
               <xsl:when test="$operation='none'">
                  <xsl:text> set local-preference </xsl:text><xsl:value-of select="$value"/>
                  <xsl:text>&#13;&#10;</xsl:text>
               </xsl:when><!-- /none -->
               <xsl:when test="$operation='add'">
               <xsl:message>Warning (Add Local Preference): this characteristic is not implemented as a set rule in Zebra</xsl:message>
               </xsl:when><!-- /add -->
               <xsl:otherwise><!-- $operation='subtract' -->
               <xsl:message>Warning (Subtract Local Preference): this characteristic is not implemented as a set rule in Zebra</xsl:message>
               </xsl:otherwise><!-- /subtract -->
            </xsl:choose><!-- /local pref -->
         </xsl:when>
         <xsl:when test="$characteristics_type='preference'">
            <xsl:variable name="operation" select="netml:characteristics/@operation"/>
            <xsl:variable name="value" select="netml:characteristics/netml:value"/>
            <xsl:choose><!-- preference -->
               <xsl:when test="$operation='none'">
               <xsl:message>Warning (Preference): this characteristic is not implemented as a set rule in Zebra</xsl:message>
               </xsl:when><!-- /none -->
               <xsl:when test="$operation='add'">
               <xsl:message>Warning (Add Preference): this characteristic is not implemented as a set rule in Zebra</xsl:message>
               </xsl:when><!-- /add -->
               <xsl:otherwise><!-- $operation='subtract' -->
               <xsl:message>Warning (Subtract Preference): this characteristic is not implemented as a set rule in Zebra</xsl:message>
               </xsl:otherwise><!-- /subtract -->
            </xsl:choose><!-- /preference -->
         </xsl:when>
         <xsl:when test="$characteristics_type='tag'">
            <xsl:variable name="operation" select="netml:characteristics/@operation"/>
            <xsl:variable name="value" select="netml:characteristics/netml:value"/>
            <xsl:choose><!-- tag -->
               <xsl:when test="$operation='none'">
               <xsl:message>Warning (Tag): this characteristic is not implemented as a set rule in Zebra</xsl:message>
               </xsl:when><!-- /none -->
               <xsl:when test="$operation='add'">
               <xsl:message>Warning (Add Tag): this characteristic is not implemented as a set rule in Zebra</xsl:message>
               </xsl:when><!-- /add -->
               <xsl:otherwise><!-- $operation='subtract' -->
               <xsl:message>Warning (Subtract Tag): this characteristic is not implemented as a set rule in Zebra</xsl:message>
               </xsl:otherwise><!-- /subtract -->
            </xsl:choose><!-- /tag -->
         </xsl:when>
         <xsl:when test="$characteristics_type='metric'">
            <xsl:variable name="operation" select="netml:characteristics/@operation"/>
            <xsl:variable name="value" select="netml:characteristics/netml:value"/>
            <xsl:choose><!-- metric -->
               <xsl:when test="$operation='none'">
                  <xsl:text> set metric </xsl:text><xsl:value-of select="$value"/>
               </xsl:when><!-- /none -->
               <xsl:when test="$operation='add'">
               <xsl:message>Warning (Add Metric): this characteristic is not implemented as a set rule in Zebra</xsl:message>
               </xsl:when><!-- /add -->
               <xsl:otherwise><!-- $operation='subtract' -->
               <xsl:message>Warning (Subtract Metric): this characteristic is not implemented as a set rule in Zebra</xsl:message>
               </xsl:otherwise><!-- /subtract -->
            </xsl:choose><!-- /metric -->
         </xsl:when>
         <xsl:otherwise><!--$characteristics_type='colour' -->
            <xsl:variable name="operation" select="netml:characteristics/@operation"/>
            <xsl:variable name="value" select="netml:characteristics/netml:value"/>
            <xsl:choose>
               <xsl:when test="$operation='none'">
               <xsl:message>Warning (Colour): this characteristic is not implemented as a set rule in Zebra</xsl:message>
               </xsl:when><!-- /none -->
               <xsl:when test="$operation='add'">
               <xsl:message>Warning (Add Colour): this characteristic is not implemented as a set rule in Zebra</xsl:message>
               </xsl:when><!-- /add -->
               <xsl:otherwise><!-- $operation='subtract' -->
               <xsl:message>Warning (Subtract Colour): this characteristic is not implemented as a set rule in Zebra</xsl:message>
               </xsl:otherwise><!-- /subtract -->
            </xsl:choose><!-- /colour -->
         </xsl:otherwise>
      </xsl:choose>
   </xsl:if><!-- characteristics -->

   <xsl:if test="count(netml:AS_Path_List)>0">
      <xsl:for-each select="netml:AS_Path_List">
         <xsl:for-each select="key('kASPList',string(.))/netml:RegExpr">
            <xsl:text> set as-path prepend </xsl:text><xsl:value-of select="."/><xsl:text>&#13;&#10;</xsl:text>
         </xsl:for-each>
      </xsl:for-each>
   </xsl:if><!-- AS Path List -->

   <xsl:if test="count(Community_List)>0"><!-- add the communities to the community list in the route -->
      <xsl:for-each select="netml:Community_List">
         <xsl:variable name="operation" select="@operation"/>
            <xsl:choose>
               <xsl:when test="$operation='none'">
                  <xsl:text> set community none</xsl:text><xsl:text>&#13;&#10;</xsl:text>
               </xsl:when>
               <xsl:when test="$operation='add'">
                  <xsl:for-each select="key('kCommList', string(.))/netml:Community">
                     <xsl:text> set community </xsl:text><xsl:value-of select="@asNumber"/>
                     <xsl:text>:</xsl:text><xsl:value-of select="@CommunityValue"/>
                     <xsl:text>&#13;&#10;</xsl:text>
                  </xsl:for-each>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:text> set comm-list </xsl:text><xsl:value-of select="."/>
                  <xsl:text> delete</xsl:text><xsl:text>&#13;&#10;</xsl:text>
               </xsl:otherwise>
            </xsl:choose>
      </xsl:for-each>
   </xsl:if><!-- Community List -->

   <xsl:if test="count(netml:List)>0"><!-- set the next hop or the 'install next hop' value for the route -->
      <xsl:for-each select="netml:List">
         <xsl:variable name="operation" select="@operation"/>
         <xsl:choose>
            <xsl:when test="$operation='next_hop'">
               <xsl:choose>
                  <xsl:when test="name(key('list', .)/child::*)='PrefixList'">
                     <xsl:for-each select="key('list',.)/netml:PrefixList/netml:prefix">
                        <xsl:text> set next-hop </xsl:text><xsl:value-of select="."/>
               <xsl:text>&#13;&#10;</xsl:text>
                     </xsl:for-each>
                  </xsl:when>
                  <xsl:otherwise>
                     <xsl:for-each select="key('list',.)/netml:RouteList/netml:routeFilter/netml:prefix">
                        <xsl:text> set next-hop </xsl:text><xsl:value-of select="."/>
               <xsl:text>&#13;&#10;</xsl:text>
                     </xsl:for-each>
                  </xsl:otherwise>
               </xsl:choose>
            </xsl:when>
            <xsl:when test="$operation='IP_next_hop'">
               <xsl:choose>
                  <xsl:when test="name(key('list', .)/child::*)='PrefixList'">
                     <xsl:for-each select="key('list',.)/netml:PrefixList/netml:prefix">
                        <xsl:text> set ip next-hop </xsl:text><xsl:value-of select="."/>
               <xsl:text>&#13;&#10;</xsl:text>
                     </xsl:for-each>
                  </xsl:when>
                  <xsl:otherwise>
                     <xsl:for-each select="key('list',.)/netml:RouteList/netml:routeFilter/netml:prefix">
                        <xsl:text> set ip next-hop </xsl:text><xsl:value-of select="."/>
               <xsl:text>&#13;&#10;</xsl:text>
                     </xsl:for-each>
                  </xsl:otherwise>
               </xsl:choose>
            </xsl:when>
            <xsl:otherwise><!-- $operation='install_next_hop' -->
               <xsl:message>Warning (install next hop): This set rule is not implemented in Zebra</xsl:message>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:for-each>
   </xsl:if><!-- List -->
   </xsl:template>
</xsl:stylesheet>
