<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0"
								xmlns:netml="http://www.xmlnetwork.org">

	<xsl:output method="text" />
	<xsl:strip-space elements="*"/>

	<!-- As the definitions of the templates for Policy and for List already exist we can include their files and use those definitions -->
	<!-- ###ProtocolIncludeStart###
	<xsl:include href="policy-cisco.xsl"/>
	<xsl:include href="list-cisco.xsl"/>
	###ProtocolIncludeEnd### -->

	<!-- ###RouterID### --><xsl:variable name="routerID">r01</xsl:variable><!-- ###RouterID### -->
	<xsl:key name="kRouter" match="netml:NetML/netml:Routers/netml:RouterConf" use="@id"/>
	<xsl:key name="kPolicy" match="netml:NetML/netml:Routers/netml:RouterConf[@id=string($routerID)]/netml:Policy" use="@name"/>
	<xsl:key name="kIfaceAddr" match="netml:NetML/netml:Routers/netml:RouterConf[@id=string($routerID)]/netml:Interface/netml:ipAddress" use="../@name"/>


	<xsl:template match="/">
		<xsl:apply-templates select="key('kRouter', string($routerID))/netml:RipConf"/>
	</xsl:template>

	<xsl:template match="netml:RipConf">
	<!-- Global variables -->
		<xsl:variable name="DEF_ROUTE_MAP_PREFIX">AUTO_</xsl:variable>
		<xsl:variable name="nl">&#13;&#10;</xsl:variable>
	<!-- RIPCONF-Cisco variables -->
		<xsl:variable name="DEF_TIMER_UPDATE">30</xsl:variable>
		<xsl:variable name="DEF_TIMER_INVALID">180</xsl:variable>
		<xsl:variable name="DEF_TIMER_HOLDDOWN">180</xsl:variable>
		<xsl:variable name="DEF_TIMER_FLUSH">240</xsl:variable>


		<xsl:text>// Start of ripconf</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		<xsl:text>router rip</xsl:text><xsl:text>&#13;&#10;</xsl:text>
		<xsl:apply-templates select="netml:neighbour" />
		<xsl:if test="count(netml:metric_offset)>0">
			<xsl:call-template name="metric_offset_list" >
				<xsl:with-param name="ancest">rip</xsl:with-param>
			</xsl:call-template>
		</xsl:if>
		<xsl:variable name="s_version" select="netml:send_version"/>
		<xsl:variable name="r_version" select="netml:receive_version"/>
		<xsl:if test="$r_version=$s_version">
			<xsl:call-template name="version"/>
		</xsl:if>
		<xsl:apply-templates select="netml:default_information_originate" /><!-- ok -->

	<!-- This block changes the default timers -->
		<xsl:variable name="t_update" select="./netml:timer_update" />
		<xsl:variable name="t_invalid" select="./netml:timer_invalid" />
		<xsl:variable name="t_garbage" select="./netml:timer_garbage"/>

		<xsl:if test="$t_update &gt; 0  or $t_invalid &gt; 0 or $t_garbage &gt; 0">
			<xsl:text>  timers basic </xsl:text>
			<xsl:choose>
				<xsl:when test="$t_update &gt; 0 "><xsl:value-of select="$t_update"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="$DEF_TIMER_UPDATE"/></xsl:otherwise>
			</xsl:choose>
			<xsl:text> </xsl:text>
			<xsl:choose>
				<xsl:when test="$t_invalid &gt; 0 "><xsl:value-of select="$t_invalid"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="$DEF_TIMER_INVALID"/></xsl:otherwise>
			</xsl:choose>
			<xsl:text> </xsl:text>
			<xsl:value-of select="$DEF_TIMER_HOLDDOWN"/>
			<xsl:text> </xsl:text>
			<xsl:choose>
				<xsl:when test="$t_garbage &gt; 0 "><xsl:value-of select="$t_garbage"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="$DEF_TIMER_FLUSH"/></xsl:otherwise>
			</xsl:choose>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
	<!-- End of default timers block-->


		<xsl:apply-templates select="netml:static_route" />
	<!-- NEW Redistribution Section -->
		<xsl:for-each select="netml:policy">
			<xsl:variable name="pRipName" select="@name"/>
			<xsl:variable name="pRedistr" select="count(key('kPolicy',$pRipName)/netml:Redistribution)"/>
			<xsl:variable name="pDistr" select="count(key('kPolicy',$pRipName)/netml:Distribution)"/>
			<xsl:variable name="pGen" select="count(key('kPolicy',$pRipName)/netml:General)"/>
			<xsl:choose>

				<!-- REDISTRIBUTION -->
				<xsl:when test="$pRedistr >0">
					<xsl:variable name="pInProto" select="key('kPolicy',$pRipName)/netml:Redistribution/netml:to_protocol"/>
					<xsl:variable name="pFromProto" select="key('kPolicy',$pRipName)/netml:Redistribution/netml:from_protocol"/>
					<!-- If redistribution references to a route-map it must be written here -->
					<xsl:if test="$pFromProto !='' and $pInProto='rip' ">
						<xsl:text>  redistribute </xsl:text><xsl:value-of select="$pFromProto"/><xsl:text> </xsl:text>
						<xsl:if test="count(key('kPolicy',$pRipName)//netml:term)>0" >
							<xsl:text> route-map </xsl:text>
							<!-- <xsl:value-of select="$DEF_ROUTE_MAP_PREFIX" /> -->
							<xsl:value-of select="$pRipName" />
						</xsl:if>
					</xsl:if>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:when>

				<!-- DISTRIBUTION -->
				<xsl:when test="$pDistr >0">
					<xsl:variable name="pInProto" select="key('kPolicy',$pRipName)"/>
					<xsl:text>  distribute-list </xsl:text>
					<xsl:value-of select="@list"/>
					<xsl:choose>
						<xsl:when test= "@target = 'import' ">
							<xsl:text> in </xsl:text>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text> out </xsl:text>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:when>

				<!-- GENERAL -->
				<xsl:when test="$pGen >0">
				</xsl:when>
			</xsl:choose>
		</xsl:for-each>
	<!-- End of NEW Policy Section -->
		<xsl:text>/// End of ripconf</xsl:text>
		<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>


	<xsl:template name="version">
		<xsl:text>  version </xsl:text>
		<xsl:choose>
			<xsl:when test="netml:send_version='1-2'">1-2 </xsl:when>
			<xsl:otherwise><xsl:value-of select="netml:send_version"/></xsl:otherwise>
		</xsl:choose>
		<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template name="metric_offset_list">
		<xsl:param name="ancest">rip</xsl:param>
		<xsl:choose>
			<xsl:when test="$ancest='rip'">
				<!-- creating the offset list -->
				<xsl:text>  offset-list RIP in </xsl:text><xsl:value-of select="netml:metric_offset"/>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
			<xsl:when test="$ancest='n_iface'">
				<!-- creation of the offset list -->
				<xsl:variable name="iface" select="netml:interface"/>
				<xsl:text>  offset-list N_IFACE_</xsl:text><xsl:value-of select="$iface"/><xsl:text> in </xsl:text><xsl:value-of select="netml:metric_offset"/>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<!-- creation of the offset list -->
				<xsl:text>  offset-list N_ADDR_</xsl:text><xsl:value-of select="netml:address"/><xsl:text> in </xsl:text><xsl:value-of select="netml:metric_offset"/>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="netml:default_information_originate">
		<xsl:if test=".='true'">
			<xsl:text>  default-information originate</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
	</xsl:template>

	<xsl:template match="netml:static_route"><!-- definizione delle rotte static tra le liste e una route-map che parla delle rip static; si ridistribuisce static qui. -->
		<xsl:text>  redistribute static route-map RIP_STATIC</xsl:text>
		<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template match="netml:neighbour">
		<xsl:variable name="interface_count" select="count(netml:interface)"/>
		<xsl:choose>
			<xsl:when test="$interface_count>0 ">
				<xsl:variable name="iface" select="netml:interface"/>
				<!--xsl:for-each select="/netml:addresses/netml:interface/netml:address">
					<xsl:if test="../@name=$iface"-->
				<xsl:for-each select="key('kIfaceAddr',string($iface))">
					<xsl:text>  network </xsl:text><xsl:value-of select="."/>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:for-each>
				<xsl:if test="count(netml:metric_offset)>0">
					<xsl:call-template name="metric_offset_list">
						<xsl:with-param name="ancest">n_iface</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="addr" select="netml:address"/>
				<xsl:text>  network </xsl:text><xsl:value-of select="$addr"/>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:if test="count(netml:metric_offset)>0">
					<xsl:call-template name="metric_offset_list">
						<xsl:with-param name="ancest">n_addr</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
</xsl:stylesheet>
