<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
               version="1.0"
							 xmlns:netml="http://www.xmlnetwork.org"
							 exclude-result-prefixes="netml">

	<xsl:output method="xml" indent="yes"/>

	<!-- ###RouterID### --><xsl:variable name="routerID">r01</xsl:variable><!-- ###RouterID### -->
	<xsl:key name="kRouter" match="netml:NetML/netml:Routers/netml:RouterConf" use="@id"/>
	<xsl:key name="kIface" match="netml:NetML/netml:Routers/netml:RouterConf[@id=string($routerID)]/netml:Interface" use="netml:ipAddress"/>

	<!--This translation file has been conceived with the supposition that before this module there will be a module which will transalte Network addresses into the related interfaces which will be used to enable Rip-->

	<xsl:template match="/">
		<configuration>
		<xsl:if test="count(key('kRouter',string($routerID))/netml:RipConf/netml:static_route)>0 or count(key('kRouter',string($routerID))/netml:RipConf/netml:default_information_originate)>0">
			<routing-options>
     	   <static>
		 	<!-- Rip Static Routes are being configured here -->
			<xsl:apply-templates select="key('kRouter',string($routerID))/netml:RipConf/netml:static_route"/>
		 	<!-- The default route is being configured here -->
			<xsl:apply-templates select="key('kRouter',string($routerID))/netml:RipConf/netml:default_information_originate"/>
  	      </static>
   	 </routing-options>
		</xsl:if>
		<xsl:if test="count(key('kRouter',string($routerID))/netml:RipConf)>0">
  	 <protocols>
		 	<!-- Rip Configuration starts here -->
		<xsl:apply-templates select="key('kRouter',string($routerID))/netml:RipConf"/>
			<!-- Rip Configuration finishes here -->
    	</protocols>
		</xsl:if>
		</configuration>
	</xsl:template>
	
	<xsl:template match="netml:RipConf">

	<!-- the export and the import variables defined below are used to separate the policies which are for importing (exporting) routes into the routing tables-->
	<xsl:variable name="export">exp<xsl:for-each select="netml:policy">
			<xsl:variable name="importBool"><xsl:value-of select="@type"/></xsl:variable>
			<xsl:if test="$importBool='export'">
				<xsl:text> , </xsl:text><xsl:value-of select="@name"/>
			</xsl:if>
		</xsl:for-each>
	</xsl:variable>
	<xsl:variable name="import">imp<xsl:for-each select="netml:policy">
			<xsl:variable name="importBool"><xsl:value-of select="@type"/></xsl:variable>
			<xsl:if test="$importBool='import'">
				<xsl:text> , </xsl:text><xsl:value-of select="@name"/>
			</xsl:if>
		</xsl:for-each>
	</xsl:variable>

  	 	 <rip>
				<!-- Group Configuration starts here -->
 				<group>
				<xsl:apply-templates select="netml:neighbour"/>
				<xsl:call-template name="netml:policy">
					<xsl:with-param name="ancest">group</xsl:with-param>
					<xsl:with-param name="imp_exp"><xsl:value-of select="$export"/></xsl:with-param>
				</xsl:call-template>
				</group>
				<!-- Group Configuration finishes here -->
				<xsl:apply-templates select="netml:send_version|netml:receive_version|netml:metric_offset"/>
				<xsl:apply-templates select="netml:split_horizon|netml:timer_update|netml:timer_garbage|netml:timer_invalid"/>
				<!-- Import policy configuration starts here -->
				<xsl:call-template name="netml:policy">
					<xsl:with-param name="ancest">rip</xsl:with-param>
					<xsl:with-param name="imp_exp"><xsl:value-of select="$import"/></xsl:with-param>
				</xsl:call-template>
				<!-- Import policy configuration finishes here -->
				<xsl:apply-templates select="netml:authentication"/>
     	 </rip>
	</xsl:template>

	<xsl:template match="netml:send_version">
<!-- the send version element can have only certain values. We have to detect which value has been choosen so as to put the right tag in the configuration-->
            <send>
						<xsl:variable name="send" select="."/>

						<xsl:choose>
							<xsl:when test="$send=none">
                <none/>
							</xsl:when>
							<xsl:when test="$send=1">
                <version-1/>
							</xsl:when>
							<xsl:when test="$send=2">
                <multicast/>
							</xsl:when>
							<xsl:otherwise>
								<broadcast/>
							</xsl:otherwise>
						</xsl:choose>

            </send>
	</xsl:template>

	<xsl:template match="netml:receive_version">
<!-- the receive version element can have only certain values. We have to detect which value has been choosen so as to put the right tag in the configuration-->
            <receive>
						<xsl:variable name="receive" select="."/>

						<xsl:choose>
							<xsl:when test="$receive=none">
                <none/>
							</xsl:when>
							<xsl:when test="$receive=1">
                <version-1/>
							</xsl:when>
							<xsl:when test="$receive=2">
                <version-2/>
							</xsl:when>
							<xsl:otherwise>
                <both/>
							</xsl:otherwise>
						</xsl:choose>

            </receive>

	</xsl:template>
	
		<xsl:template match="netml:metric_offset">
		<!-- the metric offset value is the value which will be added to incoming route's metric  -->
            <metric-in>
						<xsl:apply-templates/>
						</metric-in>

	</xsl:template>
	
	<xsl:template match="netml:default_information_originate">
	<!-- if we want to send the default route in the updates we have to set this to true and place the route in the forwarding tabk -->
	<xsl:if test=".='true'">
		<route>
			<name>0.0.0.0/0</name>
			<retain/>
		</route>
	</xsl:if>

	</xsl:template>

	<xsl:template match="netml:split_horizon">
<!-- split horizon gives the possibility to choose if we want it activated or not. In Juniper this is not possibile  -->
		<xsl:message> Warning (split horizon): It is not possibile to change the split horizon value for Juniper
routers. By default it is activated.
		</xsl:message>

	</xsl:template>

	<xsl:template match="netml:timer_update">
<!-- timer update is the timer which indicates how often updates should be sent. By default it is 30 secs -->
		<xsl:message>Warning (timer update): It is not possibile to change the timer values for Juniper. They use
the default values as requested by the rfc.
		</xsl:message>

	</xsl:template>

	<xsl:template match="netml:timer_garbage">
<!-- timer garbage is the timer which indicates after how much time a route is to be considered old (before it gets erased from the routing table) -->
		<xsl:message>Warning (timer garbage): It is not possibile to change the timer values for Juniper. They use
the default values as requested by the rfc.
		</xsl:message>

	</xsl:template>
	
	<xsl:template match="netml:timer_invalid">
<!-- timer invalid is the timer which indicates after how much time the "old" route will be eliminated from the routing table -->
		<xsl:message>Warning (timer invalid): It is not possibile to change the timer values for Juniper. They use
the default values as requested by the rfc.
		</xsl:message>

	</xsl:template>

	<xsl:template match="netml:static_route">
<!-- static routes can be defined specifically for RIP. For each static route a route element must be added -->
					<xsl:for-each select="netml:prefix">
            <route>
                <name><xsl:value-of select="."/></name>
                <retain/> <!-- this tag means that the static route will be kept in the routing table even if it is not active anymore -->
            </route>
					</xsl:for-each>

	</xsl:template>

	<xsl:template match="netml:neighbour">
<!-- each neighbour must be defined using the interface name-->
		<xsl:choose>
			<xsl:when test="count(netml:interface)>0">
								<neighbor>
        					<name>
				<xsl:value-of select="netml:interface"/>
							</name>
				<xsl:apply-templates select="netml:metric_offset|netml:send_version|netml:receive_version"/>
								</neighbor>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="prefix">
					<xsl:choose>
						<xsl:when test="substring-before(netml:address,'/')!=''">
							<xsl:value-of select="substring-before(netml:address,'/')"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="netml:address"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<xsl:variable name="prefixLength">
					<xsl:choose>
						<xsl:when test="substring-after(netml:address,'/')!=''">
							<xsl:value-of select="substring-after(netml:address,'/')"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="32"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<xsl:for-each select="../../netml:Interface">
					<xsl:for-each select="netml:ipAddress">
						<xsl:variable name="ifPrefix">
							<xsl:choose>
								<xsl:when test="substring-before(.,'/')!=''">
									<xsl:value-of select="substring-before(.,'/')"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="."/>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<xsl:variable name="contained">
							<xsl:call-template name="AddrContainment">
								<xsl:with-param name="neighAddr" select="$prefix"/>
								<xsl:with-param name="prefixLength" select="$prefixLength"/>
								<xsl:with-param name="ifaceAddr" select="$ifPrefix"/>
							</xsl:call-template>
						</xsl:variable>
						<xsl:if test="$contained='true'">
								<neighbor>
        					<name>
							<xsl:value-of select="../@name"/>
							</name>
							<xsl:apply-templates select="netml:metric_offset|netml:send_version|netml:receive_version"/>
								</neighbor>
						</xsl:if>
					</xsl:for-each>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="AddrContainment">
		<xsl:param name="neighAddr"/>
		<xsl:param name="prefixLength"/>
		<xsl:param name="ifaceAddr"/>
		<xsl:variable name="true">
			<xsl:text>true</xsl:text>
		</xsl:variable>
		<xsl:variable name="false">
			<xsl:text>false</xsl:text>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="$prefixLength='32'">
				<xsl:if test="contains($ifaceAddr,$neighAddr)">
					<xsl:value-of select="$true"/>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="NeighNumb">
					<xsl:call-template name="Prefix2Number">
						<xsl:with-param name="prefix" select="$neighAddr"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:variable name="IfaceNumb">
					<xsl:call-template name="Prefix2Number">
						<xsl:with-param name="prefix" select="$ifaceAddr"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:variable name="PrefLInteger">
					<xsl:call-template name="PrefL2Number">
						<xsl:with-param name="prefL" select="$prefixLength"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:choose>
					<xsl:when test="($NeighNumb - $IfaceNumb>number( - string($PrefLInteger))) and ($NeighNumb - $IfaceNumb&lt;$PrefLInteger)">
						<xsl:value-of select="$true"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$false"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="Prefix2Number">
		<xsl:param name="prefix"/>
		<xsl:variable name="tmp1" select="substring-before($prefix,'.')"/>
		<xsl:variable name="tmp2" select="substring-before(substring-after($prefix,'.'),'.')"/>
		<xsl:variable name="tmp3" select="substring-before(substring-after(substring-after($prefix,'.'),'.'),'.')"/>
		<xsl:variable name="tmp4" select="substring-after(substring-after(substring-after($prefix,'.'),'.'),'.')"/>

		<xsl:variable name="var1" select="$tmp1*256*256*256"/>
		<xsl:variable name="var2" select="$tmp2*256*256"/>
		<xsl:variable name="var3" select="$tmp3*256"/>
		<xsl:variable name="var4" select="$tmp4"/>

		<xsl:value-of select="$var1+$var2+$var3+$var4"/>
	</xsl:template>

	<xsl:template name="PrefL2Number">
		<xsl:param name="prefL"/>
		<xsl:param name="base">2</xsl:param>

		<xsl:choose>
			<xsl:when test="$prefL=0">
				<xsl:value-of select="1"/>
			</xsl:when>
			<xsl:when test="$prefL=1">
				<xsl:value-of select="$base"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="tmp">
					<xsl:call-template name="PrefL2Number">
						<xsl:with-param name="prefL" select="$prefL - 1"/>
						<xsl:with-param name="base" select="$base"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:value-of select="$tmp*$base"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="netml:policy">
		<xsl:param name="ancest"/> <!-- indicates if the ancestor tag (which called the template) was RipConf or Group -->
		<xsl:param name="imp_exp"/><!-- the list of import (or export) policies -->
	<!-- the policies are of two types: import and export. Rip level policies are (in Juniper) import policies whilst Group level policies are export policies. -->
	<xsl:choose>
		<xsl:when test="$ancest='rip'"> <!-- if RIP called the template then we have to list the import policies -->
			<xsl:variable name="imp"><xsl:if test="starts-with($imp_exp,'imp')"><!-- the import and export strings start with imp and exp respectively. we need to remove these substrings -->
				<xsl:value-of select="substring-after($imp_exp,' , ')"/></xsl:if></xsl:variable>
			<xsl:if test="$imp!=''"><!-- the list was in one variable which is a string so we must divide it into substrings -->
				<import>
				<xsl:call-template name="substring_man"><!-- we call this "function" to divide the import string into substrings and to write the name of the policies -->
					<xsl:with-param name="substr"><xsl:value-of select="$imp"/></xsl:with-param>
				</xsl:call-template>
				</import>
			</xsl:if>
		</xsl:when>
		<xsl:otherwise><!-- the template was called with ancest="group" -->
			<xsl:variable name="exp"><xsl:if test="starts-with($imp_exp,'exp')"><!-- the import and export strings start with imp and exp respectively. we need to remove these substrings -->
				<xsl:value-of select="substring-after($imp_exp,' , ')"/></xsl:if></xsl:variable>
			<xsl:if test="$exp!=''">
				<export>
				<xsl:call-template name="substring_man"><!-- we call this "function" to divide the export string into substrings and to write the name of the policies -->
					<xsl:with-param name="substr"><xsl:value-of select="$exp"/></xsl:with-param>
				</xsl:call-template>
				</export>
			</xsl:if>
		</xsl:otherwise>
	</xsl:choose>
	</xsl:template>
	
	<xsl:template name="substring_man"><!-- the function which separates the variables and writes the policy names-->
		<xsl:param name="substr"/>
			<xsl:variable name="before" select="substring-before($substr,' , ')"/><!-- if there is no ' , ' in substr this variable='' -->
			<xsl:variable name="after" select="substring-after($substr,' , ')"/><!-- if there is no ' , ' in substr this variable='' -->
			<xsl:choose>
				<xsl:when test="$before='' and $after=''"><!-- if the test is true then this means we have reached the last element of the list -->
					<name><xsl:value-of select="$substr"/></name>
				</xsl:when>
				<xsl:otherwise><!-- this is not the last element -->
					<xsl:if test="not(contains($before,$after))"><!--as the policy names could be repeated we do not want to write them more than once -->
						<name><xsl:value-of select="$before"/></name>
					</xsl:if>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:if test="$after!=''"><!-- if this was not the last element we want to recall the function with the rest of the list -->
				<xsl:call-template name="substring_man">
					<xsl:with-param name="substr"><xsl:value-of select="$after"/></xsl:with-param>
				</xsl:call-template>
			</xsl:if>
	</xsl:template>

	<xsl:template match="netml:authentication">
	<!-- authentication defines if we want to "encrypt" our updates and make our neighbours check that we sent the update (not someone else) -->
						<authentication-type><xsl:value-of select="netml:type"/></authentication-type>
            <authentication-key>
						<xsl:apply-templates select="netml:key|netml:key_chain"/>
						</authentication-key>

	</xsl:template>

	<xsl:template match="netml:key"><!-- the authentication key -->
			<xsl:value-of select="."/>
	</xsl:template>
	
	<xsl:template match="netml:key_chain"><!-- the authenication key could be kept in a chian. We choose the first one in this case -->
			<xsl:value-of select="./netml:key_list/netml:key"/>
	</xsl:template>

</xsl:stylesheet>
