<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0"
								xmlns:netml="http://www.xmlnetwork.org">

	<xsl:output method="text" />
	<xsl:strip-space elements="*"/>

	<!-- ###ProtocolIncludeStart###
	<xsl:include href="policy-zebra.xsl"/>
	<xsl:include href="list-zebra.xsl"/>
	###ProtocolIncludeEnd### -->

	<!-- ###RouterID### --><xsl:variable name="routerID">r01</xsl:variable><!-- ###RouterID### -->
	<xsl:key name="kRouter" match="netml:NetML/netml:Routers/netml:RouterConf" use="@id"/>
	<xsl:key name="kPolicy" match="netml:NetML/netml:Routers/netml:RouterConf[@id=string($routerID)]/netml:Policy" use="@name"/>
	
	<xsl:template match="/">
<!-- ###Policy-List-Start###
		<xsl:if test="count(key('kRouter',string($routerID))//netml:RipConf ) >0 ">
      <xsl:text> password zebra &#10;</xsl:text>
		<xsl:if test="count(key('kRouter',string($routerID))/netml:Policy)>0">
			<xsl:text>!Start of Policy Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:apply-templates select="key('kRouter',string($routerID))/netml:Policy/netml:Redistribution//netml:term | key('kRouter',string($routerID))/netml:Policy/netml:General//netml:term"/>
			<xsl:text>!End of Policy Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(key('kRouter',string($routerID))/netml:List)>0 or 
         count(key('kRouter',string($routerID))//netml:metric_offset)>0 or
         count(key('kRouter',string($routerID))//netml:key_chain)>0">
			<xsl:text>!Start of Access or Prefix List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:PrefixList)>0">
				<xsl:apply-templates select="key('kRouter',string($routerID))/netml:List/PrefixList"/>
				<xsl:text>!&#13;&#10;</xsl:text>
			</xsl:if>
			<xsl:if test="count(key('kRouter',string($routerID))//netml:metric_offset)>0">
				<xsl:text>!Creating the access lists which concern the metric offset value from RipConf or from the neighbour configurations</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
				<xsl:apply-templates select="key('kRouter',string($routerID))//netml:metric_offset"/>
				<xsl:text>!End of access lists concerning RIPCONF</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
			</xsl:if>
			<xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:RouteList)>0">
				<xsl:text>!Start of Prefix List Configuration</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
				<xsl:apply-templates select="key('kRouter',string($routerID))/netml:List/netml:RouteList"/>
				<xsl:text>!End of Prefix List Configuration</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
			</xsl:if>
			<xsl:if test="count(key('kRouter',string($routerID))//netml:key_chain)>0">
				<xsl:text>!Creating any key chains needed for authenticating</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
				<xsl:apply-templates select="key('kRouter',string($routerID))//netml:key_chain"/>
				<xsl:text>!End of key chain creation</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
			</xsl:if>
			<xsl:text>!End of Access or Prefix List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
		</xsl:if>
		</xsl:if>
 ###Policy-List-End### -->

		<xsl:apply-templates select="key('kRouter',string($routerID))/netml:RipConf"/>
	</xsl:template>
	
	<xsl:template match="netml:RipConf">
	
	<!-- Global variables -->
	<xsl:variable name="DEF_ROUTE_MAP_PREFIX">AUTO_</xsl:variable>

	<!-- RIPCONF-Zebra variables -->
	<xsl:variable name="DEF_TIMER_UPDATE">30</xsl:variable>
	<xsl:variable name="DEF_TIMER_TIMEOUT">180</xsl:variable>
	<xsl:variable name="DEF_TIMER_GARBAGE">120</xsl:variable>


	<xsl:text>! Start of ripconf</xsl:text><xsl:text>&#13;&#10;</xsl:text>
	<xsl:text>router rip</xsl:text><xsl:text>&#13;&#10;</xsl:text>
	<xsl:apply-templates select="netml:static_route|netml:neighbour" />
	<xsl:if test="count(netml:metric_offset)>0">
		<xsl:call-template name="metric_offset_list" >
			<xsl:with-param name="ancest">rip</xsl:with-param>
		</xsl:call-template>
	</xsl:if>
	<xsl:variable name="s_version" select="netml:send_version"/>
	<xsl:variable name="r_version" select="netml:receive_version"/>
	<xsl:if test="$r_version=$s_version">
		<xsl:call-template name="version"/>
	</xsl:if>
	<xsl:apply-templates select="netml:default_information_originate" />

	<!-- This block changes the default timers -->
	<xsl:variable name="t_update" select="./netml:timer_update" />
	<xsl:variable name="t_invalid" select="./netml:timer_invalid" />
	<xsl:variable name="t_garbage" select="./netml:timer_garbage"/>

	<xsl:if test="$t_update >0  or $t_invalid > 0 or $t_garbage > 0">
		<xsl:text>  timers basic </xsl:text>
		<xsl:choose>
			<xsl:when test="$t_update > 0 "><xsl:value-of select="$t_update"/></xsl:when>
			<xsl:otherwise><xsl:value-of select="$DEF_TIMER_UPDATE"/></xsl:otherwise>
		</xsl:choose>
		<xsl:text> </xsl:text>
		<xsl:choose>
			<xsl:when test="$t_invalid > 0 "><xsl:value-of select="$t_invalid"/></xsl:when>
			<xsl:otherwise><xsl:value-of select="$DEF_TIMER_TIMEOUT"/></xsl:otherwise>
		</xsl:choose>
		<xsl:text> </xsl:text>
		<xsl:choose>
			<xsl:when test="$t_garbage > 0 "><xsl:value-of select="$t_garbage"/></xsl:when>
			<xsl:otherwise><xsl:value-of select="$DEF_TIMER_GARBAGE"/></xsl:otherwise>
		</xsl:choose>
		<xsl:text>&#13;&#10;</xsl:text>
	</xsl:if>
	<!-- End of default timers block-->


	<!-- NEW Redistribution Section -->
	<xsl:for-each select="netml:policy">
		<xsl:variable name="pRipName" select="@name"/>
		<xsl:variable name="pRedistr" select="count(key('kPolicy',$pRipName)/netml:Redistribution)"/>
		<xsl:variable name="pDistr" select="count(key('kPolicy',$pRipName)/netml:Distribution)"/>
		<xsl:variable name="pGen" select="count(key('kPolicy',$pRipName)/netml:General)"/>
		<xsl:choose>

				<!-- REDISTRIBUTION -->
				<xsl:when test="$pRedistr >0">
					<xsl:variable name="pInProto" select="key('kPolicy',$pRipName)/netml:Redistribution/netml:to_protocol"/>
					<xsl:variable name="pFromProto" select="key('kPolicy',$pRipName)/netml:Redistribution/netml:from_protocol"/>
					<!-- If redistribution references a route-map it must be written here -->
					<xsl:if test="$pFromProto !='' and $pInProto='rip' ">
						<xsl:text>  redistribute </xsl:text><xsl:value-of select="$pFromProto"/><xsl:text> </xsl:text>
						<xsl:if test="count(key('kPolicy',$pRipName)//netml:term)>0" >
							<xsl:text> route-map </xsl:text>
							<!-- <xsl:value-of select="$DEF_ROUTE_MAP_PREFIX" /> -->
							<xsl:value-of select="$pRipName" />
						</xsl:if>
					</xsl:if>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:when>

				<!-- DISTRIBUTION -->
				<xsl:when test="$pDistr >0">
					<xsl:text>  distribute-list </xsl:text>
					<xsl:value-of select="key('kPolicy',$pRipName)/netml:Distribution/netml:list"/>
					<xsl:choose>
						<xsl:when test= "key('kPolicy',$pRipName)/netml:Distribution/@target = 'import' ">
							<xsl:text> in </xsl:text>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text> out </xsl:text>
						</xsl:otherwise>
					</xsl:choose>
				<xsl:if test="count(key('kPolicy',$pRipName)/netml:Distribution/netml:interface)>0">
					<xsl:value-of select="key('kPolicy',$pRipName)/netml:Distribution/netml:interface"/>
				</xsl:if>
				<xsl:text>&#13;&#10;</xsl:text>
				</xsl:when>

				<!-- GENERAL -->
				<xsl:when test="$pGen >0">
               <xsl:message> Warning (General Policies): General Policies are not supported by Zebra.</xsl:message>
				</xsl:when>
		</xsl:choose>
	</xsl:for-each>

	<!-- End of NEW Redistribution Section -->
<xsl:text>!End of ripconf</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
	</xsl:template>

	<xsl:template name="version">
		<xsl:text>  version </xsl:text>
		<xsl:choose>
			<xsl:when test="netml:send_version='1-2'">1-2 </xsl:when>
			<xsl:otherwise><xsl:value-of select="netml:send_version"/></xsl:otherwise>
		</xsl:choose>
				<xsl:text>&#13;&#10;</xsl:text>
	</xsl:template>


	<xsl:template name="metric_offset_list">
	<xsl:param name="ancest">rip</xsl:param>
		<xsl:choose>
			<xsl:when test="$ancest='rip'">
				<!-- creating the offset list -->
				<xsl:text>  offset-list RIP in </xsl:text><xsl:value-of select="netml:metric_offset"/>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
			<xsl:when test="$ancest='n_iface'">
				<!-- creation of the offset list -->
				<xsl:variable name="iface" select="netml:interface"/>
						<xsl:text>  offset-list N_IFACE_</xsl:text><xsl:value-of select="$iface"/>
                  <xsl:text> in </xsl:text><xsl:value-of select="netml:metric_offset"/>
						<xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<!-- creation of the offset list -->
				<xsl:text>  offset-list N_ADDR_</xsl:text><xsl:value-of select="netml:address"/>
            <xsl:text> in </xsl:text><xsl:value-of select="netml:metric_offset"/>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="netml:default_information_originate">
		<xsl:if test=".='true'">
		<xsl:text>  default-information originate</xsl:text>
		<xsl:text>&#13;&#10;</xsl:text>
		</xsl:if>
	</xsl:template>

	<xsl:template match="netml:static_route">
		<xsl:for-each select="netml:prefix">
			<xsl:text>  route </xsl:text><xsl:value-of select="."/>
			<xsl:text>&#13;&#10;</xsl:text>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template match="netml:neighbour">
		<xsl:variable name="interface_count" select="count(netml:interface)"/>
		<xsl:choose>
			<xsl:when test="$interface_count>0 ">
				<xsl:text>  network </xsl:text><xsl:value-of select="netml:interface"/>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:if test="count(netml:metric_offset)>0">
					<xsl:call-template name="metric_offset_list">
						<xsl:with-param name="ancest">n_iface</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
			<xsl:variable name="addr" select="netml:address"/>
				<xsl:text>  network </xsl:text><xsl:value-of select="$addr"/>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:if test="count(netml:metric_offset)>0">
					<xsl:call-template name="metric_offset_list">
						<xsl:with-param name="ancest">n_addr</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	</xsl:stylesheet>
