<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0"
								xmlns:netml="http://www.xmlnetwork.org">

	<xsl:output method="text" />
	<xsl:strip-space elements="*"/>

	<!-- As the definitions of the templates for Policy and for RipConf already 
exist we can include their files and use those definitions -->
	<xsl:include href="policy-cisco.xsl"/>
	<xsl:include href="ripconf-cisco.xsl"/>
	<xsl:include href="list-cisco.xsl"/>
	<xsl:include href="interface-cisco.xsl"/>
	<xsl:include href="bgp-cisco.xsl"/>

	<!-- ###RouterID### --><xsl:variable name="routerID">r_100_1</xsl:variable><!-- ###RouterID### -->
	<xsl:key name="kRouter" match="netml:NetML/netml:Routers/netml:RouterConf" 
use="@id"/>

	<xsl:template match="/">
		<xsl:if test="count(key('kRouter',string($routerID))/netml:Policy)>0">
			<xsl:text>!Start of Policy Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:Policy/netml:Redistribution//netml:term | key('kRouter',string($routerID))/netml:Policy/netml:General//netml:term"/>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:text>!End of Policy Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(key('kRouter',string($routerID))/netml:List)>0 or 
count(key('kRouter',string($routerID))/netml:RipConf/netml:static_route)>0 or 
count(key('kRouter',string($routerID))//netml:metric_offset)>0 or 
count(key('kRouter',string($routerID))//netml:key_chain)>0 or 
count(key('kRouter',string($routerID))/netml:RipConf/netml:authentication/netml:key)>0 or count(key('kRouter',string($routerID))/netml:List/netml:ASPathList)>0 or count(key('kRouter',string($routerID))/netml:List/netml:ComunityList)>0 or count(key('kRouter',string($routerID))/netml:List/netml:RouteList)>0">
			<xsl:text>!Start of Access or Prefix List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:if 
test="count(key('kRouter',string($routerID))/netml:List/netml:PrefixList)>0">
				<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:List/netml:PrefixList"/>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
			</xsl:if>
			<xsl:if 
test="count(key('kRouter',string($routerID))/netml:RipConf/netml:static_route)>0">
				<xsl:text>!Creating an access-list for Rip's Static Routes</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
				<xsl:text>ip access list standard RIP_STATIC </xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:call-template name="static_route_list"/>
				<xsl:text>!End of Static Route section</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
			</xsl:if>
			<xsl:if 
test="count(key('kRouter',string($routerID))//netml:metric_offset)>0">
				<xsl:text>!Creating the access lists which concern the metric offset value 
from RipConf or from the neighbour configurations</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
				<xsl:apply-templates 
select="key('kRouter',string($routerID))//netml:metric_offset"/>
				<xsl:text>!End of access lists concerning RIPCONF</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
			</xsl:if>
			<xsl:if 
test="count(key('kRouter',string($routerID))/netml:List/netml:RouteList)>0">
				<xsl:text>!Start of Prefix List Configuration</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
				<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:List/netml:RouteList"/>
				<xsl:text>!End of Prefix List Configuration</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
			</xsl:if>
			<xsl:variable name="countCommList" 
select="count(key('kRouter',string($routerID))/netml:List/netml:ComunityList)"/>
			<xsl:if test="$countCommList>0">
				<xsl:text>!Start of Community List Configuration</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
				<xsl:variable name="CommListNames">
					<xsl:for-each 
select="key('kRouter',string($routerID))/netml:List/netml:ComunityList">
						<xsl:value-of select="../netml:id"/><xsl:text> ; </xsl:text>
					</xsl:for-each>
				</xsl:variable>
				<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:List/netml:CommunityList">
					<xsl:with-param name="nameList" select="$CommListNames"/>
				</xsl:apply-templates>
				<xsl:text>!&#13;&#10;</xsl:text>
				<xsl:text>!End of Community List Configuration</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
			</xsl:if>
			<xsl:if
test="count(key('kRouter',string($routerID))/netml:List/netml:ASPathList)>0">
				<xsl:text>!Start of AS Path List Configuration</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
				<xsl:variable name="ASPathListNames">
					<xsl:for-each 
select="key('kRouter',string($routerID))/netml:List/netml:ASPathList">
						<xsl:value-of select="../netml:id"/><xsl:text> ; </xsl:text>
					</xsl:for-each>
				</xsl:variable>
				<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:List/netml:ASPathList">
					<xsl:with-param name="nameList" select="$ASPathListNames"/>
				</xsl:apply-templates>
				<xsl:text>!End of AS Path List Configuration</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
			</xsl:if>
			<xsl:if test="count(key('kRouter',string($routerID))//netml:key_chain)>0 or 
count(key('kRouter',string($routerID))/netml:RipConf/netml:authentication/netml:key)>0">
				<xsl:text>!Creating any key chains needed for authenticating</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
				<xsl:apply-templates 
select="key('kRouter',string($routerID))//netml:key_chain"/>
				<xsl:variable name="count" 
select="count(key('kRouter',string($routerID))/netml:RipConf/netml:authentication/netml:key)"/>
				<xsl:if test="$count>0">
					<xsl:call-template name="key_strings">
						<xsl:with-param name="key_count" select="$count"/>
					</xsl:call-template>
				</xsl:if>
				<xsl:text>!&#13;&#10;</xsl:text>
				<xsl:text>!End of key chain creation</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>!&#13;&#10;</xsl:text>
			</xsl:if>
			<xsl:text>!End of Access or Prefix List Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
		</xsl:if>
		<!-- This part was constructed assuming that a tool connected to the router, 
got the existing information about the interfaces and constructed an extra bit 
of the xml
		<xsl:variable name="ifaces"><!- lista delle interfacce che Rip utilizza per 
definire i neighbour->
			<xsl:for-each 
select="key('kRouter',string($routerID))/netml:RipConf/netml:neighbour/netml:interface">
				<xsl:value-of select="."/><xsl:text> ; </xsl:text>
			</xsl:for-each>
		</xsl:variable><!- fine $ifaces ->
		<xsl:variable name="addrIfacesTmp"><!- lista delle interfacce che Rip richiama 

attraverso gli indirizzi ip->
			<xsl:for-each 
select="key('kRouter',string($routerID))/netml:RipConf/netml:neighbour/netml:address">
				<xsl:variable name="addr" select="."/>
				<xsl:for-each select="/netml:addresses/netml:interface/netml:address">
					<xsl:if test=".=$addr">
						<xsl:value-of select="."/><xsl:text> / </xsl:text><xsl:value-of 
select="../@name"/><xsl:text> ; </xsl:text>
					</xsl:if>
				</xsl:for-each>
			</xsl:for-each>
		</xsl:variable><!- fine $addrIfaces ->
		<!- it is important to check that an interface present in ifaces is not 
present in addrIfaces. This can be done rewriting addrIfaces ->
		<xsl:variable name="addrIfaces">
			<xsl:call-template name="comparisonIfacesAddrIfaces"><!- checks that 
interfaces in addrIfaces are not present in addrIfaces ->
				<xsl:with-param name="ifaces" select="$ifaces"/>
				<xsl:with-param name="addrIfaces" select="$addrIfacesTmp"/>
			</xsl:call-template>
		</xsl:variable> -->
		<xsl:if test="count(key('kRouter',string($routerID))/netml:Interface)>0">
			<xsl:text>!Start of Interface Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:Interface">
				<!-- These parameters were used when considering the list of existing 
intefaces
				<xsl:with-param name="ifaces" select="$ifaces"/>
				<xsl:with-param name="addrIfaces" select="$addrIfaces"/> -->
			</xsl:apply-templates>
			<xsl:text>!End of Interface Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(key('kRouter',string($routerID))/netml:RipConf)>0">
			<xsl:text>!Start of Rip Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:RipConf"/>
			<xsl:text>!End of Rip Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
		</xsl:if>
		<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf)>0">
			<xsl:text>!Start of BGP Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
			<xsl:choose>
				<xsl:when test="count(key('kRouter',string($routerID))/netml:BGPConf)=1">
					<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:BGPConf"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:message>Warning: in Cisco it is not possible to have more than one
configuration for BGP on the same router.
					 The first view will be transformed into the BGP 
Configuration.</xsl:message>
					<xsl:apply-templates
select="key('kRouter',string($routerID))/netml:BGPConf[position()=1]"/>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:text>!End of BGP Configuration</xsl:text>
			<xsl:text>&#13;&#10;</xsl:text>
			<xsl:text>!&#13;&#10;</xsl:text>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>
