<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                           version="1.0"
                           xmlns:netml="http://www.xmlnetwork.org"
                           exclude-result-prefixes="netml">

   <xsl:output method="text" indent="no"/>

   <!-- As the definitions of the templates for Policy and for RipConf already exist we can include their files and use those definitions -->
   <xsl:include href="policy-juniper-text.xsl"/>
   <xsl:include href="ripconf-juniper-text.xsl"/>
   <xsl:include href="bgp-juniper-text.xsl"/>

   <!-- ###RouterID### --><xsl:variable name="routerID">r01</xsl:variable><!-- ###RouterID### -->
   <xsl:key name="kRouter" match="netml:NetML/netml:Routers/netml:RouterConf" use="@id"/>
   <xsl:key name="kIface" match="netml:NetML/netml:Routers/netml:RouterConf[@id=string($routerID)]/netml:Interface/netml:ipAddress" use="../@name"/>
   <xsl:key name="kConfederation" match="netml:NetML/netml:AS-List/netml:Confederations/netml:C" use="netml:ASMember/@id"/>


   <xsl:template match="/">
   <!-- Configuring the router -->
      <xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:PrefixList)>0 or count(key('kRouter',string($routerID))/netml:List/netml:ASPathList)>0 or count(key('kRouter',string($routerID))/netml:List/netml:CommunityList)>0 or count(key('kRouter',string($routerID))/netml:Policy)>0 or count(key('kRouter',string($routerID))/netml:BGPConf//netml:Next-Hop-Self)>0 or count(key('kRouter',string($routerID))/netml:BGPConf//netml:List)>0 or count(key('kRouter',string($routerID))/netml:BGPConf/netml:Dampening)>0">
policy-options {
         <xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:PrefixList)>0">
            <!-- Configuring Prefix Lists -->
            <xsl:apply-templates select="key('kRouter',string($routerID))//netml:PrefixList"/>
         </xsl:if>
         <xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:ASPathList)>0">
            <xsl:apply-templates select="key('kRouter',string($routerID))/netmlList/netml:ASPathList"/>
         </xsl:if>
         <xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:CommunityList)>0">
            <xsl:apply-templates select="key('kRouter',string($routerID))/netml:List/netml:CommunityList"/>
         </xsl:if>
         <xsl:if test="count(key('kRouter',string($routerID))/netml:Policy)>0">
            <!-- Configuring Policies -->
            <xsl:apply-templates select="key('kRouter',string($routerID))/netml:Policy"/>
         </xsl:if>
         <xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:Next-Hop-Self)>0">
            <xsl:call-template name="netml:Next-Hop-Self"/>
         </xsl:if>
         <xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:List)>0">
            <xsl:apply-templates select="key('kRouter',string($routerID))/netml:BGPConf//netml:List"/>
         </xsl:if>
         <xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:Dampening)>0">
            <xsl:apply-templates select="key('kRouter',string($routerID))/netml:BGPConf/netml:Dampening"/>
         </xsl:if>
}
		</xsl:if>
		<xsl:if test="count(key('kRouter',string($routerID))/netml:Interface)>0">
		<!-- Configuring Interfaces -->
interfaces {
	<xsl:apply-templates select="key('kRouter',string($routerID))/netml:Interface"/>
}
		</xsl:if>
		<xsl:choose>
			<xsl:when test="count(key('kRouter',string($routerID))//netml:BGPConf/netml:view)>1">
				<xsl:text>    routing-instances {</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>        </xsl:text><xsl:value-of select="key('kRouter',string($routerID))/netml:BGPConf/netml:view"/><xsl:text> {</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>            instance-type vrf;</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
					<xsl:variable name="localAS" select="key('kRouter',string($routerID))/netml:BGPConf/@as"/>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf)>0">
					<xsl:text>            routing-options {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					<xsl:text>                autonomous-system </xsl:text><xsl:value-of select="$localAS"/>
					<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:RouterId)>0">
					<xsl:text>                router-id </xsl:text><xsl:value-of select="key('kRouter',string($routerID))/netml:BGPConf/netml:RouterId"/>
					<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:if test="count(key('kConfederation',string($localAS)))>0">
					<xsl:for-each select="key('kConfederation',string($localAS))">
						<xsl:text>                confederation </xsl:text><xsl:value-of select="@id"/>
						<xsl:text> members [ </xsl:text>
						<xsl:for-each select="netml:ASMember">
							<xsl:value-of select="@id"/><xsl:text> </xsl:text>
						</xsl:for-each>
						<xsl:text>];</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:for-each>
				</xsl:if>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:Default-Originate)>0 or count(key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes)>0 or count(key('kRouter',string($routerID))/netml:RipConf/netml:static_route)>0 or count(key('kRouter',string($routerID))/netml:RipConf/netml:default_information_originate)>0">
					<xsl:text>                static {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:RipConf/netml:static_route)>0">
						<!-- Rip Static Routes are being configured here -->
						<xsl:apply-templates select="key('kRouter',string($routerID))/netml:RipConf/netml:static_route"/>
					</xsl:if>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:RipConf/netml:default_information_originate)>0">
						<!-- The default route is being configured here -->
						<xsl:apply-templates select="key('kRouter',string($routerID))/netml:RipConf/netml:default_information_originate"/>
					</xsl:if>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:Default-Originate)>0">
						<xsl:text>                     route default {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                           next-hop self; </xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                           retain;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                     }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:if>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes)>0">
						<xsl:for-each select="key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes/netml:s">
							<xsl:text>                     route </xsl:text><xsl:value-of select="@address"/>
							<xsl:text> {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							<xsl:text>                           next-hop self;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							<xsl:text>                           retain;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							<xsl:text>                     }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						</xsl:for-each>
					</xsl:if>
					<xsl:text>                }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:AggregateRoutes)>0">
					<xsl:text>                aggregate {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					<xsl:for-each select="key('kRouter',string($routerID))/netml:BGPConf/netml:AggregateRoutes/netml:route">
						<xsl:text>                    route </xsl:text><xsl:value-of select="@address"/>
						<xsl:text> {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                          next-hop self;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:if test="count(netml:advertise)>0 or count(netml:attribute)>0">
							<xsl:text>                          policy [ </xsl:text>
							<xsl:if test="count(netml:advertise)>0">
								<xsl:value-of select="netml:advertise"/><xsl:text> </xsl:text>
							</xsl:if>
							<xsl:if test="count(netml:attribute)>0">
								<xsl:value-of select="netml:attribute"/>
							</xsl:if>
							<xsl:text> ];</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						</xsl:if>
						<xsl:choose>
							<xsl:when test="@as-set='true'">
								<xsl:text>                          full;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							</xsl:when>
							<xsl:otherwise>
								<xsl:text>                          brief;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							</xsl:otherwise>
						</xsl:choose>
						<xsl:text>                          </xsl:text><xsl:value-of select="@type"/>
						<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                    </xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:for-each>
					<xsl:text>                }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:text>            }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf)>0 or count(key('kRouter',string($routerID))/netml:RipConf)>0">
					<xsl:text>            protocols {</xsl:text>
					<xsl:text>&#13;&#10;</xsl:text>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:RipConf)>0">
						<!-- Rip Configuration starts here -->
						<xsl:apply-templates select="key('kRouter',string($routerID))/netml:RipConf"/>
						<!-- Rip Configuration finishes here -->
					</xsl:if>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf)>0 or count(key('kRouter',string($routerID))/netml:RipConf)>0">
						<xsl:text>                bgp {</xsl:text>
						<xsl:text>&#13;&#10;</xsl:text>
						<xsl:apply-templates select="key('kRouter',string($routerID))/netml:BGPConf"/>
						<xsl:text>                }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>&#13;&#10;</xsl:text>
					</xsl:if>
					<xsl:text>            }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>        }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>    }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:text>&#13;&#10;</xsl:text>
			</xsl:when>
			<xsl:when test="count(key('kRouter',string($routerID))//netml:BGPConf/netml:view)=1 or count(key('kRouter',string($routerID))/netml:RipConf)>0">
					<xsl:variable name="localAS" select="key('kRouter',string($routerID))/netml:BGPConf/@as"/>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf)>0">
					<xsl:text>routing-options {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					<xsl:text>     autonomous-system </xsl:text><xsl:value-of select="$localAS"/>
					<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:RouterId)>0">
					<xsl:text>     router-id </xsl:text><xsl:value-of select="key('kRouter',string($routerID))/netml:BGPConf/netml:RouterId"/>
					<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:if test="count(key('kConfederation',string($localAS)))>0">
					<xsl:for-each select="key('kConfederation',string($localAS))">
						<xsl:text>     confederation </xsl:text><xsl:value-of select="@id"/>
						<xsl:text> members [ </xsl:text>
						<xsl:for-each select="netml:ASMember">
							<xsl:value-of select="@id"/><xsl:text> </xsl:text>
						</xsl:for-each>
						<xsl:text>];</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:for-each>
				</xsl:if>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:Default-Originate)>0 or count(key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes)>0 or count(key('kRouter',string($routerID))/netml:RipConf/netml:static_route)>0 or count(key('kRouter',string($routerID))/netml:RipConf/netml:default_information_originate)>0">
					<xsl:text>     static {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:RipConf/netml:static_route)>0">
						<!-- Rip Static Routes are being configured here -->
						<xsl:apply-templates select="key('kRouter',string($routerID))/netml:RipConf/netml:static_route"/>
					</xsl:if>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:RipConf/netml:default_information_originate)>0">
						<!-- The default route is being configured here -->
						<xsl:apply-templates select="key('kRouter',string($routerID))/netml:RipConf/netml:default_information_originate"/>
					</xsl:if>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:Default-Originate)>0">
						<xsl:text>         route default {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                next-hop self; </xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                retain;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>         }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:if>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes)>0">
						<xsl:for-each select="key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes/netml:s">
							<xsl:text>         route </xsl:text><xsl:value-of select="@address"/>
							<xsl:text> {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							<xsl:text>                next-hop self;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							<xsl:text>                retain;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							<xsl:text>         }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						</xsl:for-each>
					</xsl:if>
					<xsl:text>     }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:AggregateRoutes)>0">
					<xsl:text>     aggregate {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					<xsl:for-each select="key('kRouter',string($routerID))/netml:BGPConf/netml:AggregateRoutes/netml:route">
						<xsl:text>         route </xsl:text><xsl:value-of select="@address"/>
						<xsl:text> {</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>                next-hop self;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:if test="count(netml:advertise)>0 or count(netml:attribute)>0">
							<xsl:text>             policy [ </xsl:text>
							<xsl:if test="count(netml:advertise)>0">
								<xsl:value-of select="netml:advertise"/><xsl:text> </xsl:text>
							</xsl:if>
							<xsl:if test="count(netml:attribute)>0">
								<xsl:value-of select="netml:attribute"/>
							</xsl:if>
							<xsl:text> ];</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						</xsl:if>
						<xsl:choose>
							<xsl:when test="@as-set='true'">
								<xsl:text>             full;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							</xsl:when>
							<xsl:otherwise>
								<xsl:text>             brief;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
							</xsl:otherwise>
						</xsl:choose>
						<xsl:text>             </xsl:text><xsl:value-of select="@type"/>
						<xsl:text>;</xsl:text><xsl:text>&#13;&#10;</xsl:text>
						<xsl:text>         }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
					</xsl:for-each>
					<xsl:text>    }</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
				<xsl:text>}</xsl:text><xsl:text>&#13;&#10;</xsl:text>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf)>0 or count(key('kRouter',string($routerID))/netml:RipConf)>0">
					<xsl:text>protocols {</xsl:text>
					<xsl:text>&#13;&#10;</xsl:text>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:RipConf)>0">
						<!-- Rip Configuration starts here -->
						<xsl:apply-templates select="key('kRouter',string($routerID))/netml:RipConf"/>
						<!-- Rip Configuration finishes here -->
					</xsl:if>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf)>0">
						<xsl:text>    bgp {</xsl:text>
						<xsl:text>&#13;&#10;</xsl:text>
						<xsl:apply-templates select="key('kRouter',string($routerID))/netml:BGPConf"/>
						<xsl:text>    }</xsl:text>
						<xsl:text>&#13;&#10;</xsl:text>
					</xsl:if>
					<xsl:text>}</xsl:text>
					<xsl:text>&#13;&#10;</xsl:text>
				</xsl:if>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="netml:Interface">
	<!-- Interface Configuration -->
	<!-- This is a simplified version of the interface configuration commands -->
   <xsl:value-of select="@name"/>  { <!-- identifier -->
		description <xsl:value-of select="netml:description"/>;
		unit  {
			family inet {
				address <xsl:value-of select="netml:ipAddress"/>;
			}
		}
	}
	</xsl:template><!-- /Interface -->

	</xsl:stylesheet>
