<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                           version="1.0"
                           xmlns:netml="http://www.xmlnetwork.org"
                           exclude-result-prefixes="netml">

	<xsl:output method="xml" indent="yes"/>

	<!-- As the definitions of the templates for Policy and for RipConf already 
exist we can include their files and use those definitions -->
	<xsl:include href="policy-juniper.xsl"/>
	<xsl:include href="ripconf-juniper.xsl"/>
	<xsl:include href="bgp-juniper.xsl"/>

	<!-- ###RouterID### --><xsl:variable name="routerID">r_100_1</xsl:variable><!-- ###RouterID### -->
	<xsl:key name="kRouter" match="netml:NetML/netml:Routers/netml:RouterConf" 
use="@id"/>

	<xsl:template match="/">
	<!-- Configuring the router -->
	<configuration>
		<xsl:if 
test="count(key('kRouter',string($routerID))/netml:List/netml:PrefixList)>0 or 
count(key('kRouter',string($routerID))/netml:List/netml:ASPathList)>0 or 
count(key('kRouter',string($routerID))/netml:List/netml:CommunityList)>0 or 
count(key('kRouter',string($routerID))/netml:Policy)>0">
		<policy-options>
			<xsl:if 
test="count(key('kRouter',string($routerID))/netml:List/netml:PrefixList)>0">
				<!-- Configuring Prefix Lists -->
				<xsl:apply-templates 
select="key('kRouter',string($routerID))//netml:PrefixList"/>
			</xsl:if>
			<xsl:if 
test="count(key('kRouter',string($routerID))/netml:List/netml:ASPathList)>0">
				<!-- Configuring AS Path Lists -->
				<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:List/netml:ASPathList"/>
			</xsl:if>
			<xsl:if 
test="count(key('kRouter',string($routerID))/netml:List/netml:CommunityList)>0">
				<!-- Configuring Community Lists -->
				<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:List/netml:CommunityList"/>
			</xsl:if>
			<xsl:if test="count(key('kRouter',string($routerID))/netml:Policy)>0">
				<!-- Configuring Policies -->
				<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:Policy"/>
			</xsl:if>
		</policy-options>
		</xsl:if>
		<xsl:if test="count(key('kRouter',string($routerID))/netml:Interface)>0">
		<!-- Configuring Interfaces -->
		<interfaces>
			<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:Interface"/>
		</interfaces>
		</xsl:if>
		<xsl:choose>
			<xsl:when 
test="count(key('kRouter',string($routerID))//netml:BGPConf/netml:view)>1">
<xsl:text>    </xsl:text><routing-instances>
	 			<xsl:for-each 
select="key('kRouter',string($routerID))/netml:BGPConf/netml:view">
<xsl:text>        </xsl:text><instance>
<xsl:text>            </xsl:text><name><xsl:value-of select="."/></name>
<xsl:text>            </xsl:text><instance-type>vrf</instance-type>
<xsl:text>            </xsl:text><routing-options>
					<xsl:variable name="localAS" 
select="key('kRouter',string($routerID))/netml:BGPConf/@as"/>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf)>0">
<xsl:text>                </xsl:text><autonomous-system>
<xsl:text>                    </xsl:text><as-number><xsl:value-of 
select="$localAS"/></as-number>
<xsl:text>					 </xsl:text></autonomous-system>
					</xsl:if>
					<xsl:if 
test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:RouterId)>0">
<xsl:text>                </xsl:text><router-id><xsl:value-of 
select="key('kRouter',string($routerID))/netml:BGPConf/netml:RouterId"/></router-id>
					</xsl:if>
					<xsl:if test="count(key('kConfederation',string($localAS)))>0">
						<xsl:for-each select="key('kConfederation',string($localAS))">
<xsl:text>                </xsl:text><confederation>
<xsl:text>                    </xsl:text><confederation-as><xsl:value-of 
select="@id"/></confederation-as>
							<xsl:for-each select="netml:ASMember">
<xsl:text>                    </xsl:text><members>
<xsl:text>                        </xsl:text><name><xsl:value-of 
select="@id"/></name>
<xsl:text>						  </xsl:text></members>
							</xsl:for-each>
<xsl:text>                </xsl:text></confederation>
						</xsl:for-each>
					</xsl:if>
					<xsl:if 
test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:Default-Originate)>0 or count(key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes)>0">
<xsl:text>                </xsl:text><static>
						<xsl:if 
test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:Default-Originate)>0">
<xsl:text>                    </xsl:text><route>
<xsl:text>                        </xsl:text><name>default</name>
<xsl:text>                        </xsl:text><next-hop>
<xsl:text>                            </xsl:text><name>self</name>
<xsl:text>                        </xsl:text></next-hop>
<xsl:text>                        </xsl:text><discard/>
 <xsl:text>                       </xsl:text><retain/>
 <xsl:text>                   </xsl:text></route>
						</xsl:if>
						<xsl:if 
test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes)>0">
							<xsl:for-each 
select="key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes/netml:s">
<xsl:text>                    </xsl:text><route>
<xsl:text>                        </xsl:text><name><xsl:value-of 
select="@address"/></name>
<xsl:text>                        </xsl:text><next-hop>
<xsl:text>                            </xsl:text><name>self</name>
<xsl:text>                        </xsl:text></next-hop>
<xsl:text>                        </xsl:text><discard/>
<xsl:text>                        </xsl:text><retain/>
<xsl:text>                    </xsl:text></route>
							</xsl:for-each>
						</xsl:if>
<xsl:text>                </xsl:text></static>
					</xsl:if>
					<xsl:if 
test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:AggregateRoutes)>0">
<xsl:text>                </xsl:text><aggregate>
						<xsl:for-each 
select="key('kRouter',string($routerID))/netml:BGPConf/netml:AggregateRoutes/netml:route">
<xsl:text>                    </xsl:text><route>
<xsl:text>                        </xsl:text><name><xsl:value-of 
select="@address"/></name>
							<xsl:if test="count(netml:advertise)>0">
<xsl:text>                        </xsl:text><policy>
<xsl:text>                            </xsl:text><name><xsl:value-of 
select="netml:advertise"/></name>
<xsl:text>								</xsl:text></policy>
							</xsl:if>
							<xsl:if test="count(netml:attribute)>0">
<xsl:text>                        </xsl:text><policy>
<xsl:text>                            </xsl:text><name><xsl:value-of 
select="netml:attribute"/></name>
<xsl:text>								</xsl:text></policy>
							</xsl:if>
							<xsl:choose>
								<xsl:when test="@as-set='true'">
<xsl:text>                        </xsl:text><full/>
								</xsl:when>
								<xsl:otherwise>
<xsl:text>                        </xsl:text><brief/>
								</xsl:otherwise>
							</xsl:choose>
							<xsl:choose>
								<xsl:when test="@type='active'">
<xsl:text>                        </xsl:text><active/>
								</xsl:when>
								<xsl:otherwise>
<xsl:text>                        </xsl:text><passive/>
								</xsl:otherwise>
							</xsl:choose>
<xsl:text>                    </xsl:text></route>
						</xsl:for-each>
<xsl:text>					.</xsl:text></aggregate>
					</xsl:if>
<xsl:text>            </xsl:text></routing-options>
					<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf)>0">
<xsl:text>            </xsl:text><protocols>
	 					<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf)>0">
<xsl:text>                </xsl:text><bgp>
							<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:BGPConf"/>
					 </bgp>
					 	</xsl:if>
            </protocols>
				</xsl:if>
        </instance>
		  		</xsl:for-each>
    </routing-instances>
			</xsl:when>
			<xsl:when 
test="count(key('kRouter',string($routerID))//netml:BGPConf/netml:view)=1 or 
count(key('kRouter',string($routerID))/netml:RipConf)>0">
					<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf)>0 or 
count(key('kRouter',string($routerID))/netml:RipConf/netml:static_route)>0 or 
count(key('kRouter',string($routerID))/netml:RipConf/netml:default_information_originate)>0">
    <routing-options>
				<xsl:variable name="localAS" 
select="key('kRouter',string($routerID))/netml:BGPConf/@as"/>
        <autonomous-system>
            <as-number><xsl:value-of select="$localAS"/></as-number>
		  </autonomous-system>
				<xsl:if 
test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:RouterId)>0">
        <router-id><xsl:value-of 
select="key('kRouter',string($routerID))/netml:BGPConf/netml:RouterId"/></router-id>
				</xsl:if>
				<xsl:if test="count(key('kConfederation',string($localAS)))>0">
					<xsl:for-each select="key('kConfederation',string($localAS))">
        <confederation>
            <confederation-as><xsl:value-of select="@id"/></confederation-as>
						<xsl:for-each select="netml:ASMember">
            <members>
                <name><xsl:value-of select="@id"/></name>
				</members>
						</xsl:for-each>
		  </confederation>
					</xsl:for-each>
				</xsl:if>
				<xsl:if 
test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:Default-Originate)>0 or count(key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes)>0 or count(key('kRouter',string($routerID))/netml:RipConf/netml:static_route)>0 or count(key('kRouter',string($routerID))/netml:RipConf/netml:default_information_originate)>0">
        <static>
		 	<!-- Rip Static Routes are being configured here -->
			<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:RipConf/netml:static_route"/>
		 	<!-- The default route is being configured here -->
			<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:RipConf/netml:default_information_originate"/>
					<xsl:if 
test="count(key('kRouter',string($routerID))/netml:BGPConf//netml:Default-Originate)>0">
            <route>
                <name>default</name>
                <next-hop>
                    <name>self</name>
					 </next-hop>
                <discard/>
                <retain/>
				</route>
					</xsl:if>
					<xsl:if 
test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes)>0">
						<xsl:for-each 
select="key('kRouter',string($routerID))/netml:BGPConf/netml:StaticRoutes/netml:s">
            <route>
                <name><xsl:value-of select="@address"/></name>
                <next-hop>
                    <name>self</name>
					 </next-hop>
                <discard/>
                <retain/>
				</route>
						</xsl:for-each>
					</xsl:if>
		  </static>
				</xsl:if>
				<xsl:if 
test="count(key('kRouter',string($routerID))/netml:BGPConf/netml:AggregateRoutes)>0">
        <aggregate>
					<xsl:for-each 
select="key('kRouter',string($routerID))/netml:BGPConf/netml:AggregateRoutes/netml:route">
            <route>
                <name><xsl:value-of select="@address"/></name>
						<xsl:if test="count(netml:advertise)>0">
                <policy>
                		<name><xsl:value-of select="netml:advertise"/></name>
					 </policy>
						</xsl:if>
						<xsl:if test="count(netml:attribute)>0">
                <policy>
                     <name><xsl:value-of select="netml:attribute"/></name>
					 </policy>
						</xsl:if>
						<xsl:choose>
							<xsl:when test="@as-set='true'">
                <full/>
							</xsl:when>
							<xsl:otherwise>
                <brief/>
							</xsl:otherwise>
						</xsl:choose>
						<xsl:choose>
							<xsl:when test="@type='active'">
                <active/>
							</xsl:when>
							<xsl:otherwise>
                <passive/>
							</xsl:otherwise>
						</xsl:choose>
            </route>
					</xsl:for-each>
		  </aggregate>
				</xsl:if>
    </routing-options>
			</xsl:if>
				<xsl:if test="count(key('kRouter',string($routerID))/netml:RipConf)>0 or 
count(key('kRouter',string($routerID))/netml:BGPConf)>0">
    <protocols>
	 				<xsl:if test="count(key('kRouter',string($routerID))/netml:RipConf)>0">
		 				<!-- Rip Configuration starts here -->
						<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:RipConf"/>
						<!-- Rip Configuration finishes here -->
					</xsl:if>
	 				<xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf)>0">
        <bgp>
						<xsl:apply-templates 
select="key('kRouter',string($routerID))/netml:BGPConf"/>
		  </bgp>
					</xsl:if>
    </protocols>
	 			</xsl:if>
			</xsl:when>
		</xsl:choose>
	</configuration>
	</xsl:template>

	<xsl:template match="netml:Interface">
	<!-- Interface Configuration -->
	<!-- This is a simplified version of the interface configuration commands -->
		        <interface>
								<name><xsl:value-of select="@name"/></name> <!-- identifier -->
      		      <description><xsl:value-of 
select="netml:description"/></description>
		            <unit>
									 <family>
                   	 <inet>
                        <address>
													<name><xsl:value-of select="netml:ipAddress"/></name> <!-- 
identifier -->
												</address>
                    </inet>
                </family>
								</unit>
							</interface>
	</xsl:template><!-- /Interface -->

	</xsl:stylesheet>
