<?xml version='1.0'?>
<xsl:stylesheet  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                           version="1.0"
                           xmlns:netml="http://www.xmlnetwork.org">

   <xsl:output method="text" />
   <xsl:strip-space elements="*"/>

   <!-- ###RouterID### --><xsl:variable name="routerID">r_100_1</xsl:variable><!-- ###RouterID### -->
   <xsl:key name="kRouter" match="netml:NetML/netml:Routers/netml:RouterConf"use="@id"/>
   <xsl:key name="kIfaceAddr" match="netml:NetML/netml:Routers/netml:RouterConf[@id=string($routerID)]/netml:Interface/netml:ipAddress" use="../@name"/>

   <!-- As the definitions of the templates for Policy and for RipConf already
exist we can include their files and use those definitions -->
   <xsl:include href="policy-zebra.xsl"/>
   <xsl:include href="ripconf-zebra.xsl"/>
   <xsl:include href="list-zebra.xsl"/>
   <xsl:include href="bgp-zebra.xsl"/>

   <xsl:template match="/">
      <xsl:if test="count(key('kRouter',string($routerID))/netml:Policy)>0">
         <xsl:text>!Start of Policy Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:text>!&#13;&#10;</xsl:text>
         <xsl:apply-templates select="key('kRouter',string($routerID))/netml:Policy/netml:Redistribution//netml:term | key('kRouter',string($routerID))/netml:Policy/netml:General//netml:term"/>
         <xsl:text>!End of Policy Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:text>!&#13;&#10;</xsl:text>
      </xsl:if>
      <xsl:if test="count(key('kRouter',string($routerID))/netml:List)>0 or
         count(key('kRouter',string($routerID))//netml:metric_offset)>0 or
         count(key('kRouter',string($routerID))//netml:key_chain)>0">
         <xsl:text>!Start of Access or Prefix List Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:text>!&#13;&#10;</xsl:text>
         <xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:PrefixList)>0">
            <xsl:apply-templates select="key('kRouter',string($routerID))/netml:List/PrefixList"/>
            <xsl:text>!&#13;&#10;</xsl:text>
         </xsl:if>
         <xsl:if test="count(key('kRouter',string($routerID))//netml:metric_offset)>0">
            <xsl:text>!Creating the access lists which concern the metric offset value from RipConf or from the neighbour configurations</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
            <xsl:apply-templates select="key('kRouter',string($routerID))//netml:metric_offset"/>
            <xsl:text>!End of access lists concerning RIPCONF</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
         </xsl:if>
         <xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:RouteList)>0">
            <xsl:text>!Start of Prefix List Configuration</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
            <xsl:apply-templates select="key('kRouter',string($routerID))/netml:List/netml:RouteList"/>
            <xsl:text>!End of Prefix List Configuration</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
         </xsl:if>
         <xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:ComunityList)>0">
            <xsl:text>!Start of Community List Configuration</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
            <xsl:apply-templates select="key('kRouter',string($routerID))/netml:List/netml:CommunityList"/>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!End of Community List Configuration</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
         </xsl:if>
         <xsl:if test="count(key('kRouter',string($routerID))/netml:List/netml:ASPathList)>0">
            <xsl:text>!Start of AS Path List Configuration</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
            <xsl:apply-templates select="key('kRouter',string($routerID))/netml:List/netml:ASPathList"/>
            <xsl:text>!&#13;&#10;</xsl:text>
            <xsl:text>!End of AS Path List Configuration</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
         </xsl:if>
         <xsl:if test="count(key('kRouter',string($routerID))//netml:key_chain)>0">
            <xsl:text>!Creating any key chains needed for authenticating</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
            <xsl:apply-templates select="key('kRouter',string($routerID))//netml:key_chain"/>
            <xsl:text>!End of key chain creation</xsl:text>
            <xsl:text>&#13;&#10;</xsl:text>
            <xsl:text>!&#13;&#10;</xsl:text>
         </xsl:if>
         <xsl:text>!End of Access or Prefix List Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:text>!&#13;&#10;</xsl:text>
      </xsl:if>
      <xsl:if test="count(key('kRouter',string($routerID))/netml:Interface)>0">
         <xsl:text>!Start of Interface Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:text>!&#13;&#10;</xsl:text>
         <xsl:apply-templates select="key('kRouter',string($routerID))/netml:Interface"/>
         <xsl:text>!End of Interface Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:text>!&#13;&#10;</xsl:text>
      </xsl:if>
      <xsl:if test="count(key('kRouter',string($routerID))/netml:RipConf)>0">
         <xsl:text>!Start of Rip Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:text>!&#13;&#10;</xsl:text>
         <xsl:apply-templates select="key('kRouter',string($routerID))/netml:RipConf"/>
         <xsl:text>!End of Rip Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:text>!&#13;&#10;</xsl:text>
      </xsl:if>
      <xsl:if test="count(key('kRouter',string($routerID))/netml:BGPConf)>0">
         <xsl:text>!Start of BGP Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:text>!&#13;&#10;</xsl:text>
         <xsl:apply-templates select="key('kRouter',string($routerID))/netml:BGPConf"/>
         <xsl:text>!End of BGP Configuration</xsl:text>
         <xsl:text>&#13;&#10;</xsl:text>
         <xsl:text>!&#13;&#10;</xsl:text>
      </xsl:if>
   </xsl:template>

   <xsl:template match="netml:Interface">
      <xsl:text>interface </xsl:text><xsl:value-of select="@name"/>
      <xsl:text>&#13;&#10;</xsl:text>
      <xsl:if test="count(netml:description)>0">
         <xsl:text>  description </xsl:text><xsl:value-of select="netml:description"/>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:if>
      <xsl:if test="count(netml:ipAddress)>0">
      <xsl:for-each select="netml:ipAddress">
         <xsl:text>  ip address </xsl:text><xsl:value-of select="."/>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:for-each>
      </xsl:if>
      <xsl:if test="count(../netml:RipConf)>0">
         <xsl:apply-templates select="//netml:send_version|//netml:receive_version|//netml:split_horizon|//netml:authentication">
            <xsl:with-param name="iface" select="netml:name"/>
         </xsl:apply-templates>
      </xsl:if>
      <xsl:text>!&#13;&#10;</xsl:text>
   </xsl:template>

   <xsl:template match="netml:send_version">
   <xsl:param name="iface"/>
   <xsl:variable name="ancest" select="name(parent::*)"/>
   <xsl:variable name="send" select="."/>
      <xsl:choose>
         <xsl:when test="$ancest='RipConf'">
            <xsl:if test="$send!=../netml:receive_version">
               <xsl:for-each select="../netml:neighbour/netml:interface">
                  <xsl:if test="$iface=.">
                     <xsl:text>  ip rip send version </xsl:text>
                     <xsl:choose>
                        <xsl:when test=".='1-2'">1-2 </xsl:when>
                        <xsl:otherwise><xsl:value-of select="$send"/></xsl:otherwise>
                     </xsl:choose>
                     <xsl:text>&#13;&#10;</xsl:text>
                  </xsl:if>
               </xsl:for-each>
               <xsl:for-each select="../netml:neighbour/netml:address">
                  <xsl:variable name="addr" select="."/>
                  <xsl:for-each select="key('kIfaceAddr',string($iface))">
                     <xsl:if test=".=$addr">
                  <!--xsl:for-each select="/netml:addresses/netml:interface/netml:address">
                     <xsl:if test="../@name=$iface and .=$addr"-->
                        <xsl:text>  ip rip send version </xsl:text>
                        <xsl:choose>
                           <xsl:when test=".='1-2'">1-2 </xsl:when>
                           <xsl:otherwise><xsl:value-of select="$send"/></xsl:otherwise>
                        </xsl:choose>
                        <xsl:text>&#13;&#10;</xsl:text>
                     </xsl:if>
                  </xsl:for-each>
               </xsl:for-each>
            </xsl:if>
         </xsl:when>
         <xsl:when test="$ancest='neighbour'">
            <xsl:if test="count(../netml:interface)>0">
               <xsl:if test="$iface=../netml:interface">
                  <xsl:text>  ip rip send version </xsl:text>
                  <xsl:choose>
                     <xsl:when test=".='1-2'">1-2 </xsl:when>
                     <xsl:otherwise><xsl:value-of select="."/></xsl:otherwise>
                  </xsl:choose>
                  <xsl:text>&#13;&#10;</xsl:text>
               </xsl:if>
            </xsl:if>
            <xsl:if test="count(../netml:address)>0">
               <xsl:variable name="addr" select="../netml:address"/>
               <!--xsl:for-each select="/netml:addresses/netml:interface/netml:address">
                  <xsl:if test="../@name=$iface and .=$addr"-->
               <xsl:for-each select="key('kIfaceAddr',string($iface))">
                  <xsl:if test=".=$addr">
                     <xsl:text>  ip rip send version </xsl:text>
                     <xsl:choose>
                        <xsl:when test=".='1-2'">1-2 </xsl:when>
                        <xsl:otherwise><xsl:value-of select="$send"/></xsl:otherwise>
                     </xsl:choose>
                     <xsl:text>&#13;&#10;</xsl:text>
                  </xsl:if>
               </xsl:for-each>
            </xsl:if>
         </xsl:when>
      </xsl:choose>
   </xsl:template>

   <xsl:template match="netml:receive_version">
   <xsl:param name="iface"/>
   <xsl:variable name="ancest" select="name(parent::*)"/>
   <xsl:variable name="receive" select="."/>
      <xsl:choose>
         <xsl:when test="$ancest='RipConf'">
            <xsl:if test="$receive!=../netml:send_version">
               <xsl:for-each select="../netml:neighbour/netml:interface">
                  <xsl:if test="$iface=.">
                     <xsl:text>  ip rip receive version </xsl:text>
                     <xsl:choose>
                        <xsl:when test=".='1-2'">1-2 </xsl:when>
                        <xsl:otherwise><xsl:value-of select="$receive"/></xsl:otherwise>
                     </xsl:choose>
                     <xsl:text>&#13;&#10;</xsl:text>
                  </xsl:if>
               </xsl:for-each>
               <xsl:for-each select="../netml:neighbour/netml:address">
                  <xsl:variable name="addr" select="."/>
                  <!--xsl:for-each select="/netml:addresses/netml:interface/netml:address">
                     <xsl:if test="../@name=$iface and .=$addr"-->
                  <xsl:for-each select="key('kIfaceAddr',string($iface))">
                     <xsl:if test=".=$addr">
                        <xsl:text>  ip rip receive version </xsl:text>
                        <xsl:choose>
                           <xsl:when test=".='1-2'">1-2 </xsl:when>
                           <xsl:otherwise><xsl:value-of select="$receive"/></xsl:otherwise>
                        </xsl:choose>
                        <xsl:text>&#13;&#10;</xsl:text>
                     </xsl:if>
                  </xsl:for-each>
               </xsl:for-each>
            </xsl:if>
         </xsl:when>
         <xsl:when test="$ancest='neighbour'">
            <xsl:if test="count(../netml:interface)>0">
               <xsl:if test="$iface=../netml:interface">
                  <xsl:text>  ip rip receive version </xsl:text>
                  <xsl:choose>
                     <xsl:when test=".='1-2'">1-2 </xsl:when>
                     <xsl:otherwise><xsl:value-of select="."/></xsl:otherwise>
                  </xsl:choose>
                  <xsl:text>&#13;&#10;</xsl:text>
               </xsl:if>
            </xsl:if>
            <xsl:if test="count(../netml:address)>0">
               <xsl:variable name="addr" select="../netml:address"/>
               <!--xsl:for-each select="/netml:addresses/netml:interface/netml:address">
                  <xsl:if test="../@name=$iface and .=$addr"-->
               <xsl:for-each select="key('kIfaceAddr',string($iface))">
                  <xsl:if test=".=$addr">
                     <xsl:text>  ip rip receive version </xsl:text>
                     <xsl:choose>
                        <xsl:when test=".='1-2'">1-2 </xsl:when>
                        <xsl:otherwise><xsl:value-of select="$receive"/></xsl:otherwise>
                     </xsl:choose>
                     <xsl:text>&#13;&#10;</xsl:text>
                  </xsl:if>
               </xsl:for-each>
            </xsl:if>
         </xsl:when>
      </xsl:choose>
   </xsl:template>

   <xsl:template match="netml:split_horizon">
      <xsl:param name="iface"/>
      <xsl:variable name="sph" select="."/>
      <xsl:for-each select="../netml:neighbour/netml:interface">
         <xsl:if test="$iface=.">
         <xsl:choose>
            <xsl:when test="$sph='true' or $sph='1'">
               <xsl:text>  ip split-horizon</xsl:text>
               <xsl:text>&#13;&#10;</xsl:text>
            </xsl:when>
            <xsl:otherwise>
               <xsl:text>  no ip split-horizon</xsl:text>
               <xsl:text>&#13;&#10;</xsl:text>
            </xsl:otherwise>
         </xsl:choose>
         </xsl:if>
      </xsl:for-each>
      <xsl:for-each select="../netml:neighbour/netml:address">
         <xsl:variable name="addr" select="."/>
         <!--xsl:for-each select="/netml:addresses/netml:interface/netml:address">
            <xsl:if test="../@name=$iface and .=$addr"-->
         <xsl:for-each select="key('kIfaceAddr',string($iface))">
            <xsl:if test=".=$addr">
               <xsl:choose>
                  <xsl:when test="$sph='true' or $sph='1'">
                     <xsl:text>  ip split-horizon</xsl:text>
                     <xsl:text>&#13;&#10;</xsl:text>
                  </xsl:when>
                  <xsl:otherwise>
                     <xsl:text>  no ip split-horizon</xsl:text>
                     <xsl:text>&#13;&#10;</xsl:text>
                  </xsl:otherwise>
               </xsl:choose>
                     <xsl:text>!&#13;&#10;</xsl:text>
            </xsl:if>
         </xsl:for-each>
      </xsl:for-each>
   </xsl:template>

   <xsl:template match="netml:authentication">
   <xsl:param name="iface"/>
   <xsl:variable name="type" select="netml:type"/>
   <xsl:variable name="key" select="netml:key | netml:key_chain/netml:name"/>
   <xsl:variable name="keyType">
      <xsl:choose>
         <xsl:when test="count(netml:key)>0">key</xsl:when>
         <xsl:when test="count(netml:key_chain)>0">key_chain</xsl:when>
      </xsl:choose>
   </xsl:variable>
   <!-- using the supposition that I have a list of all the interfaces existing on the router, I can activate authentication  -->
   <xsl:for-each select="netml:applyToInterface/netml:interface">
      <xsl:variable name="inface" select="."/>
      <xsl:for-each select="../../../netml:neighbour/netml:interface">
         <xsl:if test="$iface=$inface=.">
            <xsl:choose>
               <xsl:when test="$type='text'">
                  <xsl:text>  ip rip authentication mode text</xsl:text>
                  <xsl:text>&#13;&#10;</xsl:text>
                  <xsl:choose>
                     <xsl:when test="$keyType='key'"><xsl:text>  ip rip authentication string
                        </xsl:text><xsl:value-of select="$key"/><xsl:text>&#13;&#10;</xsl:text>
                     </xsl:when>
                     <xsl:otherwise><xsl:text>  ip rip authentication key-chain </xsl:text>
                        <xsl:value-of select="$key"/><xsl:text>&#13;&#10;</xsl:text>
                     </xsl:otherwise>
                  </xsl:choose>
               </xsl:when>
               <xsl:otherwise>
                  <xsl:text>  ip rip authentication mode md5</xsl:text>
                  <xsl:text>&#13;&#10;</xsl:text>
                  <xsl:choose>
                     <xsl:when test="$keyType='key'"><xsl:text>  ip rip authentication string </xsl:text>
                        <xsl:value-of select="$key"/><xsl:text>&#13;&#10;</xsl:text>
                     </xsl:when>
                     <xsl:otherwise>
                        <xsl:text>  ip rip authentication key-chain </xsl:text>
                        <xsl:value-of select="$key"/><xsl:text>&#13;&#10;</xsl:text>
                     </xsl:otherwise>
                  </xsl:choose>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:if>
      </xsl:for-each>
   </xsl:for-each>
   <xsl:for-each select="netml:applyToInterface/netml:interface">
   <xsl:variable name="inface" select="."/>
   <xsl:for-each select="../../../netml:neighbour/netml:address">
      <xsl:variable name="addr" select="."/>
      <!--xsl:for-each select="/netml:addresses/netml:interface/netml:address">
         <xsl:if test="../@name=$iface=inface and .=$addr"-->
         <xsl:for-each select="key('kIfaceAddr',string($iface))">
            <xsl:if test=".=$addr">
               <xsl:choose>
                  <xsl:when test="$type='text'">
                     <xsl:text>  ip rip authentication mode text</xsl:text>
                     <xsl:text>&#13;&#10;</xsl:text>
                     <xsl:choose>
                        <xsl:when test="$keyType='key'"><xsl:text>  ip rip authentication string </xsl:text>
                           <xsl:value-of select="$key"/><xsl:text>&#13;&#10;</xsl:text>
                        </xsl:when>
                        <xsl:otherwise>
                           <xsl:text>  ip rip authentication key-chain </xsl:text>
                           <xsl:value-of select="$key"/><xsl:text>&#13;&#10;</xsl:text>
                        </xsl:otherwise>
                     </xsl:choose>
                  </xsl:when>
               <xsl:otherwise>
                  <xsl:text>  ip rip authentication mode md5</xsl:text>
                  <xsl:text>&#13;&#10;</xsl:text>
                  <xsl:choose>
                     <xsl:when test="$keyType='key'">
                        <xsl:text>  ip rip authentication string </xsl:text>
                        <xsl:value-of select="$key"/><xsl:text>&#13;&#10;</xsl:text>
                     </xsl:when>
                     <xsl:otherwise>
                        <xsl:text>  ip rip authentication key-chain </xsl:text>
                        <xsl:value-of select="$key"/><xsl:text>&#13;&#10;</xsl:text>
                     </xsl:otherwise>
                  </xsl:choose>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:if>
         </xsl:for-each>
      </xsl:for-each>
   </xsl:for-each>
   </xsl:template>
</xsl:stylesheet>
