<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:netml="http://www.xmlnetwork.org">

	<xsl:output method="text" />
	<xsl:strip-space elements="*"/>

	<!-- <xsl:include href="list-cisco.xsl"/> -->

	<!-- ###RouterID### --><xsl:variable name="routerID">r04</xsl:variable><!-- ###RouterID### -->

	<xsl:template match="/">
		<xsl:for-each select="//netml:RouterConf[@id=$routerID]//netml:Interface">
			<xsl:variable name="if" select="@name"/>
			<xsl:for-each select="netml:ipAddress">
				<xsl:variable name="addr" select="."/>
				<xsl:variable name="ip">
					<xsl:choose>
						<xsl:when test="substring-before($addr,'/') != '' ">
							<xsl:value-of select="substring-before($addr,'/')"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$addr"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>

				<xsl:variable name="maskNum" >
					<xsl:choose>
						<xsl:when test="substring-before($addr,'/') != '' ">
							<xsl:value-of select="substring-after($addr,'/')"/>
						</xsl:when>
					</xsl:choose>
				</xsl:variable>
				
				

				<xsl:variable name="maskByte">
					<xsl:if test="$maskNum != '' ">
						<xsl:call-template name="PrefixL2Netmask"><xsl:with-param name="tmp1" select="$maskNum"/>
						</xsl:call-template>
					</xsl:if>
				</xsl:variable>

				<xsl:text> /sbin/ifconfig </xsl:text>
				<xsl:value-of select="$if"/>
				<xsl:text> </xsl:text>
				<xsl:value-of select="$ip"/>

				<xsl:if test="$maskByte != '' ">
					<xsl:text> netmask </xsl:text>
					<xsl:value-of select="$maskByte"/>
				</xsl:if>

				<xsl:text> up &#13;&#10;</xsl:text>

			</xsl:for-each>
		</xsl:for-each>
      
      <!-- OLD METHOD!!  it's a for-each statement, but it cannot be executed more than once
      <xsl:for-each select="//netml:RouterConf[@id=$routerID]/@default-gw">
         <xsl:text> /sbin/route add default gw </xsl:text>
         <xsl:value-of select="."/>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:for-each>
      -->

      <!-- NEW METHOD for adding routes -->
      <xsl:for-each select="//netml:RouterConf[@id=$routerID]/netml:StaticRoutes/netml:route">
         <xsl:text> /sbin/route add </xsl:text>
         <xsl:if test=" contains( ./netml:destination , '/' )">
            <xsl:text>-net </xsl:text>
         </xsl:if>
         <xsl:value-of select="./netml:destination"/>
         <xsl:choose>
            <xsl:when test=" count ( ./netml:nexthop ) > 0 ">
               <xsl:text> gw </xsl:text>
               <xsl:value-of select="./netml:nexthop"/>
            </xsl:when>
            <xsl:when test=" count ( ./netml:interface ) > 0 ">
               <xsl:text> dev </xsl:text>
               <xsl:value-of select="./netml:interface"/>
            </xsl:when>
         </xsl:choose>
         <xsl:if test=" count ( ./netml:metric ) > 0 ">
            <xsl:text> metric </xsl:text>
               <xsl:value-of select="./netml:metric"/>
         </xsl:if>
         <xsl:text>&#13;&#10;</xsl:text>
      </xsl:for-each>

	</xsl:template>
	
<!-- Template for the transformation of the netmask -->

	<xsl:template name="PrefixL2Netmask">
		<xsl:param name="tmp1"/>
		<xsl:variable name="tmp2" select="$tmp1 - 8"/>
		<xsl:variable name="tmp3" select="$tmp2 - 8"/>
		<xsl:variable name="tmp4" select="$tmp3 - 8"/>

		<xsl:variable name="var1">
			<xsl:choose>
				<xsl:when test="$tmp1>=0">
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp" select="$tmp1"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:message>Error: This is a negative prefix length!</xsl:message>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:variable name="var2">
			<xsl:choose>
				<xsl:when test="$tmp2>=0">
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp" select="$tmp2"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp">0</xsl:with-param>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:variable name="var3">
			<xsl:choose>
				<xsl:when test="$tmp3>=0">
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp" select="$tmp3"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp">0</xsl:with-param>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:variable name="var4">
			<xsl:choose>
				<xsl:when test="$tmp4>=0">
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp" select="$tmp4"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="convertNumber">
						<xsl:with-param name="tmp">0</xsl:with-param>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:value-of select="$var1"/><xsl:text>.</xsl:text>
		<xsl:value-of select="$var2"/><xsl:text>.</xsl:text>
		<xsl:value-of select="$var3"/><xsl:text>.</xsl:text>
		<xsl:value-of select="$var4"/>
	</xsl:template>


	<xsl:template name="convertNumber">
		<xsl:param name="tmp"/>
		<xsl:choose>
			<xsl:when test="$tmp='0'">
				<xsl:text>0</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp='1'">
				<xsl:text>128</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp='2'">
				<xsl:text>192</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp='3'">
				<xsl:text>224</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp='4'">
				<xsl:text>240</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp='5'">
				<xsl:text>248</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp='6'">
				<xsl:text>252</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp='7'">
				<xsl:text>254</xsl:text>
			</xsl:when>
			<xsl:when test="$tmp>='8'">
				<xsl:text>255</xsl:text>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

</xsl:stylesheet>
